@@ -20,11 +20,11 @@ package org.apache.spark.executor
 import java.io.{File, NotSerializableException}
 import java.lang.Thread.UncaughtExceptionHandler
 import java.lang.management.ManagementFactory
-import java.net.{URI, URL}
+import java.net.{URI, URL, URLClassLoader}
 import java.nio.ByteBuffer
 import java.util.{Locale, Properties}
 import java.util.concurrent._
-import java.util.concurrent.atomic.AtomicBoolean
+import java.util.concurrent.atomic.{AtomicBoolean, AtomicInteger}
 import java.util.concurrent.locks.ReentrantLock
 import javax.annotation.concurrent.GuardedBy
 
@@ -36,11 +36,12 @@ import scala.util.control.NonFatal
 
 import com.google.common.cache.{Cache, CacheBuilder, RemovalListener, RemovalNotification}
 import com.google.common.util.concurrent.ThreadFactoryBuilder
-import org.slf4j.MDC
+import org.slf4j.{MDC => SLF4JMDC}
 
 import org.apache.spark._
 import org.apache.spark.deploy.SparkHadoopUtil
-import org.apache.spark.internal.{Logging, LogKeys, MDC => LogMDC}
+import org.apache.spark.executor.Executor.{IDLE_TASK_THREAD_NAME, TASK_THREAD_NAME_PREFIX}
+import org.apache.spark.internal.{Logging, LogKeys}
 import org.apache.spark.internal.LogKeys._
 import org.apache.spark.internal.config._
 import org.apache.spark.internal.config.{EXECUTOR_USER_CLASS_PATH_FIRST => EXECUTOR_USER_CLASS_PATH_FIRST_CONFIG}
@@ -58,13 +59,63 @@ import org.apache.spark.util._
 import org.apache.spark.util.ArrayImplicits._
 
 private[spark] class IsolatedSessionState(
-  val sessionUUID: String,
-  var urlClassLoader: MutableURLClassLoader,
-  var replClassLoader: ClassLoader,
-  val currentFiles: HashMap[String, Long],
-  val currentJars: HashMap[String, Long],
-  val currentArchives: HashMap[String, Long],
-  val replClassDirUri: Option[String])
+    val sessionUUID: String,
+    var urlClassLoader: MutableURLClassLoader,
+    var replClassLoader: ClassLoader,
+    val currentFiles: HashMap[String, Long],
+    val currentJars: HashMap[String, Long],
+    val currentArchives: HashMap[String, Long],
+    val replClassDirUri: Option[String]) extends Logging {
+
+  // Reference count for the number of running tasks using this session.
+  private val refCount: AtomicInteger = new AtomicInteger(0)
+
+  // Whether this session has been evicted from the cache.
+  @volatile private var evicted: Boolean = false
+
+  /** Increment the reference count, indicating a task is using this session. */
+  def acquire(): Unit = refCount.incrementAndGet()
+
+  /** Decrement the reference count. If evicted and no more tasks, clean up. */
+  def release(): Unit = {
+    if (refCount.decrementAndGet() == 0 && evicted) {
+      cleanup()
+    }
+  }
+
+  /** Mark this session as evicted. If no tasks are using it, clean up immediately. */
+  def markEvicted(): Unit = {
+    evicted = true
+    if (refCount.get() == 0) {
+      cleanup()
+    } else {
+      logInfo(log"Session ${MDC(SESSION_ID, sessionUUID)} evicted but still in use by " +
+        log"${MDC(LogKeys.COUNT, refCount.get())} task(s), deferring cleanup")
+    }
+  }
+
+  private def cleanup(): Unit = {
+    // Close the urlClassLoader to release resources.
+    try {
+      urlClassLoader match {
+        case cl: URLClassLoader =>
+          cl.close()
+          logInfo(log"Closed urlClassLoader for session ${MDC(SESSION_ID, sessionUUID)}")
+        case _ =>
+      }
+    } catch {
+      case NonFatal(e) =>
+        logWarning(log"Failed to close urlClassLoader for session " +
+          log"${MDC(SESSION_ID, sessionUUID)}", e)
+    }
+    // Delete session files.
+    val sessionBasedRoot = new File(SparkFiles.getRootDirectory(), sessionUUID)
+    if (sessionBasedRoot.isDirectory && sessionBasedRoot.exists()) {
+      Utils.deleteRecursively(sessionBasedRoot)
+    }
+    logInfo(log"Session cleaned up: ${MDC(SESSION_ID, sessionUUID)}")
+  }
+}
 
 /**
  * Spark executor, backed by a threadpool to run tasks.
@@ -82,12 +133,12 @@ private[spark] class Executor(
     resources: immutable.Map[String, ResourceInformation])
   extends Logging {
 
-  logInfo(log"Starting executor ID ${LogMDC(LogKeys.EXECUTOR_ID, executorId)}" +
-    log" on host ${LogMDC(HOST, executorHostname)}")
-  logInfo(log"OS info ${LogMDC(OS_NAME, System.getProperty("os.name"))}," +
-    log" ${LogMDC(OS_VERSION, System.getProperty("os.version"))}, " +
-    log"${LogMDC(OS_ARCH, System.getProperty("os.arch"))}")
-  logInfo(log"Java version ${LogMDC(JAVA_VERSION, System.getProperty("java.version"))}")
+  logInfo(log"Starting executor ID ${MDC(LogKeys.EXECUTOR_ID, executorId)}" +
+    log" on host ${MDC(HOST, executorHostname)}")
+  logInfo(log"OS info ${MDC(OS_NAME, Utils.osName)}," +
+    log" ${MDC(OS_VERSION, Utils.osVersion)}, " +
+    log"${MDC(OS_ARCH, Utils.osArch)}")
+  logInfo(log"Java version ${MDC(JAVA_VERSION, Utils.javaVersion)}")
 
   private val executorShutdown = new AtomicBoolean(false)
   val stopHookReference = ShutdownHookManager.addShutdownHook(
@@ -102,7 +153,7 @@ private[spark] class Executor(
   private[executor] val taskNameMDCKey = if (conf.get(LEGACY_TASK_NAME_MDC_ENABLED)) {
     "mdc.taskName"
   } else {
-    LogKeys.TASK_NAME.name
+    TASK_NAME.name.toLowerCase(Locale.ROOT)
   }
 
   // SPARK-40235: updateDependencies() uses a ReentrantLock instead of the `synchronized` keyword
@@ -132,7 +183,7 @@ private[spark] class Executor(
   private[executor] val threadPool = {
     val threadFactory = new ThreadFactoryBuilder()
       .setDaemon(true)
-      .setNameFormat("Executor task launch worker-%d")
+      .setNameFormat(s"$TASK_THREAD_NAME_PREFIX-%d")
       .setThreadFactory((r: Runnable) => new UninterruptibleThread(r, "unused"))
       .build()
     Executors.newCachedThreadPool(threadFactory).asInstanceOf[ThreadPoolExecutor]
@@ -211,7 +262,7 @@ private[spark] class Executor(
   val defaultSessionState: IsolatedSessionState = newSessionState(JobArtifactState("default", None))
 
   val isolatedSessionCache: Cache[String, IsolatedSessionState] = CacheBuilder.newBuilder()
-    .maximumSize(100)
+    .maximumSize(conf.get(EXECUTOR_ISOLATED_SESSION_CACHE_SIZE))
     .expireAfterAccess(30, TimeUnit.MINUTES)
     .removalListener(new RemovalListener[String, IsolatedSessionState]() {
       override def onRemoval(
@@ -219,11 +270,9 @@ private[spark] class Executor(
         val state = notification.getValue
         // Cache is always used for isolated sessions.
         assert(!isDefaultState(state.sessionUUID))
-        val sessionBasedRoot = new File(SparkFiles.getRootDirectory(), state.sessionUUID)
-        if (sessionBasedRoot.isDirectory && sessionBasedRoot.exists()) {
-          Utils.deleteRecursively(sessionBasedRoot)
-        }
-        logInfo(log"Session evicted: ${LogMDC(SESSION_ID, state.sessionUUID)}")
+        // Mark evicted - cleanup will happen immediately if no tasks are using it,
+        // or when the last task releases it.
+        state.markEvicted()
       }
     })
     .build[String, IsolatedSessionState]
@@ -380,7 +429,24 @@ private[spark] class Executor(
       tr.kill(killMark._1, killMark._2)
       killMarks.remove(taskId)
     }
-    threadPool.execute(tr)
+    try {
+      threadPool.execute(tr)
+    } catch {
+      case t: Throwable =>
+        try {
+          logError(log"Executor launch task ${MDC(TASK_NAME, taskDescription.name)} failed," +
+            log" reason: ${MDC(REASON, t.getMessage)}")
+          context.statusUpdate(
+            taskDescription.taskId,
+            TaskState.FAILED,
+            env.closureSerializer.newInstance().serialize(new ExceptionFailure(t, Seq.empty)))
+        } catch {
+          case t: Throwable =>
+            logError(log"Executor update launching task ${MDC(TASK_NAME, taskDescription.name)} " +
+              log"failed status failed, reason: ${MDC(REASON, t.getMessage)}")
+            System.exit(-1)
+        }
+    }
     if (decommissioned) {
       log.error(s"Launching a task while in decommissioned state.")
     }
@@ -478,7 +544,7 @@ private[spark] class Executor(
 
     val taskId = taskDescription.taskId
     val taskName = taskDescription.name
-    val threadName = s"Executor task launch worker for $taskName"
+    val threadName = s"$TASK_THREAD_NAME_PREFIX for $taskName"
     val mdcProperties = taskDescription.properties.asScala
       .filter(_._1.startsWith("mdc.")).toSeq
 
@@ -505,9 +571,9 @@ private[spark] class Executor(
     @volatile var task: Task[Any] = _
 
     def kill(interruptThread: Boolean, reason: String): Unit = {
-      logInfo(log"Executor is trying to kill ${LogMDC(TASK_NAME, taskName)}, " +
-        log"interruptThread: ${LogMDC(INTERRUPT_THREAD, interruptThread)}, " +
-        log"reason: ${LogMDC(REASON, reason)}")
+      logInfo(log"Executor is trying to kill ${MDC(TASK_NAME, taskName)}, " +
+        log"interruptThread: ${MDC(INTERRUPT_THREAD, interruptThread)}, " +
+        log"reason: ${MDC(REASON, reason)}")
       reasonIfKilled = Some(reason)
       if (task != null) {
         synchronized {
@@ -568,6 +634,9 @@ private[spark] class Executor(
         case _ => defaultSessionState
       }
 
+      // Pin the session to prevent its class loader from being closed while this task is running.
+      isolatedSession.acquire()
+
       setMDCForTask(taskName, mdcProperties)
       threadId = Thread.currentThread.getId
       Thread.currentThread.setName(threadName)
@@ -579,7 +648,7 @@ private[spark] class Executor(
       } else 0L
       Thread.currentThread.setContextClassLoader(isolatedSession.replClassLoader)
       val ser = env.closureSerializer.newInstance()
-      logInfo(log"Running ${LogMDC(TASK_NAME, taskName)}")
+      logInfo(log"Running ${MDC(TASK_NAME, taskName)}")
       execBackend.statusUpdate(taskId, TaskState.RUNNING, EMPTY_BYTE_BUFFER)
       var taskStartTimeNs: Long = 0
       var taskStartCpu: Long = 0
@@ -653,7 +722,7 @@ private[spark] class Executor(
 
           if (freedMemory > 0 && !threwException) {
             val errMsg = log"Managed memory leak detected; size = " +
-              log"${LogMDC(NUM_BYTES, freedMemory)} bytes, ${LogMDC(TASK_NAME, taskName)}"
+              log"${MDC(NUM_BYTES, freedMemory)} bytes, ${MDC(TASK_NAME, taskName)}"
             if (conf.get(UNSAFE_EXCEPTION_ON_MEMORY_LEAK)) {
               throw SparkException.internalError(errMsg.message, category = "EXECUTOR")
             } else {
@@ -663,9 +732,9 @@ private[spark] class Executor(
 
           if (releasedLocks.nonEmpty && !threwException) {
             val errMsg =
-              log"${LogMDC(NUM_RELEASED_LOCKS, releasedLocks.size)} block locks" +
-                log" were not released by ${LogMDC(TASK_NAME, taskName)}\n" +
-                log" ${LogMDC(RELEASED_LOCKS, releasedLocks.mkString("[", ", ", "]"))})"
+              log"${MDC(NUM_RELEASED_LOCKS, releasedLocks.size)} block locks" +
+                log" were not released by ${MDC(TASK_NAME, taskName)}\n" +
+                log" ${MDC(RELEASED_LOCKS, releasedLocks.mkString("[", ", ", "]"))})"
             if (conf.get(STORAGE_EXCEPTION_PIN_LEAK)) {
               throw SparkException.internalError(errMsg.message, category = "EXECUTOR")
             } else {
@@ -677,10 +746,10 @@ private[spark] class Executor(
           // uh-oh.  it appears the user code has caught the fetch-failure without throwing any
           // other exceptions.  Its *possible* this is what the user meant to do (though highly
           // unlikely).  So we will log an error and keep going.
-          logError(log"${LogMDC(TASK_NAME, taskName)} completed successfully though internally " +
+          logError(log"${MDC(TASK_NAME, taskName)} completed successfully though internally " +
             log"it encountered unrecoverable fetch failures! Most likely this means user code " +
             log"is incorrectly swallowing Spark's internal " +
-            log"${LogMDC(CLASS_NAME, classOf[FetchFailedException])}", fetchFailure)
+            log"${MDC(CLASS_NAME, classOf[FetchFailedException])}", fetchFailure)
         }
         val taskFinishNs = System.nanoTime()
         val taskFinishCpu = if (threadMXBean.isCurrentThreadCpuTimeSupported) {
@@ -728,7 +797,6 @@ private[spark] class Executor(
           .inc(task.metrics.outputMetrics.bytesWritten)
         executorSource.METRIC_OUTPUT_RECORDS_WRITTEN
           .inc(task.metrics.outputMetrics.recordsWritten)
-        executorSource.METRIC_RESULT_SIZE.inc(task.metrics.resultSize)
         executorSource.METRIC_DISK_BYTES_SPILLED.inc(task.metrics.diskBytesSpilled)
         executorSource.METRIC_MEMORY_BYTES_SPILLED.inc(task.metrics.memoryBytesSpilled)
         incrementShuffleMetrics(executorSource, task.metrics)
@@ -742,14 +810,15 @@ private[spark] class Executor(
         val serializedDirectResult = SerializerHelper.serializeToChunkedBuffer(ser, directResult,
           valueByteBuffer.size + accumUpdates.size * 32 + metricPeaks.length * 8)
         val resultSize = serializedDirectResult.size
+        executorSource.METRIC_RESULT_SIZE.inc(resultSize)
 
         // directSend = sending directly back to the driver
         val serializedResult: ByteBuffer = {
           if (maxResultSize > 0 && resultSize > maxResultSize) {
-            logWarning(log"Finished ${LogMDC(TASK_NAME, taskName)}. " +
+            logWarning(log"Finished ${MDC(TASK_NAME, taskName)}. " +
               log"Result is larger than maxResultSize " +
-              log"(${LogMDC(RESULT_SIZE_BYTES, Utils.bytesToString(resultSize))} > " +
-              log"${LogMDC(RESULT_SIZE_BYTES_MAX, Utils.bytesToString(maxResultSize))}), " +
+              log"(${MDC(RESULT_SIZE_BYTES, Utils.bytesToString(resultSize))} > " +
+              log"${MDC(RESULT_SIZE_BYTES_MAX, Utils.bytesToString(maxResultSize))}), " +
               log"dropping it.")
             ser.serialize(new IndirectTaskResult[Any](TaskResultBlockId(taskId), resultSize))
           } else if (resultSize > maxDirectResultSize) {
@@ -758,12 +827,12 @@ private[spark] class Executor(
               blockId,
               serializedDirectResult,
               StorageLevel.MEMORY_AND_DISK_SER)
-            logInfo(log"Finished ${LogMDC(TASK_NAME, taskName)}." +
-              log" ${LogMDC(NUM_BYTES, resultSize)} bytes result sent via BlockManager)")
+            logInfo(log"Finished ${MDC(TASK_NAME, taskName)}." +
+              log" ${MDC(NUM_BYTES, resultSize)} bytes result sent via BlockManager)")
             ser.serialize(new IndirectTaskResult[Any](blockId, resultSize))
           } else {
-            logInfo(log"Finished ${LogMDC(TASK_NAME, taskName)}." +
-              log" ${LogMDC(NUM_BYTES, resultSize)} bytes result sent to driver")
+            logInfo(log"Finished ${MDC(TASK_NAME, taskName)}." +
+              log" ${MDC(NUM_BYTES, resultSize)} bytes result sent to driver")
             // toByteBuffer is safe here, guarded by maxDirectResultSize
             serializedDirectResult.toByteBuffer
           }
@@ -775,8 +844,8 @@ private[spark] class Executor(
         execBackend.statusUpdate(taskId, TaskState.FINISHED, serializedResult)
       } catch {
         case t: TaskKilledException =>
-          logInfo(log"Executor killed ${LogMDC(TASK_NAME, taskName)}," +
-            log" reason: ${LogMDC(REASON, t.reason)}")
+          logInfo(log"Executor killed ${MDC(TASK_NAME, taskName)}," +
+            log" reason: ${MDC(REASON, t.reason)}")
 
           val (accums, accUpdates) = collectAccumulatorsAndResetStatusOnFailure(taskStartTimeNs)
           // Here and below, put task metric peaks in an immutable.ArraySeq to expose them as an
@@ -789,8 +858,8 @@ private[spark] class Executor(
         case _: InterruptedException | NonFatal(_) if
             task != null && task.reasonIfKilled.isDefined =>
           val killReason = task.reasonIfKilled.getOrElse("unknown reason")
-          logInfo(log"Executor interrupted and killed ${LogMDC(TASK_NAME, taskName)}," +
-            log" reason: ${LogMDC(REASON, killReason)}")
+          logInfo(log"Executor interrupted and killed ${MDC(TASK_NAME, taskName)}," +
+            log" reason: ${MDC(REASON, killReason)}")
 
           val (accums, accUpdates) = collectAccumulatorsAndResetStatusOnFailure(taskStartTimeNs)
           val metricPeaks = metricsPoller.getTaskMetricPeaks(taskId).toImmutableArraySeq
@@ -803,12 +872,12 @@ private[spark] class Executor(
           if (!t.isInstanceOf[FetchFailedException]) {
             // there was a fetch failure in the task, but some user code wrapped that exception
             // and threw something else.  Regardless, we treat it as a fetch failure.
-            logWarning(log"${LogMDC(TASK_NAME, taskName)} encountered a " +
-              log"${LogMDC(CLASS_NAME, classOf[FetchFailedException].getName)} " +
+            logWarning(log"${MDC(TASK_NAME, taskName)} encountered a " +
+              log"${MDC(CLASS_NAME, classOf[FetchFailedException].getName)} " +
               log"and failed, but the " +
-              log"${LogMDC(CLASS_NAME, classOf[FetchFailedException].getName)} " +
+              log"${MDC(CLASS_NAME, classOf[FetchFailedException].getName)} " +
               log"was hidden by another exception. Spark is handling this like a fetch failure " +
-              log"and ignoring the other exception: ${LogMDC(ERROR, t)}")
+              log"and ignoring the other exception: ${MDC(ERROR, t)}")
           }
           setTaskFinishedAndClearInterruptStatus()
           plugins.foreach(_.onTaskFailed(reason))
@@ -823,13 +892,13 @@ private[spark] class Executor(
         case t: Throwable if env.isStopped =>
           // Log the expected exception after executor.stop without stack traces
           // see: SPARK-19147
-          logError(log"Exception in ${LogMDC(TASK_NAME, taskName)}: ${LogMDC(ERROR, t.getMessage)}")
+          logError(log"Exception in ${MDC(TASK_NAME, taskName)}: ${MDC(ERROR, t.getMessage)}")
 
         case t: Throwable =>
           // Attempt to exit cleanly by informing the driver of our failure.
           // If anything goes wrong (or this was a fatal exception), we will delegate to
           // the default uncaught exception handler, which will terminate the Executor.
-          logError(log"Exception in ${LogMDC(TASK_NAME, taskName)}", t)
+          logError(log"Exception in ${MDC(TASK_NAME, taskName)}", t)
 
           // SPARK-20904: Do not report failure to driver if if happened during shut down. Because
           // libraries may set up shutdown hooks that race with running tasks during shutdown,
@@ -873,6 +942,9 @@ private[spark] class Executor(
           // are known, and metricsPoller.onTaskStart was called.
           metricsPoller.onTaskCompletion(taskId, task.stageId, task.stageAttemptId)
         }
+        // Release the session reference. If evicted and this was the last task, cleanup happens.
+        isolatedSession.release()
+        Thread.currentThread().setName(s"$IDLE_TASK_THREAD_NAME#$threadId" )
       }
     }
 
@@ -934,16 +1006,16 @@ private[spark] class Executor(
 
   private def setMDCForTask(taskName: String, mdc: Seq[(String, String)]): Unit = {
     if (Executor.mdcIsSupported) {
-      mdc.foreach { case (key, value) => MDC.put(key, value) }
+      mdc.foreach { case (key, value) => SLF4JMDC.put(key, value) }
       // avoid overriding the takName by the user
-      MDC.put(taskNameMDCKey, taskName)
+      SLF4JMDC.put(taskNameMDCKey, taskName)
     }
   }
 
   private def cleanMDCForTask(taskName: String, mdc: Seq[(String, String)]): Unit = {
     if (Executor.mdcIsSupported) {
-      mdc.foreach { case (key, _) => MDC.remove(key) }
-      MDC.remove(taskNameMDCKey)
+      mdc.foreach { case (key, _) => SLF4JMDC.remove(key) }
+      SLF4JMDC.remove(taskNameMDCKey)
     }
   }
 
@@ -1016,14 +1088,14 @@ private[spark] class Executor(
             finished = true
           } else {
             val elapsedTimeMs = TimeUnit.NANOSECONDS.toMillis(elapsedTimeNs)
-            logWarning(log"Killed task ${LogMDC(TASK_ID, taskId)} " +
-              log"is still running after ${LogMDC(TIME_UNITS, elapsedTimeMs)} ms")
+            logWarning(log"Killed task ${MDC(TASK_ID, taskId)} " +
+              log"is still running after ${MDC(TIME_UNITS, elapsedTimeMs)} ms")
             if (takeThreadDump) {
               try {
                 taskRunner.theadDump().foreach { thread =>
                   if (thread.threadName == taskRunner.threadName) {
-                    logWarning(log"Thread dump from task ${LogMDC(TASK_ID, taskId)}:\n" +
-                      log"${LogMDC(THREAD, thread.toString)}")
+                    logWarning(log"Thread dump from task ${MDC(TASK_ID, taskId)}:\n" +
+                      log"${MDC(THREAD, thread.toString)}")
                   }
                 }
               } catch {
@@ -1037,8 +1109,8 @@ private[spark] class Executor(
         if (!taskRunner.isFinished && timeoutExceeded()) {
           val killTimeoutMs = TimeUnit.NANOSECONDS.toMillis(killTimeoutNs)
           if (isLocal) {
-            logError(log"Killed task ${LogMDC(TASK_ID, taskId)} could not be stopped within " +
-              log"${LogMDC(TIMEOUT, killTimeoutMs)} ms; " +
+            logError(log"Killed task ${MDC(TASK_ID, taskId)} could not be stopped within " +
+              log"${MDC(TIMEOUT, killTimeoutMs)} ms; " +
               log"not killing JVM because we are running in local mode.")
           } else {
             // In non-local-mode, the exception thrown here will bubble up to the uncaught exception
@@ -1093,8 +1165,8 @@ private[spark] class Executor(
     logInfo(
       log"Starting executor with user classpath" +
         log" (userClassPathFirst =" +
-        log" ${LogMDC(LogKeys.EXECUTOR_USER_CLASS_PATH_FIRST, userClassPathFirst)}): " +
-        log"${LogMDC(URLS, urls.mkString("'", ",", "'"))}"
+        log" ${MDC(LogKeys.EXECUTOR_USER_CLASS_PATH_FIRST, userClassPathFirst)}): " +
+        log"${MDC(URLS, urls.mkString("'", ",", "'"))}"
     )
 
     if (useStub) {
@@ -1150,13 +1222,13 @@ private[spark] class Executor(
       sessionUUID: String): ClassLoader = {
     val classUri = sessionClassUri.getOrElse(conf.get("spark.repl.class.uri", null))
     val classLoader = if (classUri != null) {
-      logInfo(log"Using REPL class URI: ${LogMDC(LogKeys.URI, classUri)}")
+      logInfo(log"Using REPL class URI: ${MDC(LogKeys.URI, classUri)}")
       new ExecutorClassLoader(conf, env, classUri, parent, userClassPathFirst)
     } else {
       parent
     }
-    logInfo(log"Created or updated repl class loader ${LogMDC(CLASS_LOADER, classLoader)}" +
-      log" for ${LogMDC(SESSION_ID, sessionUUID)}.")
+    logInfo(log"Created or updated repl class loader ${MDC(CLASS_LOADER, classLoader)}" +
+      log" for ${MDC(SESSION_ID, sessionUUID)}.")
     classLoader
   }
 
@@ -1191,16 +1263,16 @@ private[spark] class Executor(
 
       // Fetch missing dependencies
       for ((name, timestamp) <- newFiles if state.currentFiles.getOrElse(name, -1L) < timestamp) {
-        logInfo(log"Fetching ${LogMDC(FILE_NAME, name)} with" +
-          log" timestamp ${LogMDC(TIMESTAMP, timestamp)}")
+        logInfo(log"Fetching ${MDC(FILE_NAME, name)} with" +
+          log" timestamp ${MDC(TIMESTAMP, timestamp)}")
         // Fetch file with useCache mode, close cache for local mode.
         Utils.fetchFile(name, root, conf, hadoopConf, timestamp, useCache = !isLocal)
         state.currentFiles(name) = timestamp
       }
       for ((name, timestamp) <- newArchives if
           state.currentArchives.getOrElse(name, -1L) < timestamp) {
-        logInfo(log"Fetching ${LogMDC(ARCHIVE_NAME, name)} with" +
-          log" timestamp ${LogMDC(TIMESTAMP, timestamp)}")
+        logInfo(log"Fetching ${MDC(ARCHIVE_NAME, name)} with" +
+          log" timestamp ${MDC(TIMESTAMP, timestamp)}")
         val sourceURI = new URI(name)
         val uriToDownload = Utils.getUriBuilder(sourceURI).fragment(null).build()
         val source = Utils.fetchFile(uriToDownload.toString, Utils.createTempDir(), conf,
@@ -1209,10 +1281,10 @@ private[spark] class Executor(
           root,
           if (sourceURI.getFragment != null) sourceURI.getFragment else source.getName)
         logInfo(
-          log"Unpacking an archive ${LogMDC(ARCHIVE_NAME, name)}" +
-            log" (${LogMDC(BYTE_SIZE, source.length)} bytes)" +
-            log" from ${LogMDC(SOURCE_PATH, source.getAbsolutePath)}" +
-            log" to ${LogMDC(DESTINATION_PATH, dest.getAbsolutePath)}")
+          log"Unpacking an archive ${MDC(ARCHIVE_NAME, name)}" +
+            log" (${MDC(BYTE_SIZE, source.length)} bytes)" +
+            log" from ${MDC(SOURCE_PATH, source.getAbsolutePath)}" +
+            log" to ${MDC(DESTINATION_PATH, dest.getAbsolutePath)}")
         Utils.deleteRecursively(dest)
         Utils.unpack(source, dest)
         state.currentArchives(name) = timestamp
@@ -1223,8 +1295,8 @@ private[spark] class Executor(
           .orElse(state.currentJars.get(localName))
           .getOrElse(-1L)
         if (currentTimeStamp < timestamp) {
-          logInfo(log"Fetching ${LogMDC(JAR_URL, name)} with" +
-            log" timestamp ${LogMDC(TIMESTAMP, timestamp)}")
+          logInfo(log"Fetching ${MDC(JAR_URL, name)} with" +
+            log" timestamp ${MDC(TIMESTAMP, timestamp)}")
           // Fetch file with useCache mode, close cache for local mode.
           Utils.fetchFile(name, root, conf,
             hadoopConf, timestamp, useCache = !isLocal)
@@ -1232,8 +1304,8 @@ private[spark] class Executor(
           // Add it to our class loader
           val url = new File(root, localName).toURI.toURL
           if (!state.urlClassLoader.getURLs().contains(url)) {
-            logInfo(log"Adding ${LogMDC(LogKeys.URL, url)} to" +
-              log" class loader ${LogMDC(UUID, state.sessionUUID)}")
+            logInfo(log"Adding ${MDC(LogKeys.URL, url)} to" +
+              log" class loader ${MDC(UUID, state.sessionUUID)}")
             state.urlClassLoader.addURL(url)
             if (isStubbingEnabledForState(state.sessionUUID)) {
               renewClassLoader = true
@@ -1298,7 +1370,7 @@ private[spark] class Executor(
         heartbeatFailures += 1
         if (heartbeatFailures >= HEARTBEAT_MAX_FAILURES) {
           logError(log"Exit as unable to send heartbeats to driver " +
-            log"more than ${LogMDC(MAX_ATTEMPTS, HEARTBEAT_MAX_FAILURES)} times")
+            log"more than ${MDC(MAX_ATTEMPTS, HEARTBEAT_MAX_FAILURES)} times")
           System.exit(ExecutorExitCode.HEARTBEAT_FAILURE)
         }
     }
@@ -1309,13 +1381,16 @@ private[spark] class Executor(
     if (runner != null) {
       runner.theadDump()
     } else {
-      logWarning(log"Failed to dump thread for task ${LogMDC(TASK_ID, taskId)}")
+      logWarning(log"Failed to dump thread for task ${MDC(TASK_ID, taskId)}")
       None
     }
   }
 }
 
 private[spark] object Executor extends Logging {
+  val TASK_THREAD_NAME_PREFIX = "Executor task launch worker"
+  val IDLE_TASK_THREAD_NAME = "Executor task idle worker"
+
   // This is reserved for internal use by components that need to read task properties before a
   // task is fully deserialized. When possible, the TaskContext.getLocalProperty call should be
   // used instead.
@@ -1328,8 +1403,8 @@ private[spark] object Executor extends Logging {
     try {
       // This tests if any class initialization error is thrown
       val testKey = System.nanoTime().toString
-      MDC.put(testKey, "testValue")
-      MDC.remove(testKey)
+      SLF4JMDC.put(testKey, "testValue")
+      SLF4JMDC.remove(testKey)
 
       true
     } catch {
