@@ -18,16 +18,14 @@ package org.apache.spark.deploy.history
 
 import java.io.{File, FileInputStream, FileWriter, InputStream, IOException}
 import java.net.{HttpURLConnection, URI, URL}
-import java.nio.charset.StandardCharsets
+import java.nio.file.Files
 import java.util.zip.ZipInputStream
 
 import scala.concurrent.duration._
 import scala.jdk.CollectionConverters._
 
-import com.google.common.io.{ByteStreams, Files}
 import jakarta.servlet._
 import jakarta.servlet.http.{HttpServletRequest, HttpServletRequestWrapper, HttpServletResponse}
-import org.apache.commons.io.{FileUtils, IOUtils}
 import org.apache.hadoop.fs.{FileStatus, FileSystem, Path}
 import org.json4s.JsonAST._
 import org.json4s.jackson.JsonMethods
@@ -219,7 +217,7 @@ abstract class HistoryServerSuite extends SparkFunSuite with BeforeAndAfter with
   )
 
   if (regenerateGoldenFiles) {
-    FileUtils.deleteDirectory(expRoot)
+    Utils.deleteRecursively(expRoot)
     Utils.createDirectory(expRoot)
   }
 
@@ -246,7 +244,7 @@ abstract class HistoryServerSuite extends SparkFunSuite with BeforeAndAfter with
         }
       }
 
-      val exp = IOUtils.toString(new FileInputStream(goldenFile), StandardCharsets.UTF_8)
+      val exp = Utils.toString(new FileInputStream(goldenFile))
       // compare the ASTs so formatting differences don't cause failures
       val expAst = parse(exp)
       assertValidDataInJson(jsonAst, expAst)
@@ -309,9 +307,7 @@ abstract class HistoryServerSuite extends SparkFunSuite with BeforeAndAfter with
         val expectedFile = {
           new File(logDir, entry.getName)
         }
-        val expected = Files.asCharSource(expectedFile, StandardCharsets.UTF_8).read()
-        val actual = new String(ByteStreams.toByteArray(zipStream), StandardCharsets.UTF_8)
-        actual should be (expected)
+        Utils.toString(zipStream) should be (Files.readString(expectedFile.toPath))
         filesCompared += 1
       }
       entry = zipStream.getNextEntry
@@ -394,6 +390,23 @@ abstract class HistoryServerSuite extends SparkFunSuite with BeforeAndAfter with
     assert(response.contains(SPARK_VERSION))
   }
 
+  test("SPARK-54857: XSS prevention in application and user names") {
+    implicit val formats = org.json4s.DefaultFormats
+
+    val appId = "local-1766844910796"
+    val response = getUrl(s"applications/$appId")
+    val app = JsonMethods.parse(response)
+
+    // Verify that malicious content is present in JSON (not escaped at API level)
+    (app \ "name").extract[String] should be ("<script>alert('XSS')</script>")
+    val attempt = (app \ "attempts")(0)
+    (attempt \ "sparkUser").extract[String] should be ("<script>alert('XSS')</script>")
+
+    // Verify that the history page HTML properly escapes the content
+    val historyPage = HistoryServerSuite.getUrl(buildPageAttemptUrl(appId, None))
+    historyPage should not include "<script>alert('XSS')</script>"
+  }
+
   /**
    * Verify that the security manager needed for the history server can be instantiated
    * when `spark.authenticate` is `true`, rather than raise an `IllegalArgumentException`.
@@ -750,7 +763,7 @@ object HistoryServerSuite {
 
   def getContentAndCode(url: URL): (Int, Option[String], Option[String]) = {
     val (code, in, errString) = connectAndGetInputStream(url)
-    val inString = in.map(IOUtils.toString(_, StandardCharsets.UTF_8))
+    val inString = in.map(Utils.toString)
     (code, inString, errString)
   }
 
@@ -766,7 +779,7 @@ object HistoryServerSuite {
     }
     val errString = try {
       val err = Option(connection.getErrorStream())
-      err.map(IOUtils.toString(_, StandardCharsets.UTF_8))
+      err.map(Utils.toString)
     } catch {
       case io: IOException => None
     }
