@@ -20,7 +20,9 @@ from typing import (
     Callable,
     List,
     Optional,
+    Sequence,
     Union,
+    cast,
     no_type_check,
     overload,
     TYPE_CHECKING,
@@ -28,7 +30,6 @@ from typing import (
 from warnings import warn
 
 from pyspark.errors.exceptions.captured import unwrap_spark_exception
-from pyspark.loose_version import LooseVersion
 from pyspark.util import _load_from_socket
 from pyspark.sql.pandas.serializers import ArrowCollectSerializer
 from pyspark.sql.pandas.types import _dedup_names
@@ -37,11 +38,11 @@ from pyspark.sql.types import (
     MapType,
     TimestampType,
     StructType,
+    _has_type,
     DataType,
     _create_row,
     StringType,
 )
-from pyspark.sql.utils import is_timestamp_ntz_preferred
 from pyspark.traceback_utils import SCCallSiteSync
 from pyspark.errors import PySparkTypeError, PySparkValueError
 
@@ -54,13 +55,126 @@ if TYPE_CHECKING:
     from pyspark.sql import DataFrame
 
 
+def _convert_arrow_table_to_pandas(
+    arrow_table: "pa.Table",
+    schema: "StructType",
+    *,
+    timezone: Optional[str] = None,
+    struct_handling_mode: Optional[str] = None,
+    date_as_object: bool = False,
+    self_destruct: bool = False,
+) -> "PandasDataFrameLike":
+    """
+    Helper function to convert Arrow table columns to a pandas DataFrame.
+
+    This function applies Spark-specific type converters to Arrow columns and concatenates
+    them into a pandas DataFrame.
+
+    Parameters
+    ----------
+    arrow_table : pyarrow.Table
+        The Arrow table to convert
+    schema : StructType
+        The schema of the DataFrame
+    timezone : str or None
+        The timezone to use for timestamp conversions (can be None if not configured)
+    struct_handling_mode : str or None
+        How to handle struct types in pandas ("dict", "row", or "legacy", can be None
+         if not configured). If "legacy", it will be converted to "dict" and error checking
+         for duplicated field names will be enabled when StructType fields are present.
+    date_as_object : bool
+        Whether to convert date values to Python datetime.date objects (default: False)
+    self_destruct : bool
+        Whether to enable memory-efficient self-destruct mode for large tables (default: False)
+
+    Returns
+    -------
+    pandas.DataFrame
+        The converted pandas DataFrame
+    """
+    import pandas as pd
+    from pyspark.sql.pandas.types import _create_converter_to_pandas
+
+    # Build pandas options
+    # Pandas DataFrame created from PyArrow uses datetime64[ns] for date type
+    # values, but we should use datetime.date to match the behavior with when
+    # Arrow optimization is disabled.
+    pandas_options = {"coerce_temporal_nanoseconds": True}
+    if date_as_object:
+        pandas_options["date_as_object"] = True
+
+    # Handle empty columns case
+    if len(schema.fields) == 0:
+        return arrow_table.to_pandas(**pandas_options)
+
+    # Rename columns to avoid duplicated column names during processing
+    temp_col_names = [f"col_{i}" for i in range(len(schema.names))]
+    arrow_table = arrow_table.rename_columns(temp_col_names)
+
+    # Configure self-destruct mode for memory efficiency
+    if self_destruct and arrow_table.num_rows > 0:
+        # Configure PyArrow to use as little memory as possible:
+        # self_destruct - free columns as they are converted
+        # split_blocks - create a separate Pandas block for each column
+        # use_threads - convert one column at a time
+        pandas_options.update(
+            {
+                "self_destruct": True,
+                "split_blocks": True,
+                "use_threads": False,
+            }
+        )
+
+    # Handle legacy struct handling mode
+    error_on_duplicated_field_names = False
+    if struct_handling_mode == "legacy" and any(
+        _has_type(f.dataType, StructType) for f in schema.fields
+    ):
+        error_on_duplicated_field_names = True
+        struct_handling_mode = "dict"
+
+    # SPARK-51112: If the table is empty, we avoid using pyarrow to_pandas to create the
+    # DataFrame, as it may fail with a segmentation fault.
+    if arrow_table.num_rows == 0:
+        # For empty tables, create empty Series to preserve dtypes
+        column_data = (
+            pd.Series([], name=temp_col_names[i], dtype="object") for i in range(len(schema.fields))
+        )
+    else:
+        # For non-empty tables, convert arrow columns directly
+        column_data = (arrow_col.to_pandas(**pandas_options) for arrow_col in arrow_table.columns)
+
+    # Apply Spark-specific type converters to each column
+    pdf = pd.concat(
+        objs=cast(
+            Sequence[pd.Series],
+            (
+                _create_converter_to_pandas(
+                    field.dataType,
+                    field.nullable,
+                    timezone=timezone,
+                    struct_in_pandas=struct_handling_mode,
+                    error_on_duplicated_field_names=error_on_duplicated_field_names,
+                )(series)
+                for series, field in zip(column_data, schema.fields)
+            ),
+        ),
+        axis="columns",
+    )
+
+    # Restore original column names (including duplicates)
+    pdf.columns = schema.names
+
+    return pdf
+
+
 class PandasConversionMixin:
     """
-    Mix-in for the conversion from Spark to pandas. Currently, only :class:`DataFrame`
-    can use this class.
+    Mix-in for the conversion from Spark to pandas and PyArrow. Currently, only
+    :class:`DataFrame` can use this class.
     """
 
-    def toPandas(self) -> "PandasDataFrameLike":
+    def _to_pandas(self, **kwargs: Any) -> "PandasDataFrameLike":
         from pyspark.sql.dataframe import DataFrame
 
         assert isinstance(self, DataFrame)
@@ -72,18 +186,44 @@ class PandasConversionMixin:
 
         import pandas as pd
 
-        jconf = self.sparkSession._jconf
+        (
+            sessionLocalTimeZone,
+            arrowPySparkEnabled,
+            arrowUseLargeVarTypes,
+            arrowPySparkFallbackEnabled,
+            arrowPySparkSelfDestructEnabled,
+            pandasStructHandlingMode,
+        ) = self.sparkSession._jconf.getConfs(
+            [
+                "spark.sql.session.timeZone",
+                "spark.sql.execution.arrow.pyspark.enabled",
+                "spark.sql.execution.arrow.useLargeVarTypes",
+                "spark.sql.execution.arrow.pyspark.fallback.enabled",
+                "spark.sql.execution.arrow.pyspark.selfDestruct.enabled",
+                "spark.sql.execution.pandas.structHandlingMode",
+            ]
+        )
 
-        if jconf.arrowPySparkEnabled():
+        # if pandasStructHandlingMode is explicitly set, override the runtime config
+        if "pandasStructHandlingMode" in kwargs:
+            pandasStructHandlingMode = str(kwargs["pandasStructHandlingMode"])
+
+        prefers_large_var_types = arrowUseLargeVarTypes == "true"
+
+        if arrowPySparkEnabled == "true":
             use_arrow = True
             try:
                 from pyspark.sql.pandas.types import to_arrow_schema
                 from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
 
                 require_minimum_pyarrow_version()
-                to_arrow_schema(self.schema)
+                arrow_schema = to_arrow_schema(
+                    self.schema,
+                    timezone="UTC",
+                    prefers_large_types=prefers_large_var_types,
+                )
             except Exception as e:
-                if jconf.arrowPySparkFallbackEnabled():
+                if arrowPySparkFallbackEnabled == "true":
                     msg = (
                         "toPandas attempted Arrow optimization because "
                         "'spark.sql.execution.arrow.pyspark.enabled' is set to true; however, "
@@ -111,74 +251,32 @@ class PandasConversionMixin:
                 try:
                     import pyarrow as pa
 
-                    self_destruct = jconf.arrowPySparkSelfDestructEnabled()
-                    batches = self._collect_as_arrow(split_batches=self_destruct)
+                    batches = self._collect_as_arrow(
+                        split_batches=arrowPySparkSelfDestructEnabled == "true",
+                        prefers_large_var_types=prefers_large_var_types,
+                    )
+
                     if len(batches) > 0:
                         table = pa.Table.from_batches(batches)
-                        # Ensure only the table has a reference to the batches, so that
-                        # self_destruct (if enabled) is effective
-                        del batches
-                        # Pandas DataFrame created from PyArrow uses datetime64[ns] for date type
-                        # values, but we should use datetime.date to match the behavior with when
-                        # Arrow optimization is disabled.
-                        pandas_options = {"date_as_object": True}
-
-                        if LooseVersion(pa.__version__) >= LooseVersion("13.0.0"):
-                            # A legacy option to coerce date32, date64, duration, and timestamp
-                            # time units to nanoseconds when converting to pandas.
-                            # This option can only be added since 13.0.0.
-                            pandas_options.update(
-                                {
-                                    "coerce_temporal_nanoseconds": True,
-                                }
-                            )
-
-                        if self_destruct:
-                            # Configure PyArrow to use as little memory as possible:
-                            # self_destruct - free columns as they are converted
-                            # split_blocks - create a separate Pandas block for each column
-                            # use_threads - convert one column at a time
-                            pandas_options.update(
-                                {
-                                    "self_destruct": True,
-                                    "split_blocks": True,
-                                    "use_threads": False,
-                                }
-                            )
-                        # Rename columns to avoid duplicated column names.
-                        pdf = table.rename_columns(
-                            [f"col_{i}" for i in range(table.num_columns)]
-                        ).to_pandas(**pandas_options)
-
-                        # Rename back to the original column names.
-                        pdf.columns = self.columns
                     else:
-                        pdf = pd.DataFrame(columns=self.columns)
+                        # empty dataset
+                        table = arrow_schema.empty_table()
 
-                    if len(pdf.columns) > 0:
-                        timezone = jconf.sessionLocalTimeZone()
-                        struct_in_pandas = jconf.pandasStructHandlingMode()
+                    # Ensure only the table has a reference to the batches, so that
+                    # self_destruct (if enabled) is effective
+                    del batches
 
-                        error_on_duplicated_field_names = False
-                        if struct_in_pandas == "legacy":
-                            error_on_duplicated_field_names = True
-                            struct_in_pandas = "dict"
+                    pdf = _convert_arrow_table_to_pandas(
+                        arrow_table=table,
+                        schema=self.schema,
+                        timezone=sessionLocalTimeZone,
+                        struct_handling_mode=pandasStructHandlingMode,
+                        date_as_object=True,
+                        self_destruct=arrowPySparkSelfDestructEnabled == "true",
+                    )
+
+                    return pdf
 
-                        return pd.concat(
-                            [
-                                _create_converter_to_pandas(
-                                    field.dataType,
-                                    field.nullable,
-                                    timezone=timezone,
-                                    struct_in_pandas=struct_in_pandas,
-                                    error_on_duplicated_field_names=error_on_duplicated_field_names,
-                                )(pser)
-                                for (_, pser), field in zip(pdf.items(), self.schema.fields)
-                            ],
-                            axis="columns",
-                        )
-                    else:
-                        return pdf
                 except Exception as e:
                     # We might have to allow fallback here as well but multiple Spark jobs can
                     # be executed. So, simply fail in this case for now.
@@ -203,8 +301,8 @@ class PandasConversionMixin:
             pdf = pd.DataFrame(columns=self.columns)
 
         if len(pdf.columns) > 0:
-            timezone = jconf.sessionLocalTimeZone()
-            struct_in_pandas = jconf.pandasStructHandlingMode()
+            timezone = sessionLocalTimeZone
+            struct_in_pandas = pandasStructHandlingMode
 
             return pd.concat(
                 [
@@ -230,21 +328,38 @@ class PandasConversionMixin:
 
         assert isinstance(self, DataFrame)
 
-        jconf = self.sparkSession._jconf
-
         from pyspark.sql.pandas.types import to_arrow_schema
         from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
 
         require_minimum_pyarrow_version()
-        to_arrow_schema(self.schema)
+
+        (
+            arrowUseLargeVarTypes,
+            arrowPySparkSelfDestructEnabled,
+        ) = self.sparkSession._jconf.getConfs(
+            [
+                "spark.sql.execution.arrow.useLargeVarTypes",
+                "spark.sql.execution.arrow.pyspark.selfDestruct.enabled",
+            ]
+        )
+
+        prefers_large_var_types = arrowUseLargeVarTypes == "true"
+        schema = to_arrow_schema(
+            self.schema,
+            error_on_duplicated_field_names_in_struct=True,
+            timezone="UTC",
+            prefers_large_types=prefers_large_var_types,
+        )
 
         import pyarrow as pa
 
-        self_destruct = jconf.arrowPySparkSelfDestructEnabled()
+        self_destruct = arrowPySparkSelfDestructEnabled == "true"
         batches = self._collect_as_arrow(
-            split_batches=self_destruct, empty_list_if_zero_records=False
+            split_batches=self_destruct,
+            empty_list_if_zero_records=False,
+            prefers_large_var_types=prefers_large_var_types,
         )
-        table = pa.Table.from_batches(batches)
+        table = pa.Table.from_batches(batches).cast(schema)
         # Ensure only the table has a reference to the batches, so that
         # self_destruct (if enabled) is effective
         del batches
@@ -254,6 +369,7 @@ class PandasConversionMixin:
         self,
         split_batches: bool = False,
         empty_list_if_zero_records: bool = True,
+        prefers_large_var_types: bool = False,
     ) -> List["pa.RecordBatch"]:
         """
         Returns all records as a list of Arrow RecordBatches. PyArrow must be installed
@@ -282,30 +398,30 @@ class PandasConversionMixin:
 
         # Collect list of un-ordered batches where last element is a list of correct order indices
         try:
-            batch_stream = _load_from_socket((port, auth_secret), ArrowCollectSerializer())
-            if split_batches:
-                # When spark.sql.execution.arrow.pyspark.selfDestruct.enabled, ensure
-                # each column in each record batch is contained in its own allocation.
-                # Otherwise, selfDestruct does nothing; it frees each column as its
-                # converted, but each column will actually be a list of slices of record
-                # batches, and so no memory is actually freed until all columns are
-                # converted.
-                import pyarrow as pa
+            with _load_from_socket((port, auth_secret), ArrowCollectSerializer()) as batch_stream:
+                if split_batches:
+                    # When spark.sql.execution.arrow.pyspark.selfDestruct.enabled, ensure
+                    # each column in each record batch is contained in its own allocation.
+                    # Otherwise, selfDestruct does nothing; it frees each column as its
+                    # converted, but each column will actually be a list of slices of record
+                    # batches, and so no memory is actually freed until all columns are
+                    # converted.
+                    import pyarrow as pa
 
-                results = []
-                for batch_or_indices in batch_stream:
-                    if isinstance(batch_or_indices, pa.RecordBatch):
-                        batch_or_indices = pa.RecordBatch.from_arrays(
-                            [
-                                # This call actually reallocates the array
-                                pa.concat_arrays([array])
-                                for array in batch_or_indices
-                            ],
-                            schema=batch_or_indices.schema,
-                        )
-                    results.append(batch_or_indices)
-            else:
-                results = list(batch_stream)
+                    results = []
+                    for batch_or_indices in batch_stream:
+                        if isinstance(batch_or_indices, pa.RecordBatch):
+                            batch_or_indices = pa.RecordBatch.from_arrays(
+                                [
+                                    # This call actually reallocates the array
+                                    pa.concat_arrays([array])
+                                    for array in batch_or_indices
+                                ],
+                                schema=batch_or_indices.schema,
+                            )
+                        results.append(batch_or_indices)
+                else:
+                    results = list(batch_stream)
         finally:
             with unwrap_spark_exception():
                 # Join serving thread and raise any exceptions from collectAsArrowToPython
@@ -320,16 +436,19 @@ class PandasConversionMixin:
             return [batches[i] for i in batch_order]
         else:
             from pyspark.sql.pandas.types import to_arrow_schema
+            import pyarrow as pa
 
-            schema = to_arrow_schema(self.schema)
+            schema = to_arrow_schema(
+                self.schema, timezone="UTC", prefers_large_types=prefers_large_var_types
+            )
             empty_arrays = [pa.array([], type=field.type) for field in schema]
             return [pa.RecordBatch.from_arrays(empty_arrays, schema=schema)]
 
 
 class SparkConversionMixin:
     """
-    Min-in for the conversion from pandas to Spark. Currently, only :class:`SparkSession`
-    can use this class.
+    Min-in for the conversion from pandas and PyArrow to Spark. Currently, only
+    :class:`SparkSession` can use this class.
     """
 
     _jsparkSession: "JavaObject"
@@ -340,6 +459,12 @@ class SparkConversionMixin:
     ) -> "DataFrame":
         ...
 
+    @overload
+    def createDataFrame(
+        self, data: "pa.Table", samplingRatio: Optional[float] = ...
+    ) -> "DataFrame":
+        ...
+
     @overload
     def createDataFrame(
         self,
@@ -349,9 +474,18 @@ class SparkConversionMixin:
     ) -> "DataFrame":
         ...
 
+    @overload
+    def createDataFrame(
+        self,
+        data: "pa.Table",
+        schema: Union[StructType, str],
+        verifySchema: bool = ...,
+    ) -> "DataFrame":
+        ...
+
     def createDataFrame(  # type: ignore[misc]
         self,
-        data: "PandasDataFrameLike",
+        data: Union["PandasDataFrameLike", "pa.Table"],
         schema: Optional[Union[StructType, List[str]]] = None,
         samplingRatio: Optional[float] = None,
         verifySchema: bool = True,
@@ -360,21 +494,81 @@ class SparkConversionMixin:
 
         assert isinstance(self, SparkSession)
 
+        (
+            timestampType,
+            sessionLocalTimeZone,
+            arrowPySparkEnabled,
+            arrowUseLargeVarTypes,
+            arrowPySparkFallbackEnabled,
+            arrowMaxRecordsPerBatch,
+            arrowSafeTypeConversion,
+            inferPandasDictAsMap,
+        ) = self._jconf.getConfs(
+            [
+                "spark.sql.timestampType",
+                "spark.sql.session.timeZone",
+                "spark.sql.execution.arrow.pyspark.enabled",
+                "spark.sql.execution.arrow.useLargeVarTypes",
+                "spark.sql.execution.arrow.pyspark.fallback.enabled",
+                "spark.sql.execution.arrow.maxRecordsPerBatch",
+                "spark.sql.execution.pandas.convertToArrowArraySafely",
+                "spark.sql.execution.pandas.inferPandasDictAsMap",
+            ]
+        )
+
+        prefer_timestamp_ntz = timestampType == "TIMESTAMP_NTZ"
+        prefers_large_var_types = arrowUseLargeVarTypes == "true"
+        timezone = sessionLocalTimeZone
+        arrow_batch_size = int(arrowMaxRecordsPerBatch)
+        selfcheck = arrowSafeTypeConversion == "true"
+        infer_pandas_dict_as_map = inferPandasDictAsMap == "true"
+
+        if type(data).__name__ == "Table":
+            # `data` is a PyArrow Table
+            from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
+
+            require_minimum_pyarrow_version()
+
+            import pyarrow as pa
+
+            assert isinstance(data, pa.Table)
+
+            # If no schema supplied by user then get the names of columns only
+            if schema is None:
+                schema = data.schema.names
+
+            return self._create_from_arrow_table(
+                data,
+                schema,
+                timezone,
+                prefer_timestamp_ntz,
+                prefers_large_var_types,
+                arrow_batch_size,
+            )
+
+        # `data` is a PandasDataFrameLike object
         from pyspark.sql.pandas.utils import require_minimum_pandas_version
 
         require_minimum_pandas_version()
 
-        timezone = self._jconf.sessionLocalTimeZone()
-
         # If no schema supplied by user then get the names of columns only
         if schema is None:
             schema = [str(x) if not isinstance(x, str) else x for x in data.columns]
 
-        if self._jconf.arrowPySparkEnabled() and len(data) > 0:
+        if arrowPySparkEnabled == "true" and len(data) > 0:
             try:
-                return self._create_from_pandas_with_arrow(data, schema, timezone)
+                return self._create_from_pandas_with_arrow(
+                    data,
+                    schema,
+                    timezone,
+                    prefer_timestamp_ntz,
+                    prefers_large_var_types,
+                    arrow_batch_size,
+                    selfcheck,
+                    infer_pandas_dict_as_map,
+                )
             except Exception as e:
-                if self._jconf.arrowPySparkFallbackEnabled():
+                if arrowPySparkFallbackEnabled == "true":
                     msg = (
                         "createDataFrame attempted Arrow optimization because "
                         "'spark.sql.execution.arrow.pyspark.enabled' is set to true; however, "
@@ -394,11 +588,15 @@ class SparkConversionMixin:
                     )
                     warn(msg)
                     raise
-        converted_data = self._convert_from_pandas(data, schema, timezone)
+        converted_data = self._convert_from_pandas(data, schema, timezone, prefer_timestamp_ntz)
         return self._create_dataframe(converted_data, schema, samplingRatio, verifySchema)
 
     def _convert_from_pandas(
-        self, pdf: "PandasDataFrameLike", schema: Union[StructType, str, List[str]], timezone: str
+        self,
+        pdf: "PandasDataFrameLike",
+        schema: Union[StructType, str, List[str]],
+        timezone: str,
+        prefer_timestamp_ntz: bool,
     ) -> List:
         """
         Convert a pandas.DataFrame to list of records that can be used to make a DataFrame
@@ -487,7 +685,7 @@ class SparkConversionMixin:
                                 else:
                                     return (
                                         pd.Timestamp(value)
-                                        .tz_localize(timezone, ambiguous=False)  # type: ignore
+                                        .tz_localize(timezone, ambiguous=False)
                                         .tz_convert(_get_local_timezone())
                                         .tz_localize(None)
                                         .to_pydatetime()
@@ -514,7 +712,7 @@ class SparkConversionMixin:
                     )
                     copied = True
             else:
-                should_localize = not is_timestamp_ntz_preferred()
+                should_localize = not prefer_timestamp_ntz
                 for column, series in pdf.items():
                     s = series
                     if (
@@ -591,7 +789,15 @@ class SparkConversionMixin:
         return np.dtype(record_type_list) if has_rec_fix else None
 
     def _create_from_pandas_with_arrow(
-        self, pdf: "PandasDataFrameLike", schema: Union[StructType, List[str]], timezone: str
+        self,
+        pdf: "PandasDataFrameLike",
+        schema: Union[StructType, List[str]],
+        timezone: str,
+        prefer_timestamp_ntz: bool,
+        prefers_large_var_types: bool,
+        arrow_batch_size: int,
+        safecheck: bool,
+        infer_pandas_dict_as_map: bool,
     ) -> "DataFrame":
         """
         Create a DataFrame from a given pandas.DataFrame by slicing it into partitions, converting
@@ -624,14 +830,9 @@ class SparkConversionMixin:
         )
         import pyarrow as pa
 
-        infer_pandas_dict_as_map = (
-            str(self.conf.get("spark.sql.execution.pandas.inferPandasDictAsMap")).lower() == "true"
-        )
-
         # Create the Spark schema from list of names passed in with Arrow types
         if isinstance(schema, (list, tuple)):
             arrow_schema = pa.Schema.from_pandas(pdf, preserve_index=False)
-            prefer_timestamp_ntz = is_timestamp_ntz_preferred()
             struct = StructType()
             if infer_pandas_dict_as_map:
                 spark_type: Union[MapType, DataType]
@@ -640,8 +841,8 @@ class SparkConversionMixin:
                     if isinstance(field_type, pa.StructType):
                         if len(field_type) == 0:
                             raise PySparkValueError(
-                                error_class="CANNOT_INFER_EMPTY_SCHEMA",
-                                message_parameters={},
+                                errorClass="CANNOT_INFER_EMPTY_SCHEMA",
+                                messageParameters={},
                             )
                         arrow_type = field_type.field(0).type
                         spark_type = MapType(
@@ -664,8 +865,8 @@ class SparkConversionMixin:
             spark_types = [_deduplicate_field_names(f.dataType) for f in schema.fields]
         elif isinstance(schema, DataType):
             raise PySparkTypeError(
-                error_class="UNSUPPORTED_DATA_TYPE_FOR_ARROW",
-                message_parameters={"data_type": str(schema)},
+                errorClass="UNSUPPORTED_DATA_TYPE_FOR_ARROW",
+                messageParameters={"data_type": str(schema)},
             )
         else:
             # Any timestamps must be coerced to be compatible with Spark
@@ -677,14 +878,20 @@ class SparkConversionMixin:
             ]
 
         # Slice the DataFrame to be batched
-        step = self._jconf.arrowMaxRecordsPerBatch()
+        step = arrow_batch_size
         step = step if step > 0 else len(pdf)
         pdf_slices = (pdf.iloc[start : start + step] for start in range(0, len(pdf), step))
 
         # Create list of Arrow (columns, arrow_type, spark_type) for serializer dump_stream
         arrow_data = [
             [
-                (c, to_arrow_type(t) if t is not None else None, t)
+                (
+                    c,
+                    to_arrow_type(t, timezone="UTC", prefers_large_types=prefers_large_var_types)
+                    if t is not None
+                    else None,
+                    t,
+                )
                 for (_, c), t in zip(pdf_slice.items(), spark_types)
             ]
             for pdf_slice in pdf_slices
@@ -692,8 +899,85 @@ class SparkConversionMixin:
 
         jsparkSession = self._jsparkSession
 
-        safecheck = self._jconf.arrowSafeTypeConversion()
-        ser = ArrowStreamPandasSerializer(timezone, safecheck)
+        ser = ArrowStreamPandasSerializer(timezone, safecheck, False)
+
+        @no_type_check
+        def reader_func(temp_filename):
+            return self._jvm.PythonSQLUtils.readArrowStreamFromFile(temp_filename)
+
+        @no_type_check
+        def create_iter_server():
+            return self._jvm.ArrowIteratorServer()
+
+        # Create Spark DataFrame from Arrow stream file, using one batch per partition
+        jiter = self._sc._serialize_to_jvm(arrow_data, ser, reader_func, create_iter_server)
+        assert self._jvm is not None
+        jdf = self._jvm.PythonSQLUtils.toDataFrame(jiter, schema.json(), jsparkSession)
+        df = DataFrame(jdf, self)
+        df._schema = schema
+        return df
+
+    def _create_from_arrow_table(
+        self,
+        table: "pa.Table",
+        schema: Union[StructType, List[str]],
+        timezone: str,
+        prefer_timestamp_ntz: bool,
+        prefers_large_var_types: bool,
+        arrow_batch_size: int,
+    ) -> "DataFrame":
+        """
+        Create a DataFrame from a given pyarrow.Table by slicing it into partitions then
+        sending to the JVM to parallelize.
+        """
+        from pyspark.sql import SparkSession
+        from pyspark.sql.dataframe import DataFrame
+
+        assert isinstance(self, SparkSession)
+
+        from pyspark.sql.pandas.serializers import ArrowStreamSerializer
+        from pyspark.sql.pandas.types import (
+            from_arrow_type,
+            from_arrow_schema,
+            to_arrow_schema,
+            _check_arrow_table_timestamps_localize,
+        )
+        from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
+
+        require_minimum_pyarrow_version()
+
+        # Create the Spark schema from list of names passed in with Arrow types
+        if isinstance(schema, (list, tuple)):
+            table = table.rename_columns(schema)
+            arrow_schema = table.schema
+            struct = StructType()
+            for name, field in zip(schema, arrow_schema):
+                struct.add(
+                    name,
+                    from_arrow_type(field.type, prefer_timestamp_ntz),
+                    nullable=field.nullable,
+                )
+            schema = struct
+
+        if not isinstance(schema, StructType):
+            schema = from_arrow_schema(table.schema, prefer_timestamp_ntz=prefer_timestamp_ntz)
+
+        table = _check_arrow_table_timestamps_localize(table, schema, True, timezone).cast(
+            to_arrow_schema(
+                schema,
+                error_on_duplicated_field_names_in_struct=True,
+                timezone="UTC",
+                prefers_large_types=prefers_large_var_types,
+            )
+        )
+
+        # Chunk the Arrow Table into RecordBatches
+        chunk_size = arrow_batch_size
+        arrow_data = table.to_batches(max_chunksize=chunk_size)
+
+        jsparkSession = self._jsparkSession
+
+        ser = ArrowStreamSerializer()
 
         @no_type_check
         def reader_func(temp_filename):
