@@ -20,6 +20,7 @@ import struct
 import sys
 import unittest
 import difflib
+import faulthandler
 import functools
 from decimal import Decimal
 from time import time, sleep
@@ -94,6 +95,21 @@ openpyxl_requirement_message = None if have_openpyxl else "No module named 'open
 have_yaml = have_package("yaml")
 yaml_requirement_message = None if have_yaml else "No module named 'yaml'"
 
+have_grpc = have_package("grpc")
+grpc_requirement_message = None if have_grpc else "No module named 'grpc'"
+
+have_grpc_status = have_package("grpc_status")
+grpc_status_requirement_message = None if have_grpc_status else "No module named 'grpc_status'"
+
+
+googleapis_common_protos_requirement_message = None
+
+try:
+    from google.rpc import error_details_pb2
+except ImportError as e:
+    googleapis_common_protos_requirement_message = str(e)
+have_googleapis_common_protos = googleapis_common_protos_requirement_message is None
+
 pandas_requirement_message = None
 try:
     from pyspark.sql.pandas.utils import require_minimum_pandas_version
@@ -117,6 +133,18 @@ except Exception as e:
 
 have_pyarrow = pyarrow_requirement_message is None
 
+
+connect_requirement_message = (
+    pandas_requirement_message
+    or pyarrow_requirement_message
+    or grpc_requirement_message
+    or googleapis_common_protos_requirement_message
+    or grpc_status_requirement_message
+)
+
+should_test_connect = connect_requirement_message is None
+
+
 is_ansi_mode_test = True
 if os.environ.get("SPARK_ANSI_SQL_MODE") == "false":
     is_ansi_mode_test = False
@@ -132,15 +160,15 @@ def write_int(i):
     return struct.pack("!i", i)
 
 
-def timeout(seconds):
+def timeout(timeout):
     def decorator(func):
         def handler(signum, frame):
-            raise TimeoutError(f"Function {func.__name__} timed out after {seconds} seconds")
+            raise TimeoutError(f"Function {func.__name__} timed out after {timeout} seconds")
 
         def wrapper(*args, **kwargs):
             signal.alarm(0)
             signal.signal(signal.SIGALRM, handler)
-            signal.alarm(seconds)
+            signal.alarm(timeout)
             try:
                 result = func(*args, **kwargs)
             finally:
@@ -155,6 +183,7 @@ def timeout(seconds):
 def eventually(
     timeout=30.0,
     catch_assertions=False,
+    catch_timeout=False,
 ):
     """
     Wait a given amount of time for a condition to pass, else fail with an error.
@@ -176,9 +205,14 @@ def eventually(
         If False (default), do not catch AssertionErrors.
         If True, catch AssertionErrors; continue, but save
         error to throw upon timeout.
+    catch_timeout : bool
+        If False (default), do not catch TimeoutError.
+        If True, catch TimeoutError; continue, but save
+        error to throw upon timeout.
     """
     assert timeout > 0
     assert isinstance(catch_assertions, bool)
+    assert isinstance(catch_timeout, bool)
 
     def decorator(condition: Callable) -> Callable:
         assert isinstance(condition, Callable)
@@ -191,13 +225,18 @@ def eventually(
             while time() - start_time < timeout:
                 numTries += 1
 
-                if catch_assertions:
-                    try:
-                        lastValue = condition(*args, **kwargs)
-                    except AssertionError as e:
-                        lastValue = e
-                else:
+                try:
                     lastValue = condition(*args, **kwargs)
+                except AssertionError as e:
+                    if catch_assertions:
+                        lastValue = e
+                    else:
+                        raise e
+                except TimeoutError as e:
+                    if catch_timeout:
+                        lastValue = e
+                    else:
+                        raise e
 
                 if lastValue is True or lastValue is None:
                     return
@@ -205,7 +244,7 @@ def eventually(
                 print(f"\nAttempt #{numTries} failed!\n{lastValue}")
                 sleep(0.01)
 
-            if isinstance(lastValue, AssertionError):
+            if isinstance(lastValue, (AssertionError, TimeoutError)):
                 raise lastValue
             else:
                 raise AssertionError(
@@ -230,7 +269,19 @@ class QuietTest:
         self.log4j.LogManager.getRootLogger().setLevel(self.old_level)
 
 
-class PySparkTestCase(unittest.TestCase):
+class PySparkBaseTestCase(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        if os.environ.get("PYSPARK_TEST_TIMEOUT"):
+            faulthandler.register(signal.SIGTERM, file=sys.__stderr__, all_threads=True)
+
+    @classmethod
+    def tearDownClass(cls):
+        if os.environ.get("PYSPARK_TEST_TIMEOUT"):
+            faulthandler.unregister(signal.SIGTERM)
+
+
+class PySparkTestCase(PySparkBaseTestCase):
     def setUp(self):
         from pyspark import SparkContext
 
@@ -243,7 +294,7 @@ class PySparkTestCase(unittest.TestCase):
         sys.path = self._old_sys_path
 
 
-class ReusedPySparkTestCase(unittest.TestCase):
+class ReusedPySparkTestCase(PySparkBaseTestCase):
     @classmethod
     def conf(cls):
         """
@@ -253,6 +304,8 @@ class ReusedPySparkTestCase(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
+        super().setUpClass()
+
         from pyspark import SparkContext
 
         cls.sc = SparkContext(cls.master(), cls.__name__, conf=cls.conf())
@@ -263,7 +316,10 @@ class ReusedPySparkTestCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        cls.sc.stop()
+        try:
+            cls.sc.stop()
+        finally:
+            super().tearDownClass()
 
     def test_assert_classic_mode(self):
         from pyspark.sql import is_remote
@@ -421,6 +477,7 @@ def assertSchemaEqual(
     ignoreColumnOrder: bool = False,
     ignoreColumnName: bool = False,
 ):
+    __tracebackhide__ = True
     r"""
     A util function to assert equality between DataFrame schemas `actual` and `expected`.
 
@@ -553,6 +610,10 @@ def assertSchemaEqual(
         if dt1.typeName() == dt2.typeName():
             if dt1.typeName() == "array":
                 return compare_datatypes_ignore_nullable(dt1.elementType, dt2.elementType)
+            elif dt1.typeName() == "map":
+                return compare_datatypes_ignore_nullable(
+                    dt1.keyType, dt2.keyType
+                ) and compare_datatypes_ignore_nullable(dt1.valueType, dt2.valueType)
             elif dt1.typeName() == "decimal":
                 # Fix for SPARK-51062: Compare precision and scale for decimal types
                 return dt1.precision == dt2.precision and dt1.scale == dt2.scale
@@ -610,6 +671,7 @@ def assertDataFrameEqual(
     showOnlyDiff: bool = False,
     includeDiffRows=False,
 ):
+    __tracebackhide__ = True
     r"""
     A util function to assert equality between `actual` and `expected`
     (DataFrames or lists of Rows), with optional parameters `checkRowOrder`, `rtol`, and `atol`.
@@ -996,6 +1058,7 @@ def assertDataFrameEqual(
     def assert_rows_equal(
         rows1: List[Row], rows2: List[Row], maxErrors: int = None, showOnlyDiff: bool = False
     ):
+        __tracebackhide__ = True
         zipped = list(zip_longest(rows1, rows2))
         diff_rows_cnt = 0
         diff_rows = []
