@@ -19,6 +19,7 @@ package org.apache.spark.sql.catalyst.util
 import java.lang.invoke.{MethodHandles, MethodType}
 import java.sql.{Date, Timestamp}
 import java.time.{Instant, LocalDate, LocalDateTime, LocalTime, ZonedDateTime, ZoneId, ZoneOffset}
+import java.time.temporal.ChronoField.NANO_OF_DAY
 import java.util.TimeZone
 import java.util.concurrent.TimeUnit.{MICROSECONDS, NANOSECONDS}
 import java.util.regex.Pattern
@@ -29,7 +30,7 @@ import org.apache.spark.QueryContext
 import org.apache.spark.sql.catalyst.util.DateTimeConstants._
 import org.apache.spark.sql.catalyst.util.RebaseDateTime.{rebaseGregorianToJulianDays, rebaseGregorianToJulianMicros, rebaseJulianToGregorianDays, rebaseJulianToGregorianMicros}
 import org.apache.spark.sql.errors.ExecutionErrors
-import org.apache.spark.sql.types.{DateType, TimestampType}
+import org.apache.spark.sql.types.{DateType, TimestampType, TimeType}
 import org.apache.spark.unsafe.types.UTF8String
 import org.apache.spark.util.SparkClassUtils
 
@@ -41,12 +42,16 @@ trait SparkDateTimeUtils {
   final val singleMinuteTz = Pattern.compile("(\\+|\\-)(\\d\\d):(\\d)$")
 
   def getZoneId(timeZoneId: String): ZoneId = {
-    // To support the (+|-)h:mm format because it was supported before Spark 3.0.
-    var formattedZoneId = singleHourTz.matcher(timeZoneId).replaceFirst("$10$2:")
-    // To support the (+|-)hh:m format because it was supported before Spark 3.0.
-    formattedZoneId = singleMinuteTz.matcher(formattedZoneId).replaceFirst("$1$2:0$3")
-
-    ZoneId.of(formattedZoneId, ZoneId.SHORT_IDS)
+    try {
+      // To support the (+|-)h:mm format because it was supported before Spark 3.0.
+      var formattedZoneId = singleHourTz.matcher(timeZoneId).replaceFirst("$10$2:")
+      // To support the (+|-)hh:m format because it was supported before Spark 3.0.
+      formattedZoneId = singleMinuteTz.matcher(formattedZoneId).replaceFirst("$1$2:0$3")
+      ZoneId.of(formattedZoneId, ZoneId.SHORT_IDS)
+    } catch {
+      case e: java.time.DateTimeException =>
+        throw ExecutionErrors.zoneOffsetError(timeZoneId, e)
+    }
   }
 
   def getTimeZone(timeZoneId: String): TimeZone = TimeZone.getTimeZone(getZoneId(timeZoneId))
@@ -54,8 +59,10 @@ trait SparkDateTimeUtils {
   /**
    * Converts an Java object to days.
    *
-   * @param obj Either an object of `java.sql.Date` or `java.time.LocalDate`.
-   * @return The number of days since 1970-01-01.
+   * @param obj
+   *   Either an object of `java.sql.Date` or `java.time.LocalDate`.
+   * @return
+   *   The number of days since 1970-01-01.
    */
   def anyToDays(obj: Any): Int = obj match {
     case d: Date => fromJavaDate(d)
@@ -65,8 +72,10 @@ trait SparkDateTimeUtils {
   /**
    * Converts an Java object to microseconds.
    *
-   * @param obj Either an object of `java.sql.Timestamp` or `java.time.{Instant,LocalDateTime}`.
-   * @return The number of micros since the epoch.
+   * @param obj
+   *   Either an object of `java.sql.Timestamp` or `java.time.{Instant,LocalDateTime}`.
+   * @return
+   *   The number of micros since the epoch.
    */
   def anyToMicros(obj: Any): Long = obj match {
     case t: Timestamp => fromJavaTimestamp(t)
@@ -75,9 +84,15 @@ trait SparkDateTimeUtils {
   }
 
   /**
-   * Converts the timestamp to milliseconds since epoch. In Spark timestamp values have microseconds
+   * Converts the time to microseconds since midnight. In Spark time values have nanoseconds
    * precision, so this conversion is lossy.
    */
+  def nanosToMicros(nanos: Long): Long = Math.floorDiv(nanos, MICROS_PER_MILLIS)
+
+  /**
+   * Converts the timestamp to milliseconds since epoch. In Spark timestamp values have
+   * microseconds precision, so this conversion is lossy.
+   */
   def microsToMillis(micros: Long): Long = {
     // When the timestamp is negative i.e before 1970, we need to adjust the milliseconds portion.
     // Example - 1965-01-01 10:11:12.123456 is represented as (-157700927876544) in micro precision.
@@ -92,13 +107,18 @@ trait SparkDateTimeUtils {
     Math.multiplyExact(millis, MICROS_PER_MILLIS)
   }
 
+  /**
+   * Converts microseconds since the midnight to nanoseconds.
+   */
+  def microsToNanos(micros: Long): Long = Math.multiplyExact(micros, NANOS_PER_MICROS)
+
   // See issue SPARK-35679
   // min second cause overflow in instant to micro
   private val MIN_SECONDS = Math.floorDiv(Long.MinValue, MICROS_PER_SECOND)
 
   /**
-   * Obtains an instance of `java.time.Instant` using microseconds from
-   * the epoch of 1970-01-01 00:00:00Z.
+   * Obtains an instance of `java.time.Instant` using microseconds from the epoch of 1970-01-01
+   * 00:00:00Z.
    */
   def microsToInstant(micros: Long): Instant = {
     val secs = Math.floorDiv(micros, MICROS_PER_SECOND)
@@ -110,8 +130,8 @@ trait SparkDateTimeUtils {
 
   /**
    * Gets the number of microseconds since the epoch of 1970-01-01 00:00:00Z from the given
-   * instance of `java.time.Instant`. The epoch microsecond count is a simple incrementing count of
-   * microseconds where microsecond 0 is 1970-01-01 00:00:00Z.
+   * instance of `java.time.Instant`. The epoch microsecond count is a simple incrementing count
+   * of microseconds where microsecond 0 is 1970-01-01 00:00:00Z.
    */
   def instantToMicros(instant: Instant): Long = {
     val secs = instant.getEpochSecond
@@ -124,11 +144,50 @@ trait SparkDateTimeUtils {
     }
   }
 
+  /**
+   * Gets the number of nanoseconds since midnight using the given time zone.
+   */
+  def instantToNanosOfDay(instant: Instant, timezone: String): Long = {
+    instantToNanosOfDay(instant, getZoneId(timezone))
+  }
+
+  /**
+   * Gets the number of nanoseconds since midnight using the given time zone.
+   */
+  def instantToNanosOfDay(instant: Instant, zoneId: ZoneId): Long = {
+    val localDateTime = LocalDateTime.ofInstant(instant, zoneId)
+    localDateTime.toLocalTime.getLong(NANO_OF_DAY)
+  }
+
+  /**
+   * Truncates a time value (in nanoseconds) to the specified fractional precision `p`.
+   *
+   * For example, if `p = 3`, we keep millisecond resolution and discard any digits beyond the
+   * thousand-nanosecond place. So a value like `123456` microseconds (12:34:56.123456) becomes
+   * `123000` microseconds (12:34:56.123).
+   *
+   * @param nanos
+   *   The original time in nanoseconds.
+   * @param p
+   *   The fractional second precision (range 0 to 6).
+   * @return
+   *   The truncated nanosecond value, preserving only `p` fractional digits.
+   */
+  def truncateTimeToPrecision(nanos: Long, p: Int): Long = {
+    assert(
+      TimeType.MIN_PRECISION <= p && p <= TimeType.MAX_PRECISION,
+      s"Fractional second precision $p out" +
+        s" of range [${TimeType.MIN_PRECISION}..${TimeType.MAX_PRECISION}].")
+    val scale = TimeType.NANOS_PRECISION - p
+    val factor = math.pow(10, scale).toLong
+    (nanos / factor) * factor
+  }
+
   /**
    * Converts the timestamp `micros` from one timezone to another.
    *
-   * Time-zone rules, such as daylight savings, mean that not every local date-time
-   * is valid for the `toZone` time zone, thus the local date-time may be adjusted.
+   * Time-zone rules, such as daylight savings, mean that not every local date-time is valid for
+   * the `toZone` time zone, thus the local date-time may be adjusted.
    */
   def convertTz(micros: Long, fromZone: ZoneId, toZone: ZoneId): Long = {
     val rebasedDateTime = getLocalDateTime(micros, toZone).atZone(fromZone)
@@ -160,14 +219,16 @@ trait SparkDateTimeUtils {
   def daysToLocalDate(days: Int): LocalDate = LocalDate.ofEpochDay(days)
 
   /**
-   * Converts microseconds since 1970-01-01 00:00:00Z to days since 1970-01-01 at the given zone ID.
+   * Converts microseconds since 1970-01-01 00:00:00Z to days since 1970-01-01 at the given zone
+   * ID.
    */
   def microsToDays(micros: Long, zoneId: ZoneId): Int = {
     localDateToDays(getLocalDateTime(micros, zoneId).toLocalDate)
   }
 
   /**
-   * Converts days since 1970-01-01 at the given zone ID to microseconds since 1970-01-01 00:00:00Z.
+   * Converts days since 1970-01-01 at the given zone ID to microseconds since 1970-01-01
+   * 00:00:00Z.
    */
   def daysToMicros(days: Int, zoneId: ZoneId): Long = {
     val instant = daysToLocalDate(days).atStartOfDay(zoneId).toInstant
@@ -175,20 +236,33 @@ trait SparkDateTimeUtils {
   }
 
   /**
-   * Converts a local date at the default JVM time zone to the number of days since 1970-01-01
-   * in the hybrid calendar (Julian + Gregorian) by discarding the time part. The resulted days are
+   * Converts the local time to the number of nanoseconds within the day, from 0 to (24 * 60 * 60
+   * * 1000 * 1000 * 1000) - 1.
+   */
+  def localTimeToNanos(localTime: LocalTime): Long = localTime.getLong(NANO_OF_DAY)
+
+  /**
+   * Converts the number of nanoseconds within the day to the local time.
+   */
+  def nanosToLocalTime(nanos: Long): LocalTime = LocalTime.ofNanoOfDay(nanos)
+
+  /**
+   * Converts a local date at the default JVM time zone to the number of days since 1970-01-01 in
+   * the hybrid calendar (Julian + Gregorian) by discarding the time part. The resulted days are
    * rebased from the hybrid to Proleptic Gregorian calendar. The days rebasing is performed via
-   * UTC time zone for simplicity because the difference between two calendars is the same in
-   * any given time zone and UTC time zone.
+   * UTC time zone for simplicity because the difference between two calendars is the same in any
+   * given time zone and UTC time zone.
    *
-   * Note: The date is shifted by the offset of the default JVM time zone for backward compatibility
-   *       with Spark 2.4 and earlier versions. The goal of the shift is to get a local date derived
-   *       from the number of days that has the same date fields (year, month, day) as the original
-   *       `date` at the default JVM time zone.
+   * Note: The date is shifted by the offset of the default JVM time zone for backward
+   * compatibility with Spark 2.4 and earlier versions. The goal of the shift is to get a local
+   * date derived from the number of days that has the same date fields (year, month, day) as the
+   * original `date` at the default JVM time zone.
    *
-   * @param date It represents a specific instant in time based on the hybrid calendar which
-   *             combines Julian and Gregorian calendars.
-   * @return The number of days since the epoch in Proleptic Gregorian calendar.
+   * @param date
+   *   It represents a specific instant in time based on the hybrid calendar which combines Julian
+   *   and Gregorian calendars.
+   * @return
+   *   The number of days since the epoch in Proleptic Gregorian calendar.
    */
   def fromJavaDate(date: Date): Int = {
     val millisUtc = date.getTime
@@ -207,18 +281,20 @@ trait SparkDateTimeUtils {
   }
 
   /**
-   * Converts days since the epoch 1970-01-01 in Proleptic Gregorian calendar to a local date
-   * at the default JVM time zone in the hybrid calendar (Julian + Gregorian). It rebases the given
+   * Converts days since the epoch 1970-01-01 in Proleptic Gregorian calendar to a local date at
+   * the default JVM time zone in the hybrid calendar (Julian + Gregorian). It rebases the given
    * days from Proleptic Gregorian to the hybrid calendar at UTC time zone for simplicity because
    * the difference between two calendars doesn't depend on any time zone. The result is shifted
-   * by the time zone offset in wall clock to have the same date fields (year, month, day)
-   * at the default JVM time zone as the input `daysSinceEpoch` in Proleptic Gregorian calendar.
+   * by the time zone offset in wall clock to have the same date fields (year, month, day) at the
+   * default JVM time zone as the input `daysSinceEpoch` in Proleptic Gregorian calendar.
    *
-   * Note: The date is shifted by the offset of the default JVM time zone for backward compatibility
-   *       with Spark 2.4 and earlier versions.
+   * Note: The date is shifted by the offset of the default JVM time zone for backward
+   * compatibility with Spark 2.4 and earlier versions.
    *
-   * @param days The number of days since 1970-01-01 in Proleptic Gregorian calendar.
-   * @return A local date in the hybrid calendar as `java.sql.Date` from number of days since epoch.
+   * @param days
+   *   The number of days since 1970-01-01 in Proleptic Gregorian calendar.
+   * @return
+   *   A local date in the hybrid calendar as `java.sql.Date` from number of days since epoch.
    */
   def toJavaDate(days: Int): Date = {
     val rebasedDays = rebaseGregorianToJulianDays(days)
@@ -233,29 +309,36 @@ trait SparkDateTimeUtils {
   }
 
   /**
-   * Converts microseconds since the epoch to an instance of `java.sql.Timestamp`
-   * via creating a local timestamp at the system time zone in Proleptic Gregorian
-   * calendar, extracting date and time fields like `year` and `hours`, and forming
-   * new timestamp in the hybrid calendar from the extracted fields.
+   * Converts microseconds since the epoch to an instance of `java.sql.Timestamp` via creating a
+   * local timestamp at the system time zone in Proleptic Gregorian calendar, extracting date and
+   * time fields like `year` and `hours`, and forming new timestamp in the hybrid calendar from
+   * the extracted fields.
    *
-   * The conversion is based on the JVM system time zone because the `java.sql.Timestamp`
-   * uses the time zone internally.
+   * The conversion is based on the JVM system time zone because the `java.sql.Timestamp` uses the
+   * time zone internally.
    *
    * The method performs the conversion via local timestamp fields to have the same date-time
-   * representation as `year`, `month`, `day`, ..., `seconds` in the original calendar
-   * and in the target calendar.
+   * representation as `year`, `month`, `day`, ..., `seconds` in the original calendar and in the
+   * target calendar.
    *
-   * @param micros The number of microseconds since 1970-01-01T00:00:00.000000Z.
-   * @return A `java.sql.Timestamp` from number of micros since epoch.
+   * @param micros
+   *   The number of microseconds since 1970-01-01T00:00:00.000000Z.
+   * @return
+   *   A `java.sql.Timestamp` from number of micros since epoch.
    */
   def toJavaTimestamp(micros: Long): Timestamp =
     toJavaTimestampNoRebase(rebaseGregorianToJulianMicros(micros))
 
+  def toJavaTimestamp(timeZoneId: String, micros: Long): Timestamp =
+    toJavaTimestampNoRebase(rebaseGregorianToJulianMicros(timeZoneId, micros))
+
   /**
    * Converts microseconds since the epoch to an instance of `java.sql.Timestamp`.
    *
-   * @param micros The number of microseconds since 1970-01-01T00:00:00.000000Z.
-   * @return A `java.sql.Timestamp` from number of micros since epoch.
+   * @param micros
+   *   The number of microseconds since 1970-01-01T00:00:00.000000Z.
+   * @return
+   *   A `java.sql.Timestamp` from number of micros since epoch.
    */
   def toJavaTimestampNoRebase(micros: Long): Timestamp = {
     val seconds = Math.floorDiv(micros, MICROS_PER_SECOND)
@@ -267,54 +350,54 @@ trait SparkDateTimeUtils {
 
   /**
    * Converts an instance of `java.sql.Timestamp` to the number of microseconds since
-   * 1970-01-01T00:00:00.000000Z. It extracts date-time fields from the input, builds
-   * a local timestamp in Proleptic Gregorian calendar from the fields, and binds
-   * the timestamp to the system time zone. The resulted instant is converted to
-   * microseconds since the epoch.
+   * 1970-01-01T00:00:00.000000Z. It extracts date-time fields from the input, builds a local
+   * timestamp in Proleptic Gregorian calendar from the fields, and binds the timestamp to the
+   * system time zone. The resulted instant is converted to microseconds since the epoch.
    *
-   * The conversion is performed via the system time zone because it is used internally
-   * in `java.sql.Timestamp` while extracting date-time fields.
+   * The conversion is performed via the system time zone because it is used internally in
+   * `java.sql.Timestamp` while extracting date-time fields.
    *
    * The goal of the function is to have the same local date-time in the original calendar
-   * - the hybrid calendar (Julian + Gregorian) and in the target calendar which is
-   * Proleptic Gregorian calendar, see SPARK-26651.
+   *   - the hybrid calendar (Julian + Gregorian) and in the target calendar which is Proleptic
+   *     Gregorian calendar, see SPARK-26651.
    *
-   * @param t It represents a specific instant in time based on
-   *          the hybrid calendar which combines Julian and
-   *          Gregorian calendars.
-   * @return The number of micros since epoch from `java.sql.Timestamp`.
+   * @param t
+   *   It represents a specific instant in time based on the hybrid calendar which combines Julian
+   *   and Gregorian calendars.
+   * @return
+   *   The number of micros since epoch from `java.sql.Timestamp`.
    */
   def fromJavaTimestamp(t: Timestamp): Long =
     rebaseJulianToGregorianMicros(fromJavaTimestampNoRebase(t))
 
+  def fromJavaTimestamp(timeZoneId: String, t: Timestamp): Long =
+    rebaseJulianToGregorianMicros(timeZoneId, fromJavaTimestampNoRebase(t))
+
   /**
    * Converts an instance of `java.sql.Timestamp` to the number of microseconds since
    * 1970-01-01T00:00:00.000000Z.
    *
-   * @param t an instance of `java.sql.Timestamp`.
-   * @return The number of micros since epoch from `java.sql.Timestamp`.
+   * @param t
+   *   an instance of `java.sql.Timestamp`.
+   * @return
+   *   The number of micros since epoch from `java.sql.Timestamp`.
    */
   def fromJavaTimestampNoRebase(t: Timestamp): Long =
     millisToMicros(t.getTime) + (t.getNanos / NANOS_PER_MICROS) % MICROS_PER_MILLIS
 
   /**
-   * Trims and parses a given UTF8 date string to a corresponding [[Int]] value.
-   * The return type is [[Option]] in order to distinguish between 0 and null. The following
-   * formats are allowed:
+   * Trims and parses a given UTF8 date string to a corresponding [[Int]] value. The return type
+   * is [[Option]] in order to distinguish between 0 and null. The following formats are allowed:
    *
-   * `[+-]yyyy*`
-   * `[+-]yyyy*-[m]m`
-   * `[+-]yyyy*-[m]m-[d]d`
-   * `[+-]yyyy*-[m]m-[d]d `
-   * `[+-]yyyy*-[m]m-[d]d *`
-   * `[+-]yyyy*-[m]m-[d]dT*`
+   * `[+-]yyyy*` `[+-]yyyy*-[m]m` `[+-]yyyy*-[m]m-[d]d` `[+-]yyyy*-[m]m-[d]d `
+   * `[+-]yyyy*-[m]m-[d]d *` `[+-]yyyy*-[m]m-[d]dT*`
    */
   def stringToDate(s: UTF8String): Option[Int] = {
     def isValidDigits(segment: Int, digits: Int): Boolean = {
       // An integer is able to represent a date within [+-]5 million years.
       val maxDigitsYear = 7
       (segment == 0 && digits >= 4 && digits <= maxDigitsYear) ||
-        (segment != 0 && digits > 0 && digits <= 2)
+      (segment != 0 && digits > 0 && digits <= 2)
     }
     if (s == null) {
       return None
@@ -374,24 +457,18 @@ trait SparkDateTimeUtils {
     }
   }
 
-  def stringToDateAnsi(
-      s: UTF8String,
-      context: QueryContext = null): Int = {
+  def stringToDateAnsi(s: UTF8String, context: QueryContext = null): Int = {
     stringToDate(s).getOrElse {
       throw ExecutionErrors.invalidInputInCastToDatetimeError(s, DateType, context)
     }
   }
 
   /**
-   * Trims and parses a given UTF8 timestamp string to the corresponding timestamp segments,
-   * time zone id and whether it is just time without a date.
-   * value. The return type is [[Option]] in order to distinguish between 0L and null. The following
-   * formats are allowed:
+   * Trims and parses a given UTF8 timestamp string to the corresponding timestamp segments, time
+   * zone id and whether it is just time without a date. value. The return type is [[Option]] in
+   * order to distinguish between 0L and null. The following formats are allowed:
    *
-   * `[+-]yyyy*`
-   * `[+-]yyyy*-[m]m`
-   * `[+-]yyyy*-[m]m-[d]d`
-   * `[+-]yyyy*-[m]m-[d]d `
+   * `[+-]yyyy*` `[+-]yyyy*-[m]m` `[+-]yyyy*-[m]m-[d]d` `[+-]yyyy*-[m]m-[d]d `
    * `[+-]yyyy*-[m]m-[d]d [h]h:[m]m:[s]s.[ms][ms][ms][us][us][us][zone_id]`
    * `[+-]yyyy*-[m]m-[d]dT[h]h:[m]m:[s]s.[ms][ms][ms][us][us][us][zone_id]`
    * `[h]h:[m]m:[s]s.[ms][ms][ms][us][us][us][zone_id]`
@@ -401,16 +478,17 @@ trait SparkDateTimeUtils {
    *   - Z - Zulu time zone UTC+0
    *   - +|-[h]h:[m]m
    *   - A short id, see https://docs.oracle.com/javase/8/docs/api/java/time/ZoneId.html#SHORT_IDS
-   *   - An id with one of the prefixes UTC+, UTC-, GMT+, GMT-, UT+ or UT-,
-   *     and a suffix in the formats:
+   *   - An id with one of the prefixes UTC+, UTC-, GMT+, GMT-, UT+ or UT-, and a suffix in the
+   *     formats:
    *     - +|-h[h]
    *     - +|-hh[:]mm
    *     - +|-hh:mm:ss
    *     - +|-hhmmss
-   *  - Region-based zone IDs in the form `area/city`, such as `Europe/Paris`
+   *   - Region-based zone IDs in the form `area/city`, such as `Europe/Paris`
    *
-   * @return timestamp segments, time zone id and whether the input is just time without a date. If
-   *         the input string can't be parsed as timestamp, the result timestamp segments are empty.
+   * @return
+   *   timestamp segments, time zone id and whether the input is just time without a date. If the
+   *   input string can't be parsed as timestamp, the result timestamp segments are empty.
    */
   def parseTimestampString(s: UTF8String): (Array[Int], Option[ZoneId], Boolean) = {
     def isValidDigits(segment: Int, digits: Int): Boolean = {
@@ -418,9 +496,9 @@ trait SparkDateTimeUtils {
       val maxDigitsYear = 6
       // For the nanosecond part, more than 6 digits is allowed, but will be truncated.
       segment == 6 || (segment == 0 && digits >= 4 && digits <= maxDigitsYear) ||
-        // For the zoneId segment(7), it's could be zero digits when it's a region-based zone ID
-        (segment == 7 && digits <= 2) ||
-        (segment != 0 && segment != 6 && segment != 7 && digits > 0 && digits <= 2)
+      // For the zoneId segment(7), it's could be zero digits when it's a region-based zone ID
+      (segment == 7 && digits <= 2) ||
+      (segment != 0 && segment != 6 && segment != 7 && digits > 0 && digits <= 2)
     }
     if (s == null) {
       return (Array.empty, None, false)
@@ -517,7 +595,7 @@ trait SparkDateTimeUtils {
             tz = Some(new String(bytes, j, strEndTrimmed - j))
             j = strEndTrimmed - 1
           }
-          if (i == 6  && b != '.') {
+          if (i == 6 && b != '.') {
             i += 1
           }
         } else {
@@ -606,11 +684,11 @@ trait SparkDateTimeUtils {
    *
    * If the input string contains a component associated with time zone, the method will return
    * `None` if `allowTimeZone` is set to `false`. If `allowTimeZone` is set to `true`, the method
-   * will simply discard the time zone component. Enable the check to detect situations like parsing
-   * a timestamp with time zone as TimestampNTZType.
+   * will simply discard the time zone component. Enable the check to detect situations like
+   * parsing a timestamp with time zone as TimestampNTZType.
    *
-   * The return type is [[Option]] in order to distinguish between 0L and null. Please
-   * refer to `parseTimestampString` for the allowed formats.
+   * The return type is [[Option]] in order to distinguish between 0L and null. Please refer to
+   * `parseTimestampString` for the allowed formats.
    */
   def stringToTimestampWithoutTimeZone(s: UTF8String, allowTimeZone: Boolean): Option[Long] = {
     try {
@@ -631,10 +709,90 @@ trait SparkDateTimeUtils {
   }
 
   /**
-   * Returns the index of the first non-whitespace and non-ISO control character in the byte array.
+   * Trims and parses a given UTF8 string to a corresponding [[Long]] value which representing the
+   * number of microseconds since the midnight. The result will be independent of time zones.
    *
-   * @param bytes The byte array to be processed.
-   * @return The start index after trimming.
+   * The return type is [[Option]] in order to distinguish between 0L and null. Please refer to
+   * `parseTimestampString` for the allowed formats.
+   */
+  def stringToTime(s: UTF8String): Option[Long] = {
+    try {
+      // Check for the AM/PM suffix.
+      val trimmed = s.trimRight
+      val numChars = trimmed.numChars()
+      var (isAM, isPM, hasSuffix) = (false, false, false)
+      if (numChars > 2) {
+        val lc = trimmed.getChar(numChars - 1)
+        if (lc == 'M' || lc == 'm') {
+          val slc = trimmed.getChar(numChars - 2)
+          isAM = slc == 'A' || slc == 'a'
+          isPM = slc == 'P' || slc == 'p'
+          hasSuffix = isAM || isPM
+        }
+      }
+      val timeString = if (hasSuffix) {
+        trimmed.substring(0, numChars - 2)
+      } else {
+        trimmed
+      }
+
+      val (segments, zoneIdOpt, justTime) = parseTimestampString(timeString)
+
+      // If the input string can't be parsed as a time, or it contains not only
+      // the time part or has time zone information, return None.
+      if (segments.isEmpty || !justTime || zoneIdOpt.isDefined) {
+        return None
+      }
+
+      // Unpack the segments.
+      var (hr, min, sec, ms) = (segments(3), segments(4), segments(5), segments(6))
+
+      // Handle AM/PM conversion in separate cases.
+      if (!hasSuffix) {
+        // For 24-hour format, validate hour range: 0-23.
+        if (hr < 0 || hr > 23) {
+          return None
+        }
+      } else {
+        // For 12-hour format, validate hour range: 1-12.
+        if (hr < 1 || hr > 12) {
+          return None
+        }
+        // For 12-hour format, convert to 24-hour format.
+        if (isAM) {
+          // AM: 12:xx:xx becomes 00:xx:xx, 1-11:xx:xx stays the same.
+          if (hr == 12) {
+            hr = 0
+          }
+        } else {
+          // PM: 12:xx:xx stays 12:xx:xx, 1-11:xx:xx becomes 13-23:xx:xx.
+          if (hr != 12) {
+            hr += 12
+          }
+        }
+      }
+
+      val localTime = LocalTime.of(hr, min, sec, MICROSECONDS.toNanos(ms).toInt)
+      Some(localTimeToNanos(localTime))
+    } catch {
+      case NonFatal(_) => None
+    }
+  }
+
+  def stringToTimeAnsi(s: UTF8String, context: QueryContext = null): Long = {
+    stringToTime(s).getOrElse {
+      throw ExecutionErrors.invalidInputInCastToDatetimeError(s, TimeType(), context)
+    }
+  }
+
+  /**
+   * Returns the index of the first non-whitespace and non-ISO control character in the byte
+   * array.
+   *
+   * @param bytes
+   *   The byte array to be processed.
+   * @return
+   *   The start index after trimming.
    */
   @inline private def getTrimmedStart(bytes: Array[Byte]) = {
     var start = 0
@@ -649,9 +807,12 @@ trait SparkDateTimeUtils {
   /**
    * Returns the index of the last non-whitespace and non-ISO control character in the byte array.
    *
-   * @param start The starting index for the search.
-   * @param bytes The byte array to be processed.
-   * @return The end index after trimming.
+   * @param start
+   *   The starting index for the search.
+   * @param bytes
+   *   The byte array to be processed.
+   * @return
+   *   The end index after trimming.
    */
   @inline private def getTrimmedEnd(start: Int, bytes: Array[Byte]) = {
     var end = bytes.length - 1
