@@ -23,14 +23,12 @@ import scala.collection.mutable
 
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.AnalysisException
-import org.apache.spark.sql.catalyst.SqlScriptingContextManager
 import org.apache.spark.sql.catalyst.expressions._
 import org.apache.spark.sql.catalyst.expressions.SubExprUtils.wrapOuterReference
-import org.apache.spark.sql.catalyst.parser.SqlScriptingLabelContext.isForbiddenLabelName
 import org.apache.spark.sql.catalyst.plans.logical._
 import org.apache.spark.sql.catalyst.trees.CurrentOrigin.withOrigin
 import org.apache.spark.sql.catalyst.trees.TreePattern._
-import org.apache.spark.sql.connector.catalog.{CatalogManager, Identifier}
+import org.apache.spark.sql.connector.catalog.CatalogManager
 import org.apache.spark.sql.errors.{DataTypeErrorsBase, QueryCompilationErrors}
 import org.apache.spark.sql.internal.SQLConf
 
@@ -142,7 +140,8 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
           }
           matched(ordinal)
 
-        case u @ UnresolvedAttribute(nameParts) =>
+        case u @ UnresolvedAttribute(nameParts) if !u.containsTag(LogicalPlan.PLAN_ID_TAG) =>
+          // UnresolvedAttribute with PLAN_ID_TAG should be resolved in resolveDataFrameColumn
           val result = withPosition(u) {
             resolveColumnByName(nameParts)
               .orElse(LiteralFunctionResolution.resolve(nameParts))
@@ -169,13 +168,24 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
           }
         }
 
-        case u @ UnresolvedExtractValue(child, fieldName) =>
+        case u @ UnresolvedExtractValue(child, field) =>
           val newChild = innerResolve(child, isTopLevel = false)
-          if (newChild.resolved) {
-            ExtractValue(newChild, fieldName, resolver)
+          val resolvedField = if (conf.getConf(SQLConf.PREFER_COLUMN_OVER_LCA_IN_ARRAY_INDEX)) {
+            innerResolve(field, isTopLevel = false)
           } else {
-            u.copy(child = newChild)
+            field
           }
+          if (newChild.resolved) {
+            ExtractValue(child = newChild, extraction = resolvedField, resolver = resolver)
+          } else {
+            u.copy(child = newChild, extraction = resolvedField)
+          }
+
+        // Default value expression can not reference real columns,
+        // so we only need to resolve special literal columns like CURRENT_DATE.
+        // See {@link LiteralFunctionResolution} for details.
+        case d @ DefaultValueExpression(c: Expression, _, _) =>
+          d.copy(child = resolveLiteralColumns(c))
 
         case _ => e.mapChildren(innerResolve(_, isTopLevel = false))
       }
@@ -197,6 +207,13 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
     }
   }
 
+  private def resolveLiteralColumns(e: Expression) = {
+    e.transformWithPruning(_.containsPattern(UNRESOLVED_ATTRIBUTE)) {
+      case u @ UnresolvedAttribute(nameParts) =>
+        LiteralFunctionResolution.resolve(nameParts).getOrElse(u)
+    }
+  }
+
   // Resolves `UnresolvedAttribute` to `OuterReference`.
   protected def resolveOuterRef(e: Expression): Expression = {
     val outerPlan = AnalysisContext.get.outerPlan
@@ -235,102 +252,16 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
     }
   }
 
-  /**
-   * Look up variable by nameParts.
-   * If in SQL Script, first check local variables, unless in EXECUTE IMMEDIATE
-   * (EXECUTE IMMEDIATE generated query cannot access local variables).
-   * if not found fall back to session variables.
-   * @param nameParts NameParts of the variable.
-   * @return Reference to the variable.
-   */
-  def lookupVariable(nameParts: Seq[String]): Option[VariableReference] = {
-    // The temp variables live in `SYSTEM.SESSION`, and the name can be qualified or not.
-    def maybeTempVariableName(nameParts: Seq[String]): Boolean = {
-      nameParts.length == 1 || {
-        if (nameParts.length == 2) {
-          nameParts.head.equalsIgnoreCase(CatalogManager.SESSION_NAMESPACE)
-        } else if (nameParts.length == 3) {
-          nameParts(0).equalsIgnoreCase(CatalogManager.SYSTEM_CATALOG_NAME) &&
-            nameParts(1).equalsIgnoreCase(CatalogManager.SESSION_NAMESPACE)
-        } else {
-          false
-        }
-      }
-    }
-
-    val namePartsCaseAdjusted = if (conf.caseSensitiveAnalysis) {
-      nameParts
-    } else {
-      nameParts.map(_.toLowerCase(Locale.ROOT))
-    }
-
-    SqlScriptingContextManager.get().map(_.getVariableManager)
-      // If we are in EXECUTE IMMEDIATE lookup only session variables.
-      .filterNot(_ => AnalysisContext.get.isExecuteImmediate)
-      // If variable name is qualified with session.<varName> treat it as a session variable.
-      .filterNot(_ =>
-        nameParts.length > 2 || (nameParts.length == 2 && isForbiddenLabelName(nameParts.head)))
-      .flatMap(_.get(namePartsCaseAdjusted))
-      .map { varDef =>
-        VariableReference(
-          nameParts,
-          FakeLocalCatalog,
-          Identifier.of(Array(varDef.identifier.namespace().last), namePartsCaseAdjusted.last),
-          varDef)
-      }
-      .orElse(
-        if (maybeTempVariableName(nameParts)) {
-          catalogManager.tempVariableManager
-            .get(namePartsCaseAdjusted)
-            .map { varDef =>
-              VariableReference(
-                nameParts,
-                FakeSystemCatalog,
-                Identifier.of(Array(CatalogManager.SESSION_NAMESPACE), namePartsCaseAdjusted.last),
-                varDef
-              )}
-        } else {
-          None
-        }
-      )
-  }
-
   // Resolves `UnresolvedAttribute` to its value.
   protected def resolveVariables(e: Expression): Expression = {
-    def resolveVariable(nameParts: Seq[String]): Option[Expression] = {
-      val isResolvingView = AnalysisContext.get.catalogAndNamespace.nonEmpty
-      if (isResolvingView) {
-        if (AnalysisContext.get.referredTempVariableNames.contains(nameParts)) {
-          lookupVariable(nameParts)
-        } else {
-          None
-        }
-      } else {
-        lookupVariable(nameParts)
-      }
-    }
+    val variableResolution = new VariableResolution(catalogManager.tempVariableManager)
 
     def resolve(nameParts: Seq[String]): Option[Expression] = {
-      var resolvedVariable: Option[Expression] = None
-      // We only support temp variables for now, so the variable name can at most have 3 parts.
-      var numInnerFields: Int = math.max(0, nameParts.length - 3)
-      // Follow the column resolution and prefer the longest match. This makes sure that users
-      // can always use fully qualified variable name to avoid name conflicts.
-      while (resolvedVariable.isEmpty && numInnerFields < nameParts.length) {
-        resolvedVariable = resolveVariable(nameParts.dropRight(numInnerFields))
-        if (resolvedVariable.isEmpty) numInnerFields += 1
-      }
-
-      resolvedVariable.map { variable =>
-        if (numInnerFields != 0) {
-          val nestedFields = nameParts.takeRight(numInnerFields)
-          nestedFields.foldLeft(variable: Expression) { (e, name) =>
-            ExtractValue(e, Literal(name), conf.resolver)
-          }
-        } else {
-          variable
-        }
-      }.map(e => Alias(e, nameParts.last)())
+      variableResolution.resolveMultipartName(
+        nameParts = nameParts,
+        resolvingView = AnalysisContext.get.catalogAndNamespace.nonEmpty,
+        referredTempVariableNames = AnalysisContext.get.referredTempVariableNames
+      ).map(e => Alias(e, nameParts.last)())
     }
 
     def innerResolve(e: Expression, isTopLevel: Boolean): Expression = withOrigin(e.origin) {
@@ -495,7 +426,8 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
   def resolveExpressionByPlanChildren(
       e: Expression,
       q: LogicalPlan,
-      includeLastResort: Boolean = false): Expression = {
+      includeLastResort: Boolean = false,
+      throws: Boolean = true): Expression = {
     resolveExpression(
       tryResolveDataFrameColumns(e, q.children),
       resolveColumnByName = nameParts => {
@@ -505,7 +437,7 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
         assert(q.children.length == 1)
         q.children.head.output
       },
-      throws = true,
+      throws,
       includeLastResort = includeLastResort)
   }
 
@@ -518,7 +450,7 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
       u: UnresolvedAttribute,
       q: LogicalPlan,
       includeLastResort: Boolean = false): Option[Expression] = {
-    assert(u.getTagValue(LogicalPlan.PLAN_ID_TAG).nonEmpty,
+    assert(u.containsTag(LogicalPlan.PLAN_ID_TAG),
       s"UnresolvedAttribute $u should have a Plan Id tag")
 
     resolveDataFrameColumn(u, q.children).map { r =>
@@ -545,8 +477,14 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
     resolveVariables(resolveOuterRef(e))
   }
 
-  def resolveExprInAssignment(expr: Expression, hostPlan: LogicalPlan): Expression = {
-    resolveExpressionByPlanChildren(expr, hostPlan) match {
+  def resolveExprInAssignment(
+      expr: Expression,
+      hostPlan: LogicalPlan,
+      throws: Boolean = true): Expression = {
+    resolveExpressionByPlanChildren(expr,
+      hostPlan,
+      includeLastResort = false,
+      throws = throws) match {
       // Assignment key and value does not need the alias when resolving nested columns.
       case Alias(child: ExtractValue, _) => child
       case other => other
@@ -557,10 +495,14 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
   // expression are from Spark Connect, and need to be resolved in this way:
   //    1. extract the attached plan id from UnresolvedAttribute;
   //    2. top-down traverse the query plan to find the plan node that matches the plan id;
-  //    3. if can not find the matching node, fail the analysis due to illegal references;
-  //    4. if more than one matching nodes are found, fail due to ambiguous column reference;
-  //    5. resolve the expression with the matching node, if any error occurs here, return the
-  //       original expression as it is.
+  //    3. if can not find the matching node, fails with 'CANNOT_RESOLVE_DATAFRAME_COLUMN';
+  //    4, if the matching node is found, but can not resolve the column, return the original one;
+  //    5, resolve the expression against the target node, the resolved attribute will be
+  //       filtered by the output attributes of nodes in the path (from matching to root node);
+  //    6. if more than one resolved attributes are found in the above recursive process,
+  //       fails with 'AMBIGUOUS_COLUMN_REFERENCE'.
+  //    7. if all the resolved attributes are filtered out, return the original expression
+  //       as it is.
   private def tryResolveDataFrameColumns(
       e: Expression,
       q: Seq[LogicalPlan]): Expression = e match {
@@ -581,7 +523,7 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
     val planId = planIdOpt.get
     logDebug(s"Extract plan_id $planId from $u")
 
-    val isMetadataAccess = u.getTagValue(LogicalPlan.IS_METADATA_COL).nonEmpty
+    val isMetadataAccess = u.containsTag(LogicalPlan.IS_METADATA_COL)
 
     val (resolved, matched) = resolveDataFrameColumnByPlanId(
       u, planId, isMetadataAccess, q, 0)
@@ -622,19 +564,16 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
       p: LogicalPlan,
       currentDepth: Int): (Option[(NamedExpression, Int)], Boolean) = {
     val (resolved, matched) = if (p.getTagValue(LogicalPlan.PLAN_ID_TAG).contains(id)) {
-      val resolved = try {
-        if (!isMetadataAccess) {
-          p.resolve(u.nameParts, conf.resolver)
-        } else if (u.nameParts.size == 1) {
-          p.getMetadataAttributeByNameOpt(u.nameParts.head)
-        } else {
-          None
-        }
-      } catch {
-        case e: AnalysisException =>
-          logDebug(s"Fail to resolve $u with $p due to $e")
-          None
+      val resolved = if (!isMetadataAccess) {
+        p.resolve(u.nameParts, conf.resolver)
+      } else if (u.nameParts.size == 1) {
+        p.getMetadataAttributeByNameOpt(u.nameParts.head)
+      } else {
+        None
       }
+      // The targe plan node is found, but might still fail to resolve.
+      // In this case, return None to delay the failure, so it is possible to be
+      // resolved in the next iteration.
       (resolved.map(r => (r, currentDepth)), true)
     } else {
       val children = p match {
@@ -662,14 +601,20 @@ trait ColumnResolutionHelper extends Logging with DataTypeErrorsBase {
     // When resolving the column reference df1.a, the target node with plan_id=1
     // can be found in both sides of the Join node.
     // To correctly resolve df1.a, the analyzer discards the resolved attribute
-    // in the right side, by filtering out the result by the output attributes of
+    // on the right side, by filtering out the result by the output attributes of
     // Project plan_id=2.
     //
     // However, there are analyzer rules (e.g. ResolveReferencesInSort)
     // supporting missing column resolution. Then a valid resolved attribute
-    // maybe filtered out here. In this case, resolveDataFrameColumnByPlanId
-    // returns None, the dataframe column will remain unresolved, and the analyzer
-    // will try to resolve it without plan id later.
+    // maybe filtered out here. For example:
+    //
+    // from pyspark.sql import functions as sf
+    // df = spark.range(10).withColumn("v", sf.col("id") + 1)
+    // df.select(df.v).sort(df.id)
+    //
+    // In this case, resolveDataFrameColumnByPlanId returns None,
+    // the dataframe column 'df.id' will remain unresolved, and the analyzer
+    // will try to resolve 'id' without plan id later.
     val filtered = resolved.filter { r =>
       if (isMetadataAccess) {
         r._1.references.subsetOf(AttributeSet(p.output ++ p.metadataOutput))
