@@ -19,7 +19,11 @@ package org.apache.spark.sql.catalyst.analysis.resolver
 
 import java.util.Locale
 
-import org.apache.spark.sql.catalyst.{FunctionIdentifier, SQLConfHelper, SqlScriptingContextManager}
+import org.apache.spark.sql.catalyst.{
+  FunctionIdentifier,
+  SQLConfHelper,
+  SqlScriptingContextManager
+}
 import org.apache.spark.sql.catalyst.analysis.{
   FunctionRegistry,
   GetViewColumnByNameAndOrdinal,
@@ -143,6 +147,8 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
         checkRepartition(repartition)
       case having: UnresolvedHaving =>
         checkHaving(having)
+      case sample: Sample =>
+        checkSample(sample)
       case _ =>
         false
     }
@@ -168,8 +174,6 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
         checkUnresolvedCast(unresolvedCast)
       case unresolvedUpCast: UpCast =>
         checkUnresolvedUpCast(unresolvedUpCast)
-      case unresolvedStar: UnresolvedStar =>
-        checkUnresolvedStar(unresolvedStar)
       case unresolvedAlias: UnresolvedAlias =>
         checkUnresolvedAlias(unresolvedAlias)
       case unresolvedAttribute: UnresolvedAttribute =>
@@ -194,6 +198,8 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
         checkUnresolvedFunction(unresolvedFunction)
       case getViewColumnByNameAndOrdinal: GetViewColumnByNameAndOrdinal =>
         checkGetViewColumnBynameAndOrdinal(getViewColumnByNameAndOrdinal)
+      case semiStructuredExtract: SemiStructuredExtract =>
+        checkSemiStructuredExtract(semiStructuredExtract)
       case expression if isGenerallySupportedExpression(expression) =>
         expression.children.forall(checkExpression)
       case _ =>
@@ -219,13 +225,23 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
   }
 
   private def checkProject(project: Project) = {
-    checkOperator(project.child) && project.projectList.forall(checkExpression)
+    checkOperator(project.child) && project.projectList.forall {
+      case _: UnresolvedStar =>
+        true
+      case other =>
+        checkExpression(other)
+    }
   }
 
   private def checkAggregate(aggregate: Aggregate) = {
     checkOperator(aggregate.child) &&
     aggregate.groupingExpressions.forall(checkExpression) &&
-    aggregate.aggregateExpressions.forall(checkExpression)
+    aggregate.aggregateExpressions.forall {
+      case _: UnresolvedStar =>
+        true
+      case other =>
+        checkExpression(other)
+    }
   }
 
   private def checkJoin(join: Join) = {
@@ -267,7 +283,8 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
   private def checkUnresolvedInlineTable(unresolvedInlineTable: UnresolvedInlineTable) =
     unresolvedInlineTable.rows.forall(_.forall(checkExpression))
 
-  private def checkUnresolvedRelation(unresolvedRelation: UnresolvedRelation) = true
+  private def checkUnresolvedRelation(unresolvedRelation: UnresolvedRelation) =
+    !unresolvedRelation.isStreaming
 
   private def checkResolvedInlineTable(resolvedInlineTable: ResolvedInlineTable) =
     resolvedInlineTable.rows.forall(_.forall(checkExpression))
@@ -306,14 +323,12 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
 
   private def checkUnresolvedUpCast(upCast: UpCast) = checkExpression(upCast.child)
 
-  private def checkUnresolvedStar(unresolvedStar: UnresolvedStar) = true
-
   private def checkUnresolvedAlias(unresolvedAlias: UnresolvedAlias) =
     checkExpression(unresolvedAlias.child)
 
   private def checkUnresolvedAttribute(unresolvedAttribute: UnresolvedAttribute) =
     !ResolverGuard.UNSUPPORTED_ATTRIBUTE_NAMES.contains(unresolvedAttribute.nameParts.head) &&
-    !unresolvedAttribute.getTagValue(LogicalPlan.PLAN_ID_TAG).isDefined
+    !unresolvedAttribute.containsTag(LogicalPlan.PLAN_ID_TAG)
 
   private def checkUnresolvedPredicate(unresolvedPredicate: Predicate) = unresolvedPredicate match {
     case inSubquery: InSubquery =>
@@ -331,6 +346,7 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
   }
 
   private def checkUnresolvedFunction(unresolvedFunction: UnresolvedFunction) =
+    unresolvedFunction.nameParts.size == 1 &&
     !ResolverGuard.UNSUPPORTED_FUNCTION_NAMES.contains(unresolvedFunction.nameParts.head) &&
     // UDFs are not supported
     FunctionRegistry.functionSet.contains(
@@ -358,6 +374,9 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
   private def checkGetViewColumnBynameAndOrdinal(
       getViewColumnByNameAndOrdinal: GetViewColumnByNameAndOrdinal) = true
 
+  private def checkSemiStructuredExtract(semiStructuredExtract: SemiStructuredExtract) =
+    checkExpression(semiStructuredExtract.child)
+
   private def checkRepartition(repartition: Repartition) = {
     checkOperator(repartition.child)
   }
@@ -365,6 +384,10 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
   private def checkHaving(having: UnresolvedHaving) =
     checkExpression(having.havingCondition) && checkOperator(having.child)
 
+  private def checkSample(sample: Sample) = {
+    checkOperator(sample.child)
+  }
+
   /**
    * Most of the expressions come from resolving the [[UnresolvedFunction]], but here we have some
    * popular expressions allowlist for two reasons:
@@ -417,8 +440,8 @@ class ResolverGuard(catalogManager: CatalogManager) extends SQLConfHelper {
           _: RegExpCount | _: RegExpSubStr | _: RegExpInStr =>
         true
       // JSON
-      case _: JsonToStructs | _: StructsToJson | _: SchemaOfJson | _: JsonObjectKeys |
-          _: LengthOfJsonArray =>
+      case _: GetJsonObject | _: JsonTuple | _: JsonToStructs | _: StructsToJson |
+          _: SchemaOfJson | _: JsonObjectKeys | _: LengthOfJsonArray =>
         true
       // CSV
       case _: SchemaOfCsv | _: StructsToCsv | _: CsvToStructs =>
