@@ -22,17 +22,17 @@ import java.time.{ZoneId, ZoneOffset}
 import java.util.Date
 
 import scala.collection.mutable
+import scala.util.Try
 import scala.util.control.NonFatal
 
 import com.fasterxml.jackson.annotation.JsonInclude.Include
 import com.fasterxml.jackson.databind.{DeserializationFeature, ObjectMapper}
 import com.fasterxml.jackson.module.scala.{ClassTagExtensions, DefaultScalaModule}
-import org.apache.commons.lang3.StringUtils
 import org.json4s.JsonAST.{JArray, JBool, JDecimal, JDouble, JInt, JLong, JNull, JObject, JString, JValue}
 import org.json4s.jackson.JsonMethods._
 
 import org.apache.spark.SparkException
-import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.Logging
 import org.apache.spark.internal.LogKeys._
 import org.apache.spark.sql.AnalysisException
 import org.apache.spark.sql.catalyst.{CurrentUserContext, FunctionIdentifier, InternalRow, SQLConfHelper, TableIdentifier}
@@ -50,6 +50,7 @@ import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types._
 import org.apache.spark.sql.util.{CaseInsensitiveStringMap, SchemaUtils}
 import org.apache.spark.util.ArrayImplicits._
+import org.apache.spark.util.Utils
 
 /**
  * Interface providing util to convert JValue to String representation of catalog entities.
@@ -277,7 +278,7 @@ case class ClusterBySpec(columnNames: Seq[NamedReference]) {
 object ClusterBySpec {
   private val mapper = {
     val ret = new ObjectMapper() with ClassTagExtensions
-    ret.setSerializationInclusion(Include.NON_ABSENT)
+    ret.setDefaultPropertyInclusion(Include.NON_ABSENT)
     ret.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
     ret.registerModule(DefaultScalaModule)
     ret
@@ -450,7 +451,15 @@ case class CatalogTable(
    */
   def partitionSchema: StructType = {
     val partitionFields = schema.takeRight(partitionColumnNames.length)
-    assert(partitionFields.map(_.name) == partitionColumnNames)
+    val actualPartitionColumnNames = partitionFields.map(_.name)
+
+    assert(actualPartitionColumnNames == partitionColumnNames,
+      "Corrupted table metadata detected for table " + identifier.quotedString + ". " +
+      "The partition column names in the table schema " +
+      "do not match the declared partition columns. " +
+      "Table schema columns: [" + schema.fieldNames.mkString(", ") + "] " +
+      "Declared partition columns: [" + partitionColumnNames.mkString(", ") + "]. " +
+      "This indicates corrupted table metadata that needs to be repaired.")
 
     StructType(partitionFields)
   }
@@ -629,16 +638,6 @@ case class CatalogTable(
       if (lastAccessTime <= 0) JString("UNKNOWN")
       else JLong(lastAccessTime)
 
-    val viewQueryOutputColumns: JValue = {
-      if (viewSchemaMode == SchemaEvolution) {
-        JArray(schema.map(_.name).map(JString).toList)
-      } else if (viewQueryColumnNames.nonEmpty) {
-        JArray(viewQueryColumnNames.map(JString).toList)
-      } else {
-        JNull
-      }
-    }
-
     val map = mutable.LinkedHashMap[String, JValue]()
 
     if (identifier.catalog.isDefined) map += "Catalog" -> JString(identifier.catalog.get)
@@ -655,21 +654,35 @@ case class CatalogTable(
     }
     if (comment.isDefined) map += "Comment" -> JString(comment.get)
     if (collation.isDefined) map += "Collation" -> JString(collation.get)
-    if (tableType == CatalogTableType.VIEW && viewText.isDefined) {
-      map += "View Text" -> JString(viewText.get)
-    }
-    if (tableType == CatalogTableType.VIEW && viewOriginalText.isDefined) {
-      map += "View Original Text" -> JString(viewOriginalText.get)
-    }
-    if (SQLConf.get.viewSchemaBindingEnabled && tableType == CatalogTableType.VIEW) {
-      map += "View Schema Mode" -> JString(viewSchemaMode.toString)
-    }
-    if (viewCatalogAndNamespace.nonEmpty && tableType == CatalogTableType.VIEW) {
-      import org.apache.spark.sql.connector.catalog.CatalogV2Implicits._
-      map += "View Catalog and Namespace" -> JString(viewCatalogAndNamespace.quoted)
-    }
-    if (viewQueryOutputColumns != JNull) {
-      map += "View Query Output Columns" -> viewQueryOutputColumns
+
+    if (tableType == CatalogTableType.VIEW) {
+      if (viewText.isDefined) {
+        map += "View Text" -> JString(viewText.get)
+      }
+      if (viewOriginalText.isDefined) {
+        map += "View Original Text" -> JString(viewOriginalText.get)
+      }
+      if (SQLConf.get.viewSchemaBindingEnabled) {
+        val viewSchemaModeInfo = Try(viewSchemaMode.toString).getOrElse("UNKNOWN")
+        map += "View Schema Mode" -> JString(viewSchemaModeInfo)
+      }
+      val viewCatalogAndNamespaceInfos = Try(viewCatalogAndNamespace).getOrElse(Seq.empty)
+      if (viewCatalogAndNamespaceInfos.nonEmpty) {
+        import org.apache.spark.sql.connector.catalog.CatalogV2Implicits._
+        map += "View Catalog and Namespace" -> JString(viewCatalogAndNamespaceInfos.quoted)
+      }
+      val viewQueryOutputColumns: JValue = Try {
+        if (viewSchemaMode == SchemaEvolution) {
+          JArray(schema.map(_.name).map(JString).toList)
+        } else if (viewQueryColumnNames.nonEmpty) {
+          JArray(viewQueryColumnNames.map(JString).toList)
+        } else {
+          JNull
+        }
+      }.getOrElse(JNull)
+      if (viewQueryOutputColumns != JNull) {
+        map += "View Query Output Columns" -> viewQueryOutputColumns
+      }
     }
     if (tableProperties != JNull) map += "Table Properties" -> tableProperties
     stats.foreach { s =>
@@ -709,6 +722,18 @@ object CatalogTable {
 
   val VIEW_CATALOG_AND_NAMESPACE = VIEW_PREFIX + "catalogAndNamespace.numParts"
   val VIEW_CATALOG_AND_NAMESPACE_PART_PREFIX = VIEW_PREFIX + "catalogAndNamespace.part."
+
+  // Property to indicate that a VIEW is actually a METRIC VIEW
+  val VIEW_WITH_METRICS = VIEW_PREFIX + "viewWithMetrics"
+
+  /**
+   * Check if a CatalogTable is a metric view by looking at its properties.
+   */
+  def isMetricView(table: CatalogTable): Boolean = {
+    table.tableType == CatalogTableType.VIEW &&
+      table.properties.get(VIEW_WITH_METRICS).contains("true")
+  }
+
   // Convert the current catalog and namespace to properties.
   def catalogAndNamespaceToProps(
       currentCatalog: String,
@@ -760,20 +785,15 @@ object CatalogTable {
 
   def readLargeTableProp(props: Map[String, String], key: String): Option[String] = {
     props.get(key).orElse {
-      if (props.exists { case (mapKey, _) => mapKey.startsWith(key) }) {
-        props.get(s"$key.numParts") match {
-          case None => throw QueryCompilationErrors.insufficientTablePropertyError(key)
-          case Some(numParts) =>
-            val parts = (0 until numParts.toInt).map { index =>
-              val keyPart = s"$key.part.$index"
-              props.getOrElse(keyPart, {
-                throw QueryCompilationErrors.insufficientTablePropertyPartError(keyPart, numParts)
-              })
-            }
-            Some(parts.mkString)
+      // only construct the property from parts if numParts exist,
+      props.get(s"$key.numParts").map { numParts =>
+        val parts = (0 until numParts.toInt).map { index =>
+          val keyPart = s"$key.part.$index"
+          props.getOrElse(keyPart, {
+            throw QueryCompilationErrors.insufficientTablePropertyPartError(keyPart, numParts)
+          })
         }
-      } else {
-        None
+        parts.mkString
       }
     }
   }
@@ -1062,7 +1082,9 @@ case class UnresolvedCatalogRelation(
     tableMeta: CatalogTable,
     options: CaseInsensitiveStringMap = CaseInsensitiveStringMap.empty(),
     override val isStreaming: Boolean = false) extends UnresolvedLeafNode {
-  assert(tableMeta.identifier.database.isDefined)
+  assert(tableMeta.identifier.database.isDefined,
+    "Table identifier " + tableMeta.identifier.quotedString + " is missing database name. " +
+    "UnresolvedCatalogRelation requires a fully qualified table identifier with database.")
 }
 
 /**
@@ -1089,7 +1111,9 @@ case class HiveTableRelation(
     tableStats: Option[Statistics] = None,
     @transient prunedPartitions: Option[Seq[CatalogTablePartition]] = None)
   extends LeafNode with MultiInstanceRelation with NormalizeableRelation {
-  assert(tableMeta.identifier.database.isDefined)
+  assert(tableMeta.identifier.database.isDefined,
+    "Table identifier " + tableMeta.identifier.quotedString + " is missing database name. " +
+    "HiveTableRelation requires a fully qualified table identifier with database.")
   assert(DataTypeUtils.sameType(tableMeta.partitionSchema, partitionCols.toStructType))
   assert(DataTypeUtils.sameType(tableMeta.dataSchema, dataCols.toStructType))
 
@@ -1150,7 +1174,7 @@ case class HiveTableRelation(
     val metadataEntries = metadata.toSeq.map {
       case (key, value) if key == "CatalogTable" => value
       case (key, value) =>
-        key + ": " + StringUtils.abbreviate(value, SQLConf.get.maxMetadataStringLength)
+        key + ": " + Utils.abbreviate(value, SQLConf.get.maxMetadataStringLength)
     }
 
     val metadataStr = truncatedString(metadataEntries, "[", ", ", "]", maxFields)
