@@ -181,10 +181,25 @@ abstract class TreeNode[BaseType <: TreeNode[BaseType]]
     }
   }
 
+  def mergeTagsFrom(other: BaseType): Unit = {
+    if (!other.isTagsEmpty) {
+      // Merge all tags from the other node into this node.
+      // Unlike copyTagsFrom which only copies when this node has no tags,
+      // mergeTagsFrom will always merge tags regardless of existing state.
+      // If both nodes have the same tag with different values, the value
+      // from the other node will overwrite the existing value in this node.
+      tags ++= other.tags
+    }
+  }
+
   def setTagValue[T](tag: TreeNodeTag[T], value: T): Unit = {
     tags(tag) = value
   }
 
+  def containsTag[T](tag: TreeNodeTag[T]): Boolean = {
+    getTagValue[T](tag).isDefined
+  }
+
   def getTagValue[T](tag: TreeNodeTag[T]): Option[T] = {
     if (isTagsEmpty) {
       None
@@ -212,7 +227,7 @@ abstract class TreeNode[BaseType <: TreeNode[BaseType]]
     children.map(_.height).reduceOption(_ max _).getOrElse(0) + 1)
   def height: Int = _height()
 
-  private val _hashCode = new BestEffortLazyVal[Integer](() => MurmurHash3.productHash(this))
+  private val _hashCode = new BestEffortLazyVal[Integer](() => MurmurHash3.caseClassHash(this))
   override def hashCode(): Int = _hashCode()
 
   /**
