@@ -30,6 +30,9 @@ import scala.util.control.Exception._
 import scala.util.control.NonFatal
 import scala.xml.SAXException
 
+import org.apache.hadoop.hdfs.BlockMissingException
+import org.apache.hadoop.security.AccessControlException
+
 import org.apache.spark.SparkIllegalArgumentException
 import org.apache.spark.internal.Logging
 import org.apache.spark.rdd.RDD
@@ -40,8 +43,9 @@ import org.apache.spark.sql.catalyst.util.LegacyDateFormats.FAST_DATE_FORMAT
 import org.apache.spark.sql.errors.QueryExecutionErrors
 import org.apache.spark.sql.internal.{LegacyBehaviorPolicy, SQLConf}
 import org.apache.spark.sql.types._
+import org.apache.spark.util.SparkErrorUtils
 
-class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
+class XmlInferSchema(private val options: XmlOptions, private val caseSensitive: Boolean)
     extends Serializable
     with Logging {
 
@@ -69,7 +73,21 @@ class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
     legacyFormat = FAST_DATE_FORMAT,
     isParsing = true)
 
+  override def equals(obj: Any): Boolean = obj match {
+    case other: XmlInferSchema =>
+      options == other.options &&
+      caseSensitive == other.caseSensitive
+    case _ => false
+  }
+
+  override def hashCode(): Int = {
+    var result = options.hashCode()
+    result = 31 * result + (if (caseSensitive) 1 else 0)
+    result
+  }
+
   private def handleXmlErrorsByParseMode(
+      parser: XMLEventReader,
       parseMode: ParseMode,
       columnNameOfCorruptRecord: String,
       e: Throwable): Option[StructType] = {
@@ -79,7 +97,9 @@ class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
       case DropMalformedMode =>
         None
       case FailFastMode =>
-        throw QueryExecutionErrors.malformedRecordsDetectedInSchemaInferenceError(e)
+        parser.close()
+        throw QueryExecutionErrors.malformedRecordsDetectedInSchemaInferenceError(
+          e, columnNameOfCorruptRecord)
     }
   }
 
@@ -140,7 +160,7 @@ class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
       schema
     } catch {
       case e @ (_: XMLStreamException | _: MalformedInputException | _: SAXException) =>
-        handleXmlErrorsByParseMode(options.parseMode, options.columnNameOfCorruptRecord, e)
+        handleXmlErrorsByParseMode(parser, options.parseMode, options.columnNameOfCorruptRecord, e)
       case e: CharConversionException if options.charset.isEmpty =>
         val msg =
           """XML parser cannot handle a character in its input.
@@ -149,6 +169,7 @@ class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
         val wrappedCharException = new CharConversionException(msg)
         wrappedCharException.initCause(e)
         handleXmlErrorsByParseMode(
+          parser,
           options.parseMode,
           options.columnNameOfCorruptRecord,
           wrappedCharException)
@@ -156,11 +177,12 @@ class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
         logWarning("Skipped missing file", e)
         Some(StructType(Nil))
       case e: FileNotFoundException if !options.ignoreMissingFiles => throw e
+      case e @ (_ : AccessControlException | _ : BlockMissingException) => throw e
       case e @ (_: IOException | _: RuntimeException) if options.ignoreCorruptFiles =>
         logWarning("Skipped the rest of the content in the corrupted file", e)
         Some(StructType(Nil))
       case NonFatal(e) =>
-        handleXmlErrorsByParseMode(options.parseMode, options.columnNameOfCorruptRecord, e)
+        handleXmlErrorsByParseMode(parser, options.parseMode, options.columnNameOfCorruptRecord, e)
     } finally {
       if (parser != null) {
         parser.close()
@@ -168,6 +190,123 @@ class XmlInferSchema(options: XmlOptions, caseSensitive: Boolean)
     }
   }
 
+  def inferFromReaders(recordReader: RDD[StaxXMLRecordReader]): StructType = {
+    val sampledRecordReader = if (options.samplingRatio < 1.0) {
+      recordReader.sample(withReplacement = false, options.samplingRatio, 1)
+    } else {
+      recordReader
+    }
+    // perform schema inference on each row and merge afterwards
+    val mergedTypesFromPartitions = sampledRecordReader.mapPartitions { iter =>
+      iter.flatMap { xmlReader =>
+        infer(xmlReader)
+      }.reduceOption(compatibleType(caseSensitive, options.valueTag)).iterator
+    }
+
+    // Here we manually submit a fold-like Spark job, so that we can set the SQLConf when running
+    // the fold functions in the scheduler event loop thread.
+    val existingConf = SQLConf.get
+    var rootType: DataType = StructType(Nil)
+    val foldPartition = (iter: Iterator[DataType]) =>
+      iter.fold(StructType(Nil))(compatibleType(caseSensitive, options.valueTag))
+    val mergeResult = (index: Int, taskResult: DataType) => {
+      rootType = SQLConf.withExistingConf(existingConf) {
+        compatibleType(caseSensitive, options.valueTag)(rootType, taskResult)
+      }
+    }
+    recordReader.sparkContext.runJob(mergedTypesFromPartitions, foldPartition, mergeResult)
+
+    canonicalizeType(rootType) match {
+      case Some(st: StructType) => st
+      case _ =>
+        // canonicalizeType erases all empty structs, including the only one we want to keep
+        // XML shouldn't run into this line
+        StructType(Seq())
+    }
+  }
+
+  /**
+   * Infer the schema of the next XML record in the XML event stream.
+   * Note that the method will **NOT** close the XML event stream as there could have more XML
+   * records to parse. The StaxXMLRecordReader will automatically close the stream when there are
+   * no more XML records to parse.
+   */
+  def infer(parser: StaxXMLRecordReader): Option[DataType] = {
+    try {
+      if (!parser.skipToNextRecord()) {
+        return None
+      }
+
+      val rootAttributes = parser.nextEvent().asStartElement.getAttributes.asScala.toArray
+      val schema = Some(inferObject(parser, rootAttributes))
+      schema
+    } catch {
+      case e: CharConversionException if options.charset.isEmpty =>
+        val msg =
+          """XML parser cannot handle a character in its input.
+            |Specifying encoding as an input option explicitly might help to resolve the issue.
+            |""".stripMargin + e.getMessage
+        val wrappedCharException = new CharConversionException(msg)
+        wrappedCharException.initCause(e)
+        handleXmlErrorsByParseMode(
+          parser,
+          options.parseMode,
+          options.columnNameOfCorruptRecord,
+          wrappedCharException)
+      case e: FileNotFoundException if options.ignoreMissingFiles =>
+        logWarning("Skipped missing file", e)
+        parser.close()
+        Some(StructType(Nil))
+      case e: FileNotFoundException if !options.ignoreMissingFiles =>
+        parser.close()
+        throw e
+      case NonFatal(e) =>
+        SparkErrorUtils.getRootCause(e) match {
+          case _: XMLStreamException | _: MalformedInputException =>
+            logWarning("Malformed XML record found", e)
+            // Close the parser from the first malformed XML record
+            parser.close()
+            handleXmlErrorsByParseMode(
+              parser = parser,
+              parseMode = options.parseMode,
+              columnNameOfCorruptRecord = options.columnNameOfCorruptRecord,
+              e = e
+            )
+          case _: SAXException =>
+            // For XSD validation errors, don't close the parser as there might be more valid
+            // records to parse.
+            // Advance the parser so that the next record can be parsed.
+            parser.nextEvent()
+            handleXmlErrorsByParseMode(
+              parser = parser,
+              parseMode = options.parseMode,
+              columnNameOfCorruptRecord = options.columnNameOfCorruptRecord,
+              e = e
+            )
+          case _: AccessControlException | _: BlockMissingException =>
+            parser.close()
+            throw e
+          case _: IOException | _: RuntimeException | _: InternalError | _: AssertionError
+              if options.ignoreCorruptFiles =>
+            logWarning("Skipped the rest of the content in the corrupted file", e)
+            parser.close()
+            Some(StructType(Nil))
+          case _: IOException | _: RuntimeException | _: InternalError
+              if !options.ignoreCorruptFiles =>
+            parser.close()
+            throw e
+          case _ =>
+            logWarning("Failed to infer schema from XML record", e)
+            handleXmlErrorsByParseMode(
+              parser = parser,
+              parseMode = options.parseMode,
+              columnNameOfCorruptRecord = options.columnNameOfCorruptRecord,
+              e = e
+            )
+        }
+    }
+  }
+
   private def inferFrom(datum: String): DataType = {
     val value = if (datum != null && options.ignoreSurroundingSpaces) {
       datum.trim()
