@@ -834,88 +834,10 @@ class AnalysisErrorSuite extends AnalysisTest with DataTypeErrorsBase {
       """"explode(array(min(a)))", "explode(array(max(a)))"""" :: Nil
   )
 
-  errorConditionTest(
-    "EXEC IMMEDIATE - nested execute immediate not allowed",
-    CatalystSqlParser.parsePlan("EXECUTE IMMEDIATE 'EXECUTE IMMEDIATE \\\'SELECT 42\\\''"),
-    "NESTED_EXECUTE_IMMEDIATE",
-    Map(
-      "sqlString" -> "EXECUTE IMMEDIATE 'SELECT 42'"))
-
-  errorConditionTest(
-    "EXEC IMMEDIATE - both positional and named used",
-    CatalystSqlParser.parsePlan("EXECUTE IMMEDIATE 'SELECT 42 where ? = :first'" +
-      " USING 1, 2 as first"),
-    "INVALID_QUERY_MIXED_QUERY_PARAMETERS",
-    Map.empty)
-
-  test("EXEC IMMEDIATE - non string variable as sqlString parameter") {
-    val execImmediatePlan = ExecuteImmediateQuery(
-      Seq.empty,
-      scala.util.Right(UnresolvedAttribute("testVarA")),
-      Seq(UnresolvedAttribute("testVarA")))
-
-    assertAnalysisErrorCondition(
-      inputPlan = execImmediatePlan,
-      expectedErrorCondition = "INVALID_VARIABLE_TYPE_FOR_QUERY_EXECUTE_IMMEDIATE",
-      expectedMessageParameters = Map(
-        "varType" -> "\"INT\""
-      ))
-  }
-
-  test("EXEC IMMEDIATE - Null string as sqlString parameter") {
-    val execImmediatePlan = ExecuteImmediateQuery(
-      Seq.empty,
-      scala.util.Right(UnresolvedAttribute("testVarNull")),
-      Seq(UnresolvedAttribute("testVarNull")))
-
-    assertAnalysisErrorCondition(
-      inputPlan = execImmediatePlan,
-      expectedErrorCondition = "NULL_QUERY_STRING_EXECUTE_IMMEDIATE",
-      expectedMessageParameters = Map("varName" -> "`testVarNull`"))
-  }
 
 
-  test("EXEC IMMEDIATE - Unsupported expr for parameter") {
-    val execImmediatePlan: LogicalPlan = ExecuteImmediateQuery(
-      Seq(UnresolvedAttribute("testVarA"), NaNvl(Literal(1), Literal(1))),
-      scala.util.Left("SELECT ?"),
-      Seq.empty)
 
-    assertAnalysisErrorCondition(
-      inputPlan = execImmediatePlan,
-      expectedErrorCondition = "UNSUPPORTED_EXPR_FOR_PARAMETER",
-      expectedMessageParameters = Map(
-        "invalidExprSql" -> "\"nanvl(1, 1)\""
-      ))
-  }
 
-  test("EXEC IMMEDIATE - Name Parametrize query with non named parameters") {
-    val execImmediateSetVariablePlan = ExecuteImmediateQuery(
-      Seq(Literal(2), new Alias(UnresolvedAttribute("testVarA"), "first")(), Literal(3)),
-      scala.util.Left("SELECT :first"),
-      Seq.empty)
-
-    assertAnalysisErrorCondition(
-      inputPlan = execImmediateSetVariablePlan,
-      expectedErrorCondition = "ALL_PARAMETERS_MUST_BE_NAMED",
-      expectedMessageParameters = Map(
-        "exprs" -> "\"2\", \"3\""
-      ))
-  }
-
-  test("EXEC IMMEDIATE - INTO specified for COMMAND query") {
-    val execImmediateSetVariablePlan = ExecuteImmediateQuery(
-      Seq.empty,
-      scala.util.Left("SET VAR testVarA = 1"),
-      Seq(UnresolvedAttribute("testVarA")))
-
-    assertAnalysisErrorCondition(
-      inputPlan = execImmediateSetVariablePlan,
-      expectedErrorCondition = "INVALID_STATEMENT_FOR_EXECUTE_INTO",
-      expectedMessageParameters = Map(
-        "sqlString" -> "SET VAR TESTVARA = 1"
-      ))
-  }
 
   test("SPARK-6452 regression test") {
     // CheckAnalysis should throw AnalysisException when Aggregate contains missing attribute(s)
