@@ -155,7 +155,6 @@ case class CSVFileFormat() extends TextBasedFileFormat with DataSourceRegister {
   private def supportDataType(dataType: DataType, allowVariant: Boolean): Boolean = dataType match {
     case _: VariantType => allowVariant
 
-    case _: TimeType => false
     case _: AtomicType => true
 
     case udt: UserDefinedType[_] => supportDataType(udt.sqlType)
@@ -168,7 +167,7 @@ case class CSVFileFormat() extends TextBasedFileFormat with DataSourceRegister {
   private def getCsvOptions(
       sparkSession: SparkSession,
       options: Map[String, String]): CSVOptions = {
-    val conf = sqlConf(sparkSession)
+    val conf = getSqlConf(sparkSession)
     new CSVOptions(
       options,
       conf.csvColumnPruning,
