@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-package org.apache.spark.sql.execution.streaming.state
+package org.apache.spark.sql.execution.streaming.operators.stateful.join
 
 import java.util.Locale
 
@@ -24,15 +24,16 @@ import scala.annotation.tailrec
 import org.apache.hadoop.conf.Configuration
 
 import org.apache.spark.TaskContext
-import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.Logging
 import org.apache.spark.internal.LogKeys.{END_INDEX, START_INDEX, STATE_STORE_ID}
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.{Attribute, AttributeReference, Expression, JoinedRow, Literal, SafeProjection, SpecificInternalRow, UnsafeProjection, UnsafeRow}
 import org.apache.spark.sql.catalyst.types.DataTypeUtils.toAttributes
 import org.apache.spark.sql.execution.metric.SQLMetric
-import org.apache.spark.sql.execution.streaming.StatefulOperatorStateInfo
-import org.apache.spark.sql.execution.streaming.StatefulOpStateStoreCheckpointInfo
-import org.apache.spark.sql.execution.streaming.StreamingSymmetricHashJoinHelper._
+import org.apache.spark.sql.execution.streaming.operators.stateful.StatefulOperatorStateInfo
+import org.apache.spark.sql.execution.streaming.operators.stateful.StatefulOpStateStoreCheckpointInfo
+import org.apache.spark.sql.execution.streaming.operators.stateful.join.StreamingSymmetricHashJoinHelper._
+import org.apache.spark.sql.execution.streaming.state.{DropLastNFieldsStatePartitionKeyExtractor, KeyStateEncoderSpec, NoopStatePartitionKeyExtractor, NoPrefixKeyStateEncoderSpec, StatePartitionKeyExtractor, StateSchemaBroadcast, StateStore, StateStoreCheckpointInfo, StateStoreColFamilySchema, StateStoreConf, StateStoreErrors, StateStoreId, StateStoreMetrics, StateStoreProvider, StateStoreProviderId, SupportsFineGrainedReplay}
 import org.apache.spark.sql.types.{BooleanType, LongType, StructField, StructType}
 import org.apache.spark.util.NextIterator
 
@@ -58,6 +59,8 @@ import org.apache.spark.util.NextIterator
  *                                  store providers being used in this class. If true, Spark will
  *                                  take care of management for state store providers, e.g. running
  *                                  maintenance task for these providers.
+ * @param snapshotOptions       Options controlling snapshot-based state replay for the state data
+ *                              source reader.
  * @param joinStoreGenerator    The generator to create state store instances, re-using the same
  *                              instance when the join implementation uses virtual column families
  *                              for join version 3.
@@ -94,15 +97,20 @@ abstract class SymmetricHashJoinStateManager(
     stateFormatVersion: Int,
     skippedNullValueCount: Option[SQLMetric] = None,
     useStateStoreCoordinator: Boolean = true,
-    snapshotStartVersion: Option[Long] = None,
+    snapshotOptions: Option[SnapshotOptions] = None,
     joinStoreGenerator: JoinStateManagerStoreGenerator) extends Logging {
   import SymmetricHashJoinStateManager._
 
-  protected val keySchema = StructType(
+  private[streaming] val keySchema = StructType(
     joinKeys.zipWithIndex.map { case (k, i) => StructField(s"field$i", k.dataType, k.nullable) })
   protected val keyAttributes = toAttributes(keySchema)
-  protected val keyToNumValues = new KeyToNumValuesStore(stateFormatVersion)
-  protected val keyWithIndexToValue = new KeyWithIndexToValueStore(stateFormatVersion)
+
+  private[streaming] val keyToNumValues = new KeyToNumValuesStore(
+    stateFormatVersion,
+    snapshotOptions.map(_.getKeyToNumValuesHandlerOpts()))
+  private[streaming] val keyWithIndexToValue = new KeyWithIndexToValueStore(
+    stateFormatVersion,
+    snapshotOptions.map(_.getKeyWithIndexToValueHandlerOpts()))
 
   /*
   =====================================================
@@ -439,7 +447,7 @@ abstract class SymmetricHashJoinStateManager(
    * NOTE: this function is only intended for use in unit tests
    * to simulate null values.
    */
-  private[state] def updateNumValuesTestOnly(key: UnsafeRow, numValues: Long): Unit = {
+  private[streaming] def updateNumValuesTestOnly(key: UnsafeRow, numValues: Long): Unit = {
     keyToNumValues.put(key, numValues)
   }
 
@@ -455,7 +463,8 @@ abstract class SymmetricHashJoinStateManager(
   /** Helper trait for invoking common functionalities of a state store. */
   protected abstract class StateStoreHandler(
       stateStoreType: StateStoreType,
-      stateStoreCkptId: Option[String]) extends Logging {
+      stateStoreCkptId: Option[String],
+      handlerSnapshotOptions: Option[HandlerSnapshotOptions] = None) extends Logging {
     private var stateStoreProvider: StateStoreProvider = _
 
     /** StateStore that the subclasses of this class is going to operate on */
@@ -496,7 +505,7 @@ abstract class SymmetricHashJoinStateManager(
       }
       val storeProviderId = StateStoreProviderId(stateInfo.get, partitionId, storeName)
       val store = if (useStateStoreCoordinator) {
-        assert(snapshotStartVersion.isEmpty, "Should not use state store coordinator " +
+        assert(handlerSnapshotOptions.isEmpty, "Should not use state store coordinator " +
           "when reading state as data source.")
         joinStoreGenerator.getStore(
           storeProviderId, keySchema, valueSchema, NoPrefixKeyStateEncoderSpec(keySchema),
@@ -508,13 +517,19 @@ abstract class SymmetricHashJoinStateManager(
           storeProviderId, keySchema, valueSchema, NoPrefixKeyStateEncoderSpec(keySchema),
           useColumnFamilies = useVirtualColumnFamilies, storeConf, hadoopConf,
           useMultipleValuesPerKey = false, stateSchemaProvider = None)
-        if (snapshotStartVersion.isDefined) {
+        if (handlerSnapshotOptions.isDefined) {
           if (!stateStoreProvider.isInstanceOf[SupportsFineGrainedReplay]) {
             throw StateStoreErrors.stateStoreProviderDoesNotSupportFineGrainedReplay(
               stateStoreProvider.getClass.toString)
           }
+          val opts = handlerSnapshotOptions.get
           stateStoreProvider.asInstanceOf[SupportsFineGrainedReplay]
-            .replayStateFromSnapshot(snapshotStartVersion.get, stateInfo.get.storeVersion)
+            .replayStateFromSnapshot(
+              opts.snapshotVersion,
+              opts.endVersion,
+              readOnly = true,
+              opts.startStateStoreCkptId,
+              opts.endStateStoreCkptId)
         } else {
           stateStoreProvider.getStore(stateInfo.get.storeVersion, stateStoreCkptId)
         }
@@ -528,7 +543,7 @@ abstract class SymmetricHashJoinStateManager(
    * Helper class for representing data returned by [[KeyWithIndexToValueStore]].
    * Designed for object reuse.
    */
-  private[state] class KeyAndNumValues(var key: UnsafeRow = null, var numValue: Long = 0) {
+  private[join] class KeyAndNumValues(var key: UnsafeRow = null, var numValue: Long = 0) {
     def withNew(newKey: UnsafeRow, newNumValues: Long): this.type = {
       this.key = newKey
       this.numValue = newNumValues
@@ -538,9 +553,12 @@ abstract class SymmetricHashJoinStateManager(
 
 
   /** A wrapper around a [[StateStore]] that stores [key -> number of values]. */
-  protected class KeyToNumValuesStore(val stateFormatVersion: Int)
-    extends StateStoreHandler(KeyToNumValuesType, keyToNumValuesStateStoreCkptId) {
-
+  protected class KeyToNumValuesStore(
+      val stateFormatVersion: Int,
+      val handlerSnapshotOptions: Option[HandlerSnapshotOptions] = None)
+    extends StateStoreHandler(
+      KeyToNumValuesType, keyToNumValuesStateStoreCkptId, handlerSnapshotOptions) {
+SnapshotOptions
     private val useVirtualColumnFamilies = stateFormatVersion == 3
     private val longValueSchema = new StructType().add("value", "long")
     private val longToUnsafeRow = UnsafeProjection.create(longValueSchema)
@@ -595,7 +613,7 @@ abstract class SymmetricHashJoinStateManager(
    * Helper class for representing data returned by [[KeyWithIndexToValueStore]].
    * Designed for object reuse.
    */
-  private[state] class KeyWithIndexAndValue(
+  private[join] class KeyWithIndexAndValue(
     var key: UnsafeRow = null,
     var valueIndex: Long = -1,
     var value: UnsafeRow = null,
@@ -706,8 +724,11 @@ abstract class SymmetricHashJoinStateManager(
    * A wrapper around a [[StateStore]] that stores the mapping; the mapping depends on the
    * state format version - please refer implementations of [[KeyWithIndexToValueRowConverter]].
    */
-  protected class KeyWithIndexToValueStore(stateFormatVersion: Int)
-    extends StateStoreHandler(KeyWithIndexToValueType, keyWithIndexToValueStateStoreCkptId) {
+  protected class KeyWithIndexToValueStore(
+      stateFormatVersion: Int,
+      handlerSnapshotOptions: Option[HandlerSnapshotOptions] = None)
+    extends StateStoreHandler(
+      KeyWithIndexToValueType, keyWithIndexToValueStateStoreCkptId, handlerSnapshotOptions) {
 
     private val useVirtualColumnFamilies = stateFormatVersion == 3
     private val keyWithIndexExprs = keyAttributes :+ Literal(1L)
@@ -737,9 +758,9 @@ abstract class SymmetricHashJoinStateManager(
     if (useVirtualColumnFamilies) {
       stateStore.createColFamilyIfAbsent(
         colFamilyName,
-        keySchema,
+        keyWithIndexSchema,
         valueRowConverter.valueAttributes.toStructType,
-        NoPrefixKeyStateEncoderSpec(keySchema)
+        NoPrefixKeyStateEncoderSpec(keyWithIndexSchema)
       )
     }
 
@@ -847,11 +868,11 @@ class SymmetricHashJoinStateManagerV1(
     stateFormatVersion: Int,
     skippedNullValueCount: Option[SQLMetric] = None,
     useStateStoreCoordinator: Boolean = true,
-    snapshotStartVersion: Option[Long] = None,
+    snapshotOptions: Option[SnapshotOptions] = None,
     joinStoreGenerator: JoinStateManagerStoreGenerator) extends SymmetricHashJoinStateManager(
   joinSide, inputValueAttributes, joinKeys, stateInfo, storeConf, hadoopConf,
   partitionId, keyToNumValuesStateStoreCkptId, keyWithIndexToValueStateStoreCkptId,
-  stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotStartVersion,
+  stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotOptions,
   joinStoreGenerator) {
 
   /** Commit all the changes to all the state stores */
@@ -926,11 +947,11 @@ class SymmetricHashJoinStateManagerV2(
     stateFormatVersion: Int,
     skippedNullValueCount: Option[SQLMetric] = None,
     useStateStoreCoordinator: Boolean = true,
-    snapshotStartVersion: Option[Long] = None,
+    snapshotOptions: Option[SnapshotOptions] = None,
     joinStoreGenerator: JoinStateManagerStoreGenerator) extends SymmetricHashJoinStateManager(
   joinSide, inputValueAttributes, joinKeys, stateInfo, storeConf, hadoopConf,
   partitionId, keyToNumValuesStateStoreCkptId, keyWithIndexToValueStateStoreCkptId,
-  stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotStartVersion,
+  stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotOptions,
   joinStoreGenerator) {
 
   /** Commit all the changes to the state store */
@@ -1033,20 +1054,20 @@ object SymmetricHashJoinStateManager {
       stateFormatVersion: Int,
       skippedNullValueCount: Option[SQLMetric] = None,
       useStateStoreCoordinator: Boolean = true,
-      snapshotStartVersion: Option[Long] = None,
+      snapshotOptions: Option[SnapshotOptions] = None,
       joinStoreGenerator: JoinStateManagerStoreGenerator): SymmetricHashJoinStateManager = {
     if (stateFormatVersion == 3) {
       new SymmetricHashJoinStateManagerV2(
         joinSide, inputValueAttributes, joinKeys, stateInfo, storeConf, hadoopConf,
         partitionId, keyToNumValuesStateStoreCkptId, keyWithIndexToValueStateStoreCkptId,
-        stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotStartVersion,
+        stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotOptions,
         joinStoreGenerator
       )
     } else {
       new SymmetricHashJoinStateManagerV1(
         joinSide, inputValueAttributes, joinKeys, stateInfo, storeConf, hadoopConf,
         partitionId, keyToNumValuesStateStoreCkptId, keyWithIndexToValueStateStoreCkptId,
-        stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotStartVersion,
+        stateFormatVersion, skippedNullValueCount, useStateStoreCoordinator, snapshotOptions,
         joinStoreGenerator
       )
     }
@@ -1134,17 +1155,17 @@ object SymmetricHashJoinStateManager {
     val ckptIds = joinCkptInfo.left.keyToNumValues.stateStoreCkptId.map(
       Array(
         _,
-        joinCkptInfo.left.valueToNumKeys.stateStoreCkptId.get,
+        joinCkptInfo.left.keyWithIndexToValue.stateStoreCkptId.get,
         joinCkptInfo.right.keyToNumValues.stateStoreCkptId.get,
-        joinCkptInfo.right.valueToNumKeys.stateStoreCkptId.get
+        joinCkptInfo.right.keyWithIndexToValue.stateStoreCkptId.get
       )
     )
     val baseCkptIds = joinCkptInfo.left.keyToNumValues.baseStateStoreCkptId.map(
       Array(
         _,
-        joinCkptInfo.left.valueToNumKeys.baseStateStoreCkptId.get,
+        joinCkptInfo.left.keyWithIndexToValue.baseStateStoreCkptId.get,
         joinCkptInfo.right.keyToNumValues.baseStateStoreCkptId.get,
-        joinCkptInfo.right.valueToNumKeys.baseStateStoreCkptId.get
+        joinCkptInfo.right.keyWithIndexToValue.baseStateStoreCkptId.get
       )
     )
 
@@ -1157,52 +1178,129 @@ object SymmetricHashJoinStateManager {
 
   /**
    * Stream-stream join has 4 state stores instead of one. So it will generate 4 different
-   * checkpoint IDs. They are translated from each joiners' state store into an array through
-   * mergeStateStoreCheckpointInfo(). This function is used to read it back into individual state
-   * store checkpoint IDs.
-   * @param partitionId
-   * @param stateInfo
-   * @return
+   * checkpoint IDs using stateStoreCkptIds. They are translated from each joiners' state
+   * store into an array through mergeStateStoreCheckpointInfo(). This function is used to read
+   * it back into individual state store checkpoint IDs for each store.
+   * If useColumnFamiliesForJoins is true, then it will always return the first checkpoint ID.
+   *
+   * @param partitionId the partition ID of the state store
+   * @param stateStoreCkptIds the array of checkpoint IDs for all the state stores
+   * @param useColumnFamiliesForJoins whether virtual column families are used for the join
+   *
+   * @return the checkpoint IDs for all state stores used by this joiner
    */
   def getStateStoreCheckpointIds(
       partitionId: Int,
-      stateInfo: StatefulOperatorStateInfo,
+      stateStoreCkptIds: Option[Array[Array[String]]],
       useColumnFamiliesForJoins: Boolean): JoinStateStoreCheckpointId = {
     if (useColumnFamiliesForJoins) {
-      val ckpt = stateInfo.stateStoreCkptIds.map(_(partitionId)).map(_.head)
+      val ckpt = stateStoreCkptIds.map(_(partitionId)).map(_.head)
       JoinStateStoreCheckpointId(
-        left = JoinerStateStoreCheckpointId(keyToNumValues = ckpt, valueToNumKeys = ckpt),
-        right = JoinerStateStoreCheckpointId(keyToNumValues = ckpt, valueToNumKeys = ckpt)
+        left = JoinerStateStoreCheckpointId(keyToNumValues = ckpt, keyWithIndexToValue = ckpt),
+        right = JoinerStateStoreCheckpointId(keyToNumValues = ckpt, keyWithIndexToValue = ckpt)
       )
     } else {
-      val stateStoreCkptIds = stateInfo.stateStoreCkptIds
+      val stateStoreCkptIdsOpt = stateStoreCkptIds
         .map(_(partitionId))
         .map(_.map(Option(_)))
         .getOrElse(Array.fill[Option[String]](4)(None))
       JoinStateStoreCheckpointId(
         left = JoinerStateStoreCheckpointId(
-          keyToNumValues = stateStoreCkptIds(0),
-          valueToNumKeys = stateStoreCkptIds(1)),
+          keyToNumValues = stateStoreCkptIdsOpt(0),
+          keyWithIndexToValue = stateStoreCkptIdsOpt(1)),
         right = JoinerStateStoreCheckpointId(
-          keyToNumValues = stateStoreCkptIds(2),
-          valueToNumKeys = stateStoreCkptIds(3)))
+          keyToNumValues = stateStoreCkptIdsOpt(2),
+          keyWithIndexToValue = stateStoreCkptIdsOpt(3)))
     }
   }
 
-  private[state] sealed trait StateStoreType
+  /**
+   * Stream-stream join has 4 state stores instead of one. So it will generate 4 different
+   * checkpoint IDs when not using virtual column families.
+   * This function is used to get the checkpoint ID for a specific state store
+   * by the name of the store, partition ID and the stateStoreCkptIds array. The expected names
+   * for the stores are generated by getStateStoreName().
+   * If useColumnFamiliesForJoins is true, then it will always return the first checkpoint ID.
+   *
+   * @param storeName the name of the state store
+   * @param partitionId the partition ID of the state store
+   * @param stateStoreCkptIds the array of checkpoint IDs for all the state stores
+   * @param useColumnFamiliesForJoins whether virtual column families are used for the join
+   *
+   * @return the checkpoint ID for the specific state store, or None if not found
+   */
+  def getStateStoreCheckpointId(
+      storeName: String,
+      partitionId: Int,
+      stateStoreCkptIds: Option[Array[Array[String]]],
+      useColumnFamiliesForJoins: Boolean = false) : Option[String] = {
+    if (useColumnFamiliesForJoins || storeName == StateStoreId.DEFAULT_STORE_NAME) {
+      stateStoreCkptIds.map(_(partitionId)).map(_.head)
+    } else {
+      val joinStateStoreCkptIds = getStateStoreCheckpointIds(
+        partitionId, stateStoreCkptIds, useColumnFamiliesForJoins)
 
-  private[state] case object KeyToNumValuesType extends StateStoreType {
+      if (storeName == getStateStoreName(LeftSide, KeyToNumValuesType)) {
+        joinStateStoreCkptIds.left.keyToNumValues
+      } else if (storeName == getStateStoreName(RightSide, KeyToNumValuesType)) {
+        joinStateStoreCkptIds.right.keyToNumValues
+      } else if (storeName == getStateStoreName(LeftSide, KeyWithIndexToValueType)) {
+        joinStateStoreCkptIds.left.keyWithIndexToValue
+      } else if (storeName == getStateStoreName(RightSide, KeyWithIndexToValueType)) {
+        joinStateStoreCkptIds.right.keyWithIndexToValue
+      } else {
+        None
+      }
+    }
+  }
+
+  private[streaming] sealed trait StateStoreType
+
+  private[sql] case object KeyToNumValuesType extends StateStoreType {
     override def toString(): String = "keyToNumValues"
   }
 
-  private[state] case object KeyWithIndexToValueType extends StateStoreType {
+  private[sql] case object KeyWithIndexToValueType extends StateStoreType {
     override def toString(): String = "keyWithIndexToValue"
   }
 
-  private[state] def getStateStoreName(joinSide: JoinSide, storeType: StateStoreType): String = {
+  private[streaming] def getStateStoreName(
+      joinSide: JoinSide, storeType: StateStoreType): String = {
     s"$joinSide-$storeType"
   }
 
+  private[join] def getStoreType(storeName: String): StateStoreType = {
+    if (storeName == getStateStoreName(LeftSide, KeyToNumValuesType) ||
+      storeName == getStateStoreName(RightSide, KeyToNumValuesType)) {
+      KeyToNumValuesType
+    } else if (storeName == getStateStoreName(LeftSide, KeyWithIndexToValueType) ||
+      storeName == getStateStoreName(RightSide, KeyWithIndexToValueType)) {
+      KeyWithIndexToValueType
+    } else {
+      throw new IllegalArgumentException(s"Unknown join store name: $storeName")
+    }
+  }
+
+  /**
+   * Returns the partition key extractor for the given join store and column family name.
+   */
+  def createPartitionKeyExtractor(
+      storeName: String,
+      colFamilyName: String,
+      stateKeySchema: StructType,
+      stateFormatVersion: Int): StatePartitionKeyExtractor = {
+    assert(stateFormatVersion <= 3, "State format version must be less than or equal to 3")
+    val name = if (stateFormatVersion == 3) colFamilyName else storeName
+    if (getStoreType(name) == KeyWithIndexToValueType) {
+      // For KeyWithIndex, the index is added to the join (i.e. partition) key.
+      // Drop the last field (index) to get the partition key
+      new DropLastNFieldsStatePartitionKeyExtractor(stateKeySchema, numLastColsToDrop = 1)
+    } else {
+      // State key is the partition key
+      new NoopStatePartitionKeyExtractor(stateKeySchema)
+    }
+  }
+
   /** Helper class for representing data (value, matched). */
   case class ValueAndMatchPair(value: UnsafeRow, matched: Boolean)
 
@@ -1234,3 +1332,36 @@ object SymmetricHashJoinStateManager {
     }
   }
 }
+
+/**
+ * Options controlling snapshot-based state replay for state data source reader.
+ */
+case class SnapshotOptions(
+    snapshotVersion: Long,
+    endVersion: Long,
+    startKeyToNumValuesStateStoreCkptId: Option[String] = None,
+    startKeyWithIndexToValueStateStoreCkptId: Option[String] = None,
+    endKeyToNumValuesStateStoreCkptId: Option[String] = None,
+    endKeyWithIndexToValueStateStoreCkptId: Option[String] = None) {
+
+  def getKeyToNumValuesHandlerOpts(): HandlerSnapshotOptions =
+    HandlerSnapshotOptions(
+      snapshotVersion = snapshotVersion,
+      endVersion = endVersion,
+      startStateStoreCkptId = startKeyToNumValuesStateStoreCkptId,
+      endStateStoreCkptId = endKeyToNumValuesStateStoreCkptId)
+
+  def getKeyWithIndexToValueHandlerOpts(): HandlerSnapshotOptions =
+    HandlerSnapshotOptions(
+      snapshotVersion = snapshotVersion,
+      endVersion = endVersion,
+      startStateStoreCkptId = startKeyWithIndexToValueStateStoreCkptId,
+      endStateStoreCkptId = endKeyWithIndexToValueStateStoreCkptId)
+}
+
+/** Snapshot options specialized for a single state store handler. */
+private[join] case class HandlerSnapshotOptions(
+    snapshotVersion: Long,
+    endVersion: Long,
+    startStateStoreCkptId: Option[String],
+    endStateStoreCkptId: Option[String])
