@@ -750,7 +750,6 @@ SELECT foo3_2e1(
 
 -- 3.3 Create and invoke function with different SQL configurations
 SET spark.sql.ansi.enabled=true;
-DROP FUNCTION IF EXISTS foo3_3at;
 CREATE FUNCTION foo3_3a(x INT) RETURNS DOUBLE RETURN 1 / x;
 CREATE FUNCTION foo3_3at(x INT) RETURNS TABLE (a DOUBLE) RETURN SELECT 1 / x;
 CREATE TEMPORARY FUNCTION foo3_3b(x INT) RETURNS DOUBLE RETURN 1 / x;
@@ -788,6 +787,74 @@ SELECT * FROM foo3_3ct();
 SELECT * FROM foo3_3dt();
 RESET spark.sql.ansi.enabled;
 
+-- 3.4 Cyclic function reference
+CREATE FUNCTION foo3_4a(x INT) RETURNS INT RETURN 1;
+CREATE FUNCTION foo3_4b(x INT) RETURNS INT RETURN foo3_4a(x);
+CREATE FUNCTION foo3_4c(x INT) RETURNS INT RETURN foo3_4b(x);
+CREATE FUNCTION foo3_4d(x INT) RETURNS INT RETURN (SELECT foo3_4c(x));
+CREATE FUNCTION foo3_4e(x INT) RETURNS TABLE(a INT) RETURN SELECT foo3_4a(x);
+CREATE FUNCTION foo3_4f(x INT) RETURNS TABLE(b INT) RETURN SELECT * FROM foo3_4e(x);
+CREATE OR REPLACE TEMP FUNCTION foo3_4g(x INT) RETURN x + 1;
+CREATE OR REPLACE TEMP FUNCTION foo3_4h(x INT) RETURN foo3_4g(x) + 1;
+-- Expect error
+CREATE OR REPLACE FUNCTION foo3_4a(x INT) RETURNS INT RETURN foo3_4b(x);
+CREATE OR REPLACE FUNCTION foo3_4a(x INT) RETURNS INT RETURN foo3_4c(x);
+CREATE OR REPLACE FUNCTION foo3_4a(x INT) RETURNS INT RETURN foo3_4d(x);
+CREATE OR REPLACE FUNCTION foo3_4a(x INT) RETURN FoO3_4b(x);
+CREATE OR REPLACE FUNCTION foo3_4a(x INT) RETURNS INT RETURN SELECT SUM(a) FROM foo3_4e(x);
+CREATE OR REPLACE FUNCTION foo3_4e(x INT) RETURNS TABLE (c INT) RETURN SELECT * FROM foo3_4f(x);
+CREATE OR REPLACE FUNCTION foo3_4e(x INT) RETURNS TABLE RETURN SELECT * FROM fOo3_4F(x);
+CREATE OR REPLACE TEMP FUNCTION foo3_4g(x INT) RETURN foo3_4h(x) + 1;
+
+-- 3.5 Permanent functions with temp objects are not allowed
+CREATE TEMPORARY VIEW t AS VALUES (0) t(a);
+CREATE TEMPORARY FUNCTION foo3_5a(x INT) RETURNS INT RETURN x;
+CREATE TEMPORARY FUNCTION foo3_5b(x INT) RETURNS INT RETURN (SELECT SUM(a) FROM t);
+CREATE TEMPORARY FUNCTION foo3_5c(x INT) RETURNS TABLE (a INT) RETURN SELECT a FROM t;
+-- Expect error: permanent function cannot reference temporary function
+CREATE FUNCTION foo3_5d(x INT) RETURNS INT RETURN foo3_5a(x);
+CREATE FUNCTION foo3_5d(x INT) RETURNS TABLE (a INT) RETURN SELECT foo3_5a(x);
+-- Expect error: permanent function cannot reference temporary view
+CREATE FUNCTION foo3_5d(x INT) RETURNS INT RETURN (SELECT SUM(a) FROM t);
+CREATE FUNCTION foo3_5d(x INT) RETURNS TABLE (a INT) RETURN SELECT a FROM t;
+
+-- 3.12 SQL data access routine
+-- Scalar functions
+CREATE FUNCTION foo3_12a(x INT) RETURNS INT CONTAINS SQL RETURN x;
+CREATE FUNCTION foo3_12b(x INT) RETURNS INT READS SQL DATA RETURN x;
+CREATE FUNCTION foo3_12c(x INT) RETURNS INT CONTAINS SQL RETURN (SELECT x);
+CREATE FUNCTION foo3_12d(x INT) RETURNS INT CONTAINS SQL RETURN (SELECT COUNT(*) FROM range(0, 3));
+CREATE FUNCTION foo3_12e(x INT) RETURNS INT CONTAINS SQL RETURN (SELECT COUNT(*) FROM VALUES (0, 1));
+-- Expect error: cannot create a SQL function with CONTAINS SQL that accesses SQL data
+CREATE FUNCTION foo3_12f(x INT) RETURNS INT CONTAINS SQL RETURN SELECT SUM(c2) FROM t1;
+CREATE FUNCTION foo3_12f(x INT) RETURNS INT CONTAINS SQL RETURN foo3_12b(x) + 1;
+CREATE FUNCTION foo3_12f(x INT) RETURNS INT CONTAINS SQL RETURN 1 + (SELECT SUM(c2) FROM t1);
+
+-- Table functions
+CREATE FUNCTION foo3_12g(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT x, x + 1;
+CREATE FUNCTION foo3_12h(x INT) RETURNS TABLE (a INT, b INT) READS SQL DATA RETURN SELECT x, x + 1;
+CREATE FUNCTION foo3_12i(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT * FROM VALUES (0, 1);
+CREATE FUNCTION foo3_12j(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT id, id + 1 FROM RANGE(3);
+-- Derived SQL data access: READS SQL DATA.
+CREATE FUNCTION foo3_12k(x INT) RETURNS TABLE (a INT, b INT) RETURN SELECT c1, c2 FROM t1;
+-- Expect error: cannot create a SQL function with CONTAINS SQL that accesses SQL data
+CREATE FUNCTION foo3_12f(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT c1, c2 FROM t1;
+CREATE FUNCTION foo3_12f(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT foo3_12b(x), x;
+CREATE FUNCTION foo3_12f(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT * FROM foo3_12h(x);
+CREATE FUNCTION foo3_12f(x INT) RETURNS TABLE (a INT, b INT) CONTAINS SQL RETURN SELECT * FROM foo3_12k(x);
+
+CREATE TABLE tbl USING PARQUET AS SELECT c1 FROM t2;
+CREATE FUNCTION foo3_12f(x INT) RETURNS INT CONTAINS SQL RETURN SELECT SUM(c1) FROM tbl;
+CREATE FUNCTION foo3_12f(x INT) RETURNS TABLE (a INT) CONTAINS SQL RETURN SELECT * FROM tbl;
+DROP TABLE tbl;
+
+-- 3.14 Invalid usage of SQL scalar/table functions in query clauses.
+CREATE FUNCTION foo3_14a() RETURNS INT RETURN 1;
+CREATE FUNCTION foo3_14b() RETURNS TABLE (a INT) RETURN SELECT 1;
+-- Expect error
+SELECT * FROM foo3_14a();
+SELECT foo3_14b();
+
 -- 4. SQL table functions
 CREATE FUNCTION foo4_0() RETURNS TABLE (x INT) RETURN SELECT 1;
 CREATE FUNCTION foo4_1(x INT) RETURNS TABLE (a INT) RETURN SELECT x;
@@ -824,3 +891,145 @@ SELECT (SELECT MAX(cnt) FROM foo4_3(c1)) FROM t1;
 -- Clean up
 DROP VIEW t1;
 DROP VIEW t2;
+
+DROP FUNCTION IF EXISTS foo1a0;
+DROP FUNCTION IF EXISTS foo1a1;
+DROP FUNCTION IF EXISTS foo1a2;
+DROP FUNCTION IF EXISTS foo1b0;
+DROP FUNCTION IF EXISTS foo1b1;
+DROP FUNCTION IF EXISTS foo1b2;
+DROP FUNCTION IF EXISTS foo1c1;
+DROP FUNCTION IF EXISTS foo1c2;
+DROP FUNCTION IF EXISTS foo1d1;
+DROP FUNCTION IF EXISTS foo1d2;
+DROP FUNCTION IF EXISTS foo1d4;
+DROP FUNCTION IF EXISTS foo1d5;
+DROP FUNCTION IF EXISTS foo1d6;
+DROP FUNCTION IF EXISTS foo1e1;
+DROP FUNCTION IF EXISTS foo1e2;
+DROP FUNCTION IF EXISTS foo1e3;
+DROP FUNCTION IF EXISTS foo1f1;
+DROP FUNCTION IF EXISTS foo1f2;
+DROP FUNCTION IF EXISTS foo1g1;
+DROP FUNCTION IF EXISTS foo1g2;
+DROP FUNCTION IF EXISTS foo2a0;
+DROP FUNCTION IF EXISTS foo2a2;
+DROP FUNCTION IF EXISTS foo2a4;
+DROP FUNCTION IF EXISTS foo2b1;
+DROP FUNCTION IF EXISTS foo2b2;
+DROP FUNCTION IF EXISTS foo2c1;
+DROP FUNCTION IF EXISTS foo31;
+DROP FUNCTION IF EXISTS foo32;
+DROP FUNCTION IF EXISTS foo33;
+DROP FUNCTION IF EXISTS foo41;
+DROP FUNCTION IF EXISTS foo42;
+DROP FUNCTION IF EXISTS foo51;
+DROP FUNCTION IF EXISTS foo52;
+DROP FUNCTION IF EXISTS foo6c;
+DROP FUNCTION IF EXISTS foo6d;
+DROP FUNCTION IF EXISTS foo7a;
+DROP FUNCTION IF EXISTS foo7at;
+DROP FUNCTION IF EXISTS foo9a;
+DROP FUNCTION IF EXISTS foo9b;
+DROP FUNCTION IF EXISTS foo9c;
+DROP FUNCTION IF EXISTS foo9d;
+DROP FUNCTION IF EXISTS foo9e;
+DROP FUNCTION IF EXISTS foo9f;
+DROP FUNCTION IF EXISTS foo9g;
+DROP FUNCTION IF EXISTS foo9h;
+DROP FUNCTION IF EXISTS foo9i;
+DROP FUNCTION IF EXISTS foo9j;
+DROP FUNCTION IF EXISTS foo9l;
+DROP FUNCTION IF EXISTS foo9m;
+DROP FUNCTION IF EXISTS foo9n;
+DROP FUNCTION IF EXISTS foo9o;
+DROP FUNCTION IF EXISTS foo9p;
+DROP FUNCTION IF EXISTS foo9q;
+DROP FUNCTION IF EXISTS foo9r;
+DROP FUNCTION IF EXISTS foo1_10;
+DROP FUNCTION IF EXISTS bar1_10;
+DROP FUNCTION IF EXISTS foo1_11a;
+DROP FUNCTION IF EXISTS foo1_11b;
+DROP FUNCTION IF EXISTS foo1_11c;
+DROP FUNCTION IF EXISTS foo1_11d;
+DROP FUNCTION IF EXISTS foo1_11e;
+DROP FUNCTION IF EXISTS foo1_11f;
+DROP FUNCTION IF EXISTS foo1_11g;
+DROP FUNCTION IF EXISTS foo1_11h;
+DROP FUNCTION IF EXISTS foo1_11i;
+DROP FUNCTION IF EXISTS foo1_11j;
+DROP FUNCTION IF EXISTS foo1_11k;
+DROP FUNCTION IF EXISTS foo1_11l;
+DROP FUNCTION IF EXISTS foo1_11m;
+DROP FUNCTION IF EXISTS foo1_11n;
+DROP FUNCTION IF EXISTS foo1_11o;
+DROP FUNCTION IF EXISTS foo2_1a;
+DROP FUNCTION IF EXISTS foo2_1b;
+DROP FUNCTION IF EXISTS foo2_1c;
+DROP FUNCTION IF EXISTS foo2_1d;
+DROP FUNCTION IF EXISTS foo2_2a;
+DROP FUNCTION IF EXISTS foo2_2b;
+DROP FUNCTION IF EXISTS foo2_2c;
+DROP FUNCTION IF EXISTS foo2_2d;
+DROP FUNCTION IF EXISTS foo2_2e;
+DROP FUNCTION IF EXISTS foo2_2f;
+DROP FUNCTION IF EXISTS foo2_2g;
+DROP FUNCTION IF EXISTS foo2_3;
+DROP FUNCTION IF EXISTS foo2_4a;
+DROP FUNCTION IF EXISTS foo2_4b;
+DROP FUNCTION IF EXISTS foo3_1a;
+DROP FUNCTION IF EXISTS foo3_1b;
+DROP FUNCTION IF EXISTS foo3_1c;
+DROP FUNCTION IF EXISTS foo3_1d;
+DROP FUNCTION IF EXISTS foo3_1e;
+DROP FUNCTION IF EXISTS foo3_1f;
+DROP FUNCTION IF EXISTS foo3_1g;
+DROP FUNCTION IF EXISTS foo3_1x;
+DROP FUNCTION IF EXISTS foo3_1y;
+DROP FUNCTION IF EXISTS foo3_2a;
+DROP FUNCTION IF EXISTS foo3_2b1;
+DROP FUNCTION IF EXISTS foo3_2b2;
+DROP FUNCTION IF EXISTS foo3_2b3;
+DROP FUNCTION IF EXISTS foo3_2b4;
+DROP FUNCTION IF EXISTS foo3_2b5;
+DROP FUNCTION IF EXISTS foo3_2c1;
+DROP FUNCTION IF EXISTS foo3_2c2;
+DROP FUNCTION IF EXISTS foo3_2d1;
+DROP FUNCTION IF EXISTS foo3_2d2;
+DROP FUNCTION IF EXISTS foo3_2e1;
+DROP FUNCTION IF EXISTS foo3_3a;
+DROP FUNCTION IF EXISTS foo3_3at;
+DROP FUNCTION IF EXISTS foo3_14a;
+DROP FUNCTION IF EXISTS foo3_14b;
+DROP FUNCTION IF EXISTS foo3_3c;
+DROP FUNCTION IF EXISTS foo3_3ct;
+DROP FUNCTION IF EXISTS foo3_3d;
+DROP FUNCTION IF EXISTS foo3_3dt;
+DROP FUNCTION IF EXISTS foo3_4a;
+DROP FUNCTION IF EXISTS foo3_4b;
+DROP FUNCTION IF EXISTS foo3_4c;
+DROP FUNCTION IF EXISTS foo3_4d;
+DROP FUNCTION IF EXISTS foo3_4e;
+DROP FUNCTION IF EXISTS foo3_4f;
+DROP FUNCTION IF EXISTS foo3_12a;
+DROP FUNCTION IF EXISTS foo3_12b;
+DROP FUNCTION IF EXISTS foo3_12c;
+DROP FUNCTION IF EXISTS foo3_12d;
+DROP FUNCTION IF EXISTS foo3_12e;
+DROP FUNCTION IF EXISTS foo3_12f;
+DROP FUNCTION IF EXISTS foo3_12g;
+DROP FUNCTION IF EXISTS foo3_12h;
+DROP FUNCTION IF EXISTS foo3_12i;
+DROP FUNCTION IF EXISTS foo3_12j;
+DROP FUNCTION IF EXISTS foo3_12k;
+DROP FUNCTION IF EXISTS foo4_0;
+DROP FUNCTION IF EXISTS foo4_1;
+DROP FUNCTION IF EXISTS foo4_2;
+DROP FUNCTION IF EXISTS foo4_3;
+
+-- Drop temporary functions
+DROP TEMPORARY FUNCTION IF EXISTS foo1d3;
+DROP TEMPORARY FUNCTION IF EXISTS foo3_3b;
+DROP TEMPORARY FUNCTION IF EXISTS foo3_5a;
+DROP TEMPORARY FUNCTION IF EXISTS foo3_5b;
+DROP TEMPORARY FUNCTION IF EXISTS foo3_5c;
