@@ -26,8 +26,8 @@ import scala.util.Try
 import org.scalatest.BeforeAndAfter
 
 import org.apache.spark.SparkException
-import org.apache.spark.sql.{AnalysisException, DataFrame, QueryTest, SaveMode}
-import org.apache.spark.sql.catalyst.analysis.{NoSuchTableException, TableAlreadyExistsException}
+import org.apache.spark.sql.{AnalysisException, DataFrame, Dataset, QueryTest, SaveMode}
+import org.apache.spark.sql.catalyst.analysis.{AsOfTimestamp, AsOfVersion, NoSuchTableException, TableAlreadyExistsException, TimeTravelSpec}
 import org.apache.spark.sql.catalyst.plans.logical.{AppendData, LogicalPlan, OverwriteByExpression}
 import org.apache.spark.sql.catalyst.util.DateTimeUtils
 import org.apache.spark.sql.connector.catalog.{CatalogV2Util, Column, Identifier, InMemoryTableCatalog, SupportsCatalogOptions, TableCatalog}
@@ -52,6 +52,10 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
     spark.sessionState.catalogManager.catalog(name).asInstanceOf[TableCatalog]
   }
 
+  protected def sessionCatalog: InMemoryTableSessionCatalog = {
+    catalog(SESSION_CATALOG_NAME).asInstanceOf[InMemoryTableSessionCatalog]
+  }
+
   private implicit def stringToIdentifier(value: String): Identifier = {
     Identifier.of(Array.empty, value)
   }
@@ -65,7 +69,8 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
 
   override def afterEach(): Unit = {
     super.afterEach()
-    Try(catalog(SESSION_CATALOG_NAME).asInstanceOf[InMemoryTableSessionCatalog].clearTables())
+    Try(sessionCatalog.checkUsage())
+    Try(sessionCatalog.clearTables())
     catalog(catalogName).listTables(Array.empty).foreach(
       catalog(catalogName).dropTable(_))
     spark.conf.unset(V2_SESSION_CATALOG_IMPLEMENTATION.key)
@@ -146,7 +151,7 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
     val dfw = df.write.format(format).mode(SaveMode.Ignore).option("name", "t1")
     dfw.save()
 
-    val table = catalog(SESSION_CATALOG_NAME).loadTable(Identifier.of(Array("default"), "t1"))
+    val table = sessionCatalog.loadTable(Identifier.of(Array("default"), "t1"))
     assert(table.partitioning().isEmpty, "Partitioning should be empty")
     assert(table.columns() sameElements
       Array(Column.create("id", LongType)), "Schema did not match")
@@ -221,7 +226,8 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
     val e = intercept[NoSuchTableException] {
       spark.read.format(format).option("name", "non_existent_table").load()
     }
-    checkErrorTableNotFound(e, "`default`.`non_existent_table`")
+    val currentCatalogName = spark.catalog.currentCatalog().name()
+    checkErrorTableNotFound(e, s"`$currentCatalogName`.`default`.`non_existent_table`")
   }
 
   test("DataFrameWriter creates v2Relation with identifiers") {
@@ -304,8 +310,13 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
         .mode(SaveMode.Overwrite).save()
 
       // load with version
-      checkAnswer(load("t", Some(catalogName), version = Some("Snapshot123456789")), df1.toDF())
-      checkAnswer(load("t", Some(catalogName), version = Some("2345678910")), df2.toDF())
+      val readDF1 = load("t", Some(catalogName), version = Some("Snapshot123456789"))
+      checkAnswer(readDF1, df1.toDF())
+      checkTimeTravel(readDF1, expectedTimeTravelSpec = AsOfVersion("Snapshot123456789"))
+
+      val readDF2 = load("t", Some(catalogName), version = Some("2345678910"))
+      checkAnswer(readDF2, df2.toDF())
+      checkTimeTravel(readDF2, expectedTimeTravelSpec = AsOfVersion("2345678910"))
     }
 
     val ts1 = DateTimeUtils.stringToTimestampAnsi(
@@ -329,16 +340,26 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
         .mode(SaveMode.Overwrite).save()
 
       // load with timestamp
-      checkAnswer(load("t", Some(catalogName), version = None,
-        timestamp = Some("2019-01-29 00:37:58")), df3.toDF())
-      checkAnswer(load("t", Some(catalogName), version = None,
-        timestamp = Some("2021-01-29 00:37:58")), df4.toDF())
+      val readDF3 = load("t", Some(catalogName), version = None,
+        timestamp = Some("2019-01-29 00:37:58"))
+      checkAnswer(readDF3, df3.toDF())
+      checkTimeTravel(readDF3, expectedTimeTravelSpec = AsOfTimestamp(ts1))
+
+      val readDF4 = load("t", Some(catalogName), version = None,
+        timestamp = Some("2021-01-29 00:37:58"))
+      checkAnswer(readDF4, df4.toDF())
+      checkTimeTravel(readDF4, expectedTimeTravelSpec = AsOfTimestamp(ts2))
 
       // load with timestamp in number format
-      checkAnswer(load("t", Some(catalogName), version = None,
-        timestamp = Some(MICROSECONDS.toSeconds(ts1).toString)), df3.toDF())
-      checkAnswer(load("t", Some(catalogName), version = None,
-        timestamp = Some(MICROSECONDS.toSeconds(ts2).toString)), df4.toDF())
+      val readDF5 = load("t", Some(catalogName), version = None,
+        timestamp = Some(MICROSECONDS.toSeconds(ts1).toString))
+      checkAnswer(readDF5, df3.toDF())
+      checkTimeTravel(readDF5, expectedTimeTravelSpec = AsOfTimestamp(ts1))
+
+      val readDF6 = load("t", Some(catalogName), version = None,
+        timestamp = Some(MICROSECONDS.toSeconds(ts2).toString))
+      checkAnswer(readDF6, df4.toDF())
+      checkTimeTravel(readDF6, expectedTimeTravelSpec = AsOfTimestamp(ts2))
     }
 
     val e = intercept[AnalysisException] {
@@ -359,6 +380,11 @@ class SupportsCatalogOptionsSuite extends QueryTest with SharedSparkSession with
     assert(v2.catalog.exists(_ == catalogPlugin))
   }
 
+  private def checkTimeTravel(ds: Dataset[_], expectedTimeTravelSpec: TimeTravelSpec): Unit = {
+    val relation = ds.logicalPlan.asInstanceOf[DataSourceV2Relation]
+    assert(relation.timeTravelSpec.contains(expectedTimeTravelSpec))
+  }
+
   private def load(
       name: String,
       catalogOpt: Option[String],
