@@ -102,15 +102,97 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
     properties.setProperty("password", "testPass")
 
     conn = DriverManager.getConnection(url, properties)
-    conn.prepareStatement("create schema test").executeUpdate()
-    conn.prepareStatement(
-      "create table test.people (name TEXT(32) NOT NULL, theid INTEGER NOT NULL)").executeUpdate()
-    conn.prepareStatement("insert into test.people values ('fred', 1)").executeUpdate()
-    conn.prepareStatement("insert into test.people values ('mary', 2)").executeUpdate()
-    conn.prepareStatement(
-      "insert into test.people values ('joe ''foo'' \"bar\"', 3)").executeUpdate()
-    conn.commit()
+    val batchStmt = conn.createStatement()
 
+    batchStmt.addBatch("create schema test")
+
+    batchStmt.addBatch("create table test.people (name TEXT(32) NOT NULL, " +
+      "theid INTEGER NOT NULL)")
+    batchStmt.addBatch("insert into test.people values ('fred', 1)")
+    batchStmt.addBatch("insert into test.people values ('mary', 2)")
+    batchStmt.addBatch("insert into test.people values ('joe ''foo'' \"bar\"', 3)")
+
+    batchStmt.addBatch("create table test.inttypes (a INT, b BOOLEAN, c TINYINT, " +
+      "d SMALLINT, e BIGINT)")
+    batchStmt.addBatch("insert into test.inttypes values (1, false, 3, 4, 1234567890123)")
+    batchStmt.addBatch("insert into test.inttypes values (null, null, null, null, null)")
+
+    batchStmt.addBatch("create table test.timetypes (a TIME, b DATE, c TIMESTAMP(7))")
+    batchStmt.addBatch("insert into test.timetypes values " +
+      "('12:34:56', '1996-01-01', '2002-02-20 11:22:33.543543543')")
+    batchStmt.addBatch("insert into test.timetypes values " +
+      "('12:34:56', null, '2002-02-20 11:22:33.543543543')")
+
+    batchStmt.addBatch("CREATE TABLE test.timezone (tz TIMESTAMP WITH TIME ZONE) " +
+      "AS SELECT '1999-01-08 04:05:06.543543543-08:00'")
+
+    batchStmt.addBatch("CREATE TABLE test.array_table (ar Integer ARRAY) " +
+      "AS SELECT ARRAY[1, 2, 3]")
+
+    batchStmt.addBatch("create table test.flttypes (a DOUBLE, b REAL, c DECIMAL(38, 18))")
+    batchStmt.addBatch("insert into test.flttypes values " +
+      "(1.0000000000000002220446049250313080847263336181640625, " +
+      "1.00000011920928955078125, 123456789012345.543215432154321)")
+
+    batchStmt.addBatch("create table test.nulltypes (a INT, b BOOLEAN, c TINYINT, " +
+      "d BINARY(20), e VARCHAR(20), f VARCHAR_IGNORECASE(20), g CHAR(20), h BLOB, i CLOB, " +
+      "j TIME, k DATE, l TIMESTAMP, m DOUBLE, n REAL, o DECIMAL(38, 18))")
+    batchStmt.addBatch("insert into test.nulltypes values " +
+      "(null, null, null, null, null, null, null, null, null, null, null, null, null, null, null)")
+
+    batchStmt.addBatch("create table test.emp(name TEXT(32) NOT NULL, theid INTEGER, " +
+      "\"Dept\" INTEGER)")
+    batchStmt.addBatch("insert into test.emp values ('fred', 1, 10)")
+    batchStmt.addBatch("insert into test.emp values ('mary', 2, null)")
+    batchStmt.addBatch("insert into test.emp values ('joe ''foo'' \"bar\"', 3, 30)")
+    batchStmt.addBatch("insert into test.emp values ('kathy', null, null)")
+
+    batchStmt.addBatch("create table test.seq(id INTEGER)")
+    (0 to 6).foreach { value =>
+      batchStmt.addBatch(s"insert into test.seq values ($value)")
+    }
+    batchStmt.addBatch("insert into test.seq values (null)")
+
+    batchStmt.addBatch("create table test.\"mixedCaseCols\" (\"Name\" TEXT(32), " +
+      "\"Id\" INTEGER NOT NULL)")
+    batchStmt.addBatch("""insert into test."mixedCaseCols" values ('fred', 1)""")
+    batchStmt.addBatch("""insert into test."mixedCaseCols" values ('mary', 2)""")
+    batchStmt.addBatch("""insert into test."mixedCaseCols" values (null, 3)""")
+
+    batchStmt.addBatch("CREATE TABLE test.partition (THEID INTEGER, `THE ID` INTEGER) " +
+      "AS SELECT 1, 1")
+
+    batchStmt.addBatch("CREATE TABLE test.datetime (d DATE, t TIMESTAMP)")
+    batchStmt.addBatch("INSERT INTO test.datetime VALUES " +
+      "('2018-07-06', '2018-07-06 05:50:00.0')")
+    batchStmt.addBatch("INSERT INTO test.datetime VALUES " +
+      "('2018-07-06', '2018-07-06 08:10:08.0')")
+    batchStmt.addBatch("INSERT INTO test.datetime VALUES " +
+      "('2018-07-08', '2018-07-08 13:32:01.0')")
+    batchStmt.addBatch("INSERT INTO test.datetime VALUES " +
+      "('2018-07-12', '2018-07-12 09:51:15.0')")
+
+    batchStmt.addBatch("CREATE TABLE test.composite_name (`last name` TEXT(32) NOT NULL, " +
+      "id INTEGER NOT NULL)")
+    batchStmt.addBatch("INSERT INTO test.composite_name VALUES ('smith', 1)")
+    batchStmt.addBatch("INSERT INTO test.composite_name VALUES ('jones', 2)")
+
+    batchStmt.executeBatch()
+
+    conn
+      .prepareStatement("create table test.strtypes" +
+        "(a BINARY(20), b VARCHAR(20), c VARCHAR_IGNORECASE(20), d CHAR(20), e BLOB, f CLOB)")
+      .executeUpdate()
+    val strtypesStmt = conn.prepareStatement("insert into test.strtypes values (?, ?, ?, ?, ?, ?)")
+    strtypesStmt.setBytes(1, testBytes)
+    strtypesStmt.setString(2, "Sensitive")
+    strtypesStmt.setString(3, "Insensitive")
+    strtypesStmt.setString(4, "Twenty-byte CHAR")
+    strtypesStmt.setBytes(5, testBytes)
+    strtypesStmt.setString(6, "I am a clob!")
+    strtypesStmt.executeUpdate()
+
+    // Spark SQL views creation
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW foobar
@@ -143,13 +225,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |         upperBound '9223372036854775807', numPartitions '3')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement("create table test.inttypes (a INT, b BOOLEAN, c TINYINT, "
-      + "d SMALLINT, e BIGINT)").executeUpdate()
-    conn.prepareStatement("insert into test.inttypes values (1, false, 3, 4, 1234567890123)"
-        ).executeUpdate()
-    conn.prepareStatement("insert into test.inttypes values (null, null, null, null, null)"
-        ).executeUpdate()
-    conn.commit()
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW inttypes
@@ -157,16 +232,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |OPTIONS (url '$url', dbtable 'TEST.INTTYPES', user 'testUser', password 'testPass')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement("create table test.strtypes (a BINARY(20), b VARCHAR(20), "
-      + "c VARCHAR_IGNORECASE(20), d CHAR(20), e BLOB, f CLOB)").executeUpdate()
-    val stmt = conn.prepareStatement("insert into test.strtypes values (?, ?, ?, ?, ?, ?)")
-    stmt.setBytes(1, testBytes)
-    stmt.setString(2, "Sensitive")
-    stmt.setString(3, "Insensitive")
-    stmt.setString(4, "Twenty-byte CHAR")
-    stmt.setBytes(5, testBytes)
-    stmt.setString(6, "I am a clob!")
-    stmt.executeUpdate()
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW strtypes
@@ -174,13 +239,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |OPTIONS (url '$url', dbtable 'TEST.STRTYPES', user 'testUser', password 'testPass')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement("create table test.timetypes (a TIME, b DATE, c TIMESTAMP(7))"
-        ).executeUpdate()
-    conn.prepareStatement("insert into test.timetypes values ('12:34:56', "
-      + "'1996-01-01', '2002-02-20 11:22:33.543543543')").executeUpdate()
-    conn.prepareStatement("insert into test.timetypes values ('12:34:56', "
-      + "null, '2002-02-20 11:22:33.543543543')").executeUpdate()
-    conn.commit()
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW timetypes
@@ -188,23 +246,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |OPTIONS (url '$url', dbtable 'TEST.TIMETYPES', user 'testUser', password 'testPass')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement("CREATE TABLE test.timezone (tz TIMESTAMP WITH TIME ZONE) " +
-      "AS SELECT '1999-01-08 04:05:06.543543543-08:00'")
-      .executeUpdate()
-    conn.commit()
-
-    conn.prepareStatement("CREATE TABLE test.array_table (ar Integer ARRAY) " +
-      "AS SELECT ARRAY[1, 2, 3]")
-      .executeUpdate()
-    conn.commit()
-
-    conn.prepareStatement("create table test.flttypes (a DOUBLE, b REAL, c DECIMAL(38, 18))"
-        ).executeUpdate()
-    conn.prepareStatement("insert into test.flttypes values ("
-      + "1.0000000000000002220446049250313080847263336181640625, "
-      + "1.00000011920928955078125, "
-      + "123456789012345.543215432154321)").executeUpdate()
-    conn.commit()
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW flttypes
@@ -212,16 +253,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |OPTIONS (url '$url', dbtable 'TEST.FLTTYPES', user 'testUser', password 'testPass')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement(
-      s"""
-        |create table test.nulltypes (a INT, b BOOLEAN, c TINYINT, d BINARY(20), e VARCHAR(20),
-        |f VARCHAR_IGNORECASE(20), g CHAR(20), h BLOB, i CLOB, j TIME, k DATE, l TIMESTAMP,
-        |m DOUBLE, n REAL, o DECIMAL(38, 18))
-       """.stripMargin.replaceAll("\n", " ")).executeUpdate()
-    conn.prepareStatement("insert into test.nulltypes values ("
-      + "null, null, null, null, null, null, null, null, null, "
-      + "null, null, null, null, null, null)").executeUpdate()
-    conn.commit()
     sql(
       s"""
          |CREATE OR REPLACE TEMPORARY VIEW nulltypes
@@ -229,29 +260,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
          |OPTIONS (url '$url', dbtable 'TEST.NULLTYPES', user 'testUser', password 'testPass')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement(
-      "create table test.emp(name TEXT(32) NOT NULL," +
-        " theid INTEGER, \"Dept\" INTEGER)").executeUpdate()
-    conn.prepareStatement(
-      "insert into test.emp values ('fred', 1, 10)").executeUpdate()
-    conn.prepareStatement(
-      "insert into test.emp values ('mary', 2, null)").executeUpdate()
-    conn.prepareStatement(
-      "insert into test.emp values ('joe ''foo'' \"bar\"', 3, 30)").executeUpdate()
-    conn.prepareStatement(
-      "insert into test.emp values ('kathy', null, null)").executeUpdate()
-    conn.commit()
-
-    conn.prepareStatement(
-      "create table test.seq(id INTEGER)").executeUpdate()
-    (0 to 6).foreach { value =>
-      conn.prepareStatement(
-        s"insert into test.seq values ($value)").executeUpdate()
-    }
-    conn.prepareStatement(
-      "insert into test.seq values (null)").executeUpdate()
-    conn.commit()
-
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW nullparts
@@ -260,14 +268,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |partitionColumn '"Dept"', lowerBound '1', upperBound '4', numPartitions '3')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement(
-      """create table test."mixedCaseCols" ("Name" TEXT(32), "Id" INTEGER NOT NULL)""")
-      .executeUpdate()
-    conn.prepareStatement("""insert into test."mixedCaseCols" values ('fred', 1)""").executeUpdate()
-    conn.prepareStatement("""insert into test."mixedCaseCols" values ('mary', 2)""").executeUpdate()
-    conn.prepareStatement("""insert into test."mixedCaseCols" values (null, 3)""").executeUpdate()
-    conn.commit()
-
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW mixedCaseCols
@@ -275,29 +275,6 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         |OPTIONS (url '$url', dbtable 'TEST."mixedCaseCols"', user 'testUser', password 'testPass')
        """.stripMargin.replaceAll("\n", " "))
 
-    conn.prepareStatement("CREATE TABLE test.partition (THEID INTEGER, `THE ID` INTEGER) " +
-      "AS SELECT 1, 1")
-      .executeUpdate()
-    conn.commit()
-
-    conn.prepareStatement("CREATE TABLE test.datetime (d DATE, t TIMESTAMP)").executeUpdate()
-    conn.prepareStatement(
-      "INSERT INTO test.datetime VALUES ('2018-07-06', '2018-07-06 05:50:00.0')").executeUpdate()
-    conn.prepareStatement(
-      "INSERT INTO test.datetime VALUES ('2018-07-06', '2018-07-06 08:10:08.0')").executeUpdate()
-    conn.prepareStatement(
-      "INSERT INTO test.datetime VALUES ('2018-07-08', '2018-07-08 13:32:01.0')").executeUpdate()
-    conn.prepareStatement(
-      "INSERT INTO test.datetime VALUES ('2018-07-12', '2018-07-12 09:51:15.0')").executeUpdate()
-    conn.commit()
-
-    conn.prepareStatement(
-      "CREATE TABLE test.composite_name (`last name` TEXT(32) NOT NULL, id INTEGER NOT NULL)")
-      .executeUpdate()
-    conn.prepareStatement("INSERT INTO test.composite_name VALUES ('smith', 1)").executeUpdate()
-    conn.prepareStatement("INSERT INTO test.composite_name VALUES ('jones', 2)").executeUpdate()
-    conn.commit()
-
     sql(
       s"""
         |CREATE OR REPLACE TEMPORARY VIEW composite_name
@@ -803,17 +780,23 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
   }
 
   test("quote column names by jdbc dialect") {
-    val MySQL = JdbcDialects.get("jdbc:mysql://127.0.0.1/db")
-    val Postgres = JdbcDialects.get("jdbc:postgresql://127.0.0.1/db")
-    val Derby = JdbcDialects.get("jdbc:derby:db")
+    val mySQLDialect = JdbcDialects.get("jdbc:mysql://127.0.0.1/db")
+    val postgresDialect = JdbcDialects.get("jdbc:postgresql://127.0.0.1/db")
+    val derbyDialect = JdbcDialects.get("jdbc:derby:db")
+    val oracleDialect = JdbcDialects.get("jdbc:oracle:thin:@//localhost:1521/orcl")
+    val databricksDialect = JdbcDialects.get("jdbc:databricks://host/db")
 
-    val columns = Seq("abc", "key")
-    val MySQLColumns = columns.map(MySQL.quoteIdentifier(_))
-    val PostgresColumns = columns.map(Postgres.quoteIdentifier(_))
-    val DerbyColumns = columns.map(Derby.quoteIdentifier(_))
-    assert(MySQLColumns === Seq("`abc`", "`key`"))
-    assert(PostgresColumns === Seq(""""abc"""", """"key""""))
-    assert(DerbyColumns === Seq(""""abc"""", """"key""""))
+    val columns = Seq("abc", "key", "double_quote\"", "back`")
+    val mySQLColumns = columns.map(mySQLDialect.quoteIdentifier)
+    val postgresColumns = columns.map(postgresDialect.quoteIdentifier)
+    val derbyColumns = columns.map(derbyDialect.quoteIdentifier)
+    val oracleColumns = columns.map(oracleDialect.quoteIdentifier)
+    val databricksColumns = columns.map(databricksDialect.quoteIdentifier)
+    assertResult(Seq("`abc`", "`key`", "`double_quote\"`", "`back```"))(mySQLColumns)
+    assertResult(Seq("\"abc\"", "\"key\"", "\"double_quote\"\"\"", "\"back`\""))(postgresColumns)
+    assertResult(Seq("\"abc\"", "\"key\"", "\"double_quote\"\"\"", "\"back`\""))(derbyColumns)
+    assertResult(Seq("\"abc\"", "\"key\"", "\"double_quote\"\"\"", "\"back`\""))(oracleColumns)
+    assertResult(Seq("`abc`", "`key`", "`double_quote\"`", "`back```"))(databricksColumns)
   }
 
   test("compile filters") {
@@ -1710,7 +1693,7 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
 
       val quotedPrtColName = testH2Dialect.quoteIdentifier(expectedColumnName)
       df.logicalPlan match {
-        case LogicalRelationWithTable(JDBCRelation(_, parts, _), _) =>
+        case LogicalRelationWithTable(JDBCRelation(_, parts, _, _), _) =>
           val whereClauses = parts.map(_.asInstanceOf[JDBCPartition].whereClause).toSet
           assert(whereClauses === Set(
             s"$quotedPrtColName < 2 or $quotedPrtColName is null",
@@ -1852,7 +1835,7 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
       .load()
 
     df1.logicalPlan match {
-      case LogicalRelationWithTable(JDBCRelation(_, parts, _), _) =>
+      case LogicalRelationWithTable(JDBCRelation(_, parts, _, _), _) =>
         val whereClauses = parts.map(_.asInstanceOf[JDBCPartition].whereClause).toSet
         assert(whereClauses === Set(
           """"D" < '2018-07-11' or "D" is null""",
@@ -1872,7 +1855,7 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
       .load()
 
     df2.logicalPlan match {
-      case LogicalRelationWithTable(JDBCRelation(_, parts, _), _) =>
+      case LogicalRelationWithTable(JDBCRelation(_, parts, _, _), _) =>
         val whereClauses = parts.map(_.asInstanceOf[JDBCPartition].whereClause).toSet
         assert(whereClauses === Set(
           """"T" < '2018-07-15 20:50:32.5' or "T" is null""",
@@ -1956,17 +1939,20 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
     assert(getRowCount(df2) < df3.count())
   }
 
-  test("SPARK-26383 throw IllegalArgumentException if wrong kind of driver to the given url") {
-    val e = intercept[IllegalArgumentException] {
-      val opts = Map(
-        "url" -> "jdbc:mysql://localhost/db",
-        "dbtable" -> "table",
-        "driver" -> "org.postgresql.Driver"
-      )
-      spark.read.format("jdbc").options(opts).load()
-    }.getMessage
-    assert(e.contains("The driver could not open a JDBC connection. " +
-      "Check the URL: jdbc:mysql://localhost/db"))
+  test("SPARK-26383 throw FAILED_JDBC.CONNECTION if wrong kind of driver to the given url") {
+    val url = "jdbc:mysql://localhost/db"
+    checkError(
+      exception = intercept[AnalysisException] {
+        val opts = Map(
+          "url" -> url,
+          "dbtable" -> "table",
+          "driver" -> "org.postgresql.Driver"
+        )
+        spark.read.format("jdbc").options(opts).load()
+      },
+      condition = "FAILED_JDBC.CONNECTION",
+      parameters = Map("url" -> url)
+    )
   }
 
   test("support casting patterns for lower/upper bounds of TimestampType") {
@@ -2290,4 +2276,33 @@ class JDBCSuite extends QueryTest with SharedSparkSession {
         parameters = Map("jdbcDialect" -> dialect.getClass.getSimpleName))
     }
   }
+
+  test("FAILED_JDBC.CONNECTION") {
+    val testUrls = Seq(
+      "jdbc:mysql",
+      "jdbc:postgresql",
+      "jdbc:sqlserver",
+      "jdbc:db2",
+      "jdbc:h2",
+      "jdbc:teradata",
+      "jdbc:databricks"
+    )
+
+    testUrls.foreach { connectionUrl =>
+      val url = s"$connectionUrl://invalid_url/"
+      val options = new JDBCOptions(Map(
+        "url" -> url,
+        "dbtable" -> "invalid_table"
+      ))
+      checkError(
+        exception = intercept[AnalysisException] {
+          JdbcUtils.withConnection(options) { conn =>
+            conn.getMetaData
+          }
+        },
+        condition = "FAILED_JDBC.CONNECTION",
+        parameters = Map("url" -> url)
+      )
+    }
+  }
 }
