@@ -27,22 +27,25 @@ import scala.reflect.ClassTag
 import scala.util.control.ControlThrowable
 
 import com.google.common.util.concurrent.UncheckedExecutionException
-import org.apache.commons.io.FileUtils
 import org.apache.hadoop.conf.Configuration
 import org.scalatest.time.SpanSugar._
 
 import org.apache.spark.{SparkConf, SparkContext, TaskContext, TestUtils}
 import org.apache.spark.scheduler.{SparkListener, SparkListenerJobStart}
-import org.apache.spark.sql._
-import org.apache.spark.sql.catalyst.plans.logical.{Range, RepartitionByExpression}
+import org.apache.spark.sql.{AnalysisException, Encoders, Row, SQLContext, TestStrategy}
+import org.apache.spark.sql.catalyst.plans.logical.Range
 import org.apache.spark.sql.catalyst.streaming.{InternalOutputModes, StreamingRelationV2}
 import org.apache.spark.sql.catalyst.types.DataTypeUtils.toAttributes
 import org.apache.spark.sql.catalyst.util.DateTimeUtils
+import org.apache.spark.sql.classic.{DataFrame, Dataset}
+import org.apache.spark.sql.classic.ClassicConversions._
 import org.apache.spark.sql.execution.{LocalLimitExec, SimpleMode, SparkPlan}
 import org.apache.spark.sql.execution.command.ExplainCommand
 import org.apache.spark.sql.execution.streaming._
+import org.apache.spark.sql.execution.streaming.operators.stateful.StreamingLocalLimitExec
+import org.apache.spark.sql.execution.streaming.runtime.{LongOffset, MemoryStream, StreamExecution, StreamingExecutionRelation, StreamingQueryWrapper, StreamingRelation}
 import org.apache.spark.sql.execution.streaming.sources.{ContinuousMemoryStream, ForeachBatchUserFuncException, MemorySink}
-import org.apache.spark.sql.execution.streaming.state.{KeyStateEncoderSpec, StateStore, StateStoreConf, StateStoreId, StateStoreProvider}
+import org.apache.spark.sql.execution.streaming.state.{KeyStateEncoderSpec, StateSchemaProvider, StateStore, StateStoreConf, StateStoreId, StateStoreProvider}
 import org.apache.spark.sql.expressions.Window
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
@@ -113,7 +116,7 @@ class StreamSuite extends StreamTest {
     val memoryStream = MemoryStream[Int]
     val executionRelation = StreamingExecutionRelation(
       memoryStream, toAttributes(memoryStream.encoder.schema), None)(
-      memoryStream.sqlContext.sparkSession)
+      memoryStream.sparkSession)
     assert(executionRelation.computeStats().sizeInBytes ==
       spark.sessionState.conf.defaultSizeInBytes)
   }
@@ -765,7 +768,7 @@ class StreamSuite extends StreamTest {
     val dir1 = Utils.createTempDir().getCanonicalFile // not using withTempDir {}, makes test flaky
     // Copy the checkpoint to a temp dir to prevent changes to the original.
     // Not doing this will lead to the test passing on the first run, but fail subsequent runs.
-    FileUtils.copyDirectory(checkpointDir, dir1)
+    Utils.copyDirectory(checkpointDir, dir1)
     // Checkpoint data was generated by a query with 10 shuffle partitions.
     // In order to test reading from the checkpoint, the checkpoint must have two or more batches,
     // since the last batch may be rerun.
@@ -791,7 +794,7 @@ class StreamSuite extends StreamTest {
     // 2 - Check recovery with wrong num shuffle partitions
     prepareMemoryStream()
     val dir2 = Utils.createTempDir().getCanonicalFile
-    FileUtils.copyDirectory(checkpointDir, dir2)
+    Utils.copyDirectory(checkpointDir, dir2)
     // Since the number of partitions is greater than 10, should throw exception.
     withSQLConf(SQLConf.SHUFFLE_PARTITIONS.key -> "15") {
       var streamingQuery: StreamingQuery = null
@@ -1113,7 +1116,7 @@ class StreamSuite extends StreamTest {
       )
       require(execPlan != null)
 
-      val localLimits = execPlan.collect {
+      val localLimits = StreamingQueryPlanTraverseHelper.collectFromUnfoldedPlan(execPlan) {
         case l: LocalLimitExec => l
         case l: StreamingLocalLimitExec => l
       }
@@ -1296,34 +1299,60 @@ class StreamSuite extends StreamTest {
     }
   }
 
-  test("SPARK-34482: correct active SparkSession for logicalPlan") {
-    withSQLConf(
-      SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "true",
-      SQLConf.COALESCE_PARTITIONS_INITIAL_PARTITION_NUM.key -> "10") {
-      val df = spark.readStream.format(classOf[FakeDefaultSource].getName).load()
-      var query: StreamExecution = null
-      try {
-        query =
-          df.repartition($"a")
-            .writeStream
-            .format("memory")
-            .queryName("memory")
-            .start()
-            .asInstanceOf[StreamingQueryWrapper]
-            .streamingQuery
-        query.awaitInitialization(streamingTimeout.toMillis)
-        val plan = query.logicalPlan
-        val numPartition = plan
-          .find { _.isInstanceOf[RepartitionByExpression] }
-          .map(_.asInstanceOf[RepartitionByExpression].numPartitions)
-        // Before the fix of SPARK-34482, the numPartition is the value of
-        // `COALESCE_PARTITIONS_INITIAL_PARTITION_NUM`.
-        assert(numPartition.get === spark.sessionState.conf.getConf(SQLConf.SHUFFLE_PARTITIONS))
-      } finally {
-        if (query != null) {
-          query.stop()
-        }
-      }
+  test("isInterruptionException should correctly unwrap classic py4j InterruptedException") {
+    val e1 = new py4j.Py4JException(
+      """
+        |py4j.protocol.Py4JJavaError: An error occurred while calling o1073599.sql.
+        |: java.util.concurrent.ExecutionException: java.lang.InterruptedException
+        |""".stripMargin)
+    val febError1 = ForeachBatchUserFuncException(e1)
+    assert(StreamExecution.isInterruptionException(febError1, spark.sparkContext))
+
+    // scalastyle:off line.size.limit
+    val e2 = new py4j.Py4JException(
+      """
+        |py4j.protocol.Py4JJavaError: An error occurred while calling o2141502.saveAsTable.
+        |: org.apache.spark.SparkException: Job aborted.
+        |at org.apache.spark.sql.errors.QueryExecutionErrors$.jobAbortedError(QueryExecutionErrors.scala:882)
+        |at org.apache.spark.sql.execution.datasources.FileFormatWriter$.$anonfun$write$1(FileFormatWriter.scala:334)
+        |<REDACTED STACK TRACE>
+        |org.apache.spark.sql.execution.streaming.StreamExecution.withAttributionTags(StreamExecution.scala:82)
+        |at org.apache.spark.sql.execution.streaming.StreamExecution.org$apache$spark$sql$execution$streaming$StreamExecution$$runStream(StreamExecution.scala:339)
+        |at org.apache.spark.sql.execution.streaming.StreamExecution$$anon$1.$anonfun$run$2(StreamExecution.scala:262)
+        |at scala.runtime.java8.JFunction0$mcV$sp.apply(JFunction0$mcV$sp.java:23)
+        |at org.apache.spark.sql.execution.streaming.StreamExecution$$anon$1.run(StreamExecution.scala:262)
+        |*Caused by: java.lang.InterruptedException
+        |at java.util.concurrent.locks.AbstractQueuedSynchronizer.doAcquireSharedInterruptibly(AbstractQueuedSynchronizer.java:1000)*
+        |""".stripMargin)
+    // scalastyle:on line.size.limit
+    val febError2 = ForeachBatchUserFuncException(e2)
+    assert(StreamExecution.isInterruptionException(febError2, spark.sparkContext))
+
+    // scalastyle:off line.size.limit
+    val e3 = new py4j.Py4JException(
+      """
+        py4j.protocol.Py4JJavaError: An error occurred while calling o6032.save.
+        |: org.apache.spark.SparkException: [SPARK_JOB_CANCELLED] Job 89 cancelled part of cancelled job group a5bc7e26-f199-463e-95ad-4ec960a62d79 SQLSTATE: XXKDA
+        |at org.apache.spark.scheduler.DAGScheduler.handleJobCancellation(DAGScheduler.scala:3747)
+        |at org.apache.spark.scheduler.DAGScheduler.$anonfun$handleJobGroupCancelled$4(DAGScheduler.scala:1592)
+        |at scala.runtime.java8.JFunction1$mcVI$sp.apply(JFunction1$mcVI$sp.java:23)
+        |at scala.collection.mutable.HashSet.foreach(HashSet.scala:79)
+        |at org.apache.spark.scheduler.DAGScheduler.handleJobGroupCancelled(DAGScheduler.scala:1591)
+        |at org.apache.spark.scheduler.DAGSchedulerEventProcessLoop.doOnReceive(DAGScheduler.scala:4102)
+        |at org.apache.spark.scheduler.DAGSchedulerEventProcessLoop.onReceive(DAGScheduler.scala:4058)
+        |at org.apache.spark.scheduler.DAGSchedulerEventProcessLoop.onReceive(DAGScheduler.scala:4046)
+        |at org.apache.spark.util.EventLoop$$anon$1.run(EventLoop.scala:54)
+        |at org.apache.spark.scheduler.DAGScheduler.$anonfun$runJob$1(DAGScheduler.scala:1348)
+        |""".stripMargin)
+    // scalastyle:on line.size.limit
+    val febError3 = ForeachBatchUserFuncException(e3)
+    val prevJobId = spark.sparkContext.getLocalProperty("spark.jobGroup.id")
+    try {
+      spark.sparkContext.setLocalProperty(
+        "spark.jobGroup.id", "a5bc7e26-f199-463e-95ad-4ec960a62d79")
+      assert(StreamExecution.isInterruptionException(febError3, spark.sparkContext))
+    } finally {
+      spark.sparkContext.setLocalProperty("spark.jobGroup.id", prevJobId)
     }
   }
 }
@@ -1422,7 +1451,8 @@ class TestStateStoreProvider extends StateStoreProvider {
       useColumnFamilies: Boolean,
       storeConfs: StateStoreConf,
       hadoopConf: Configuration,
-      useMultipleValuesPerKey: Boolean = false): Unit = {
+      useMultipleValuesPerKey: Boolean = false,
+      stateSchemaProvider: Option[StateSchemaProvider] = None): Unit = {
     throw new Exception("Successfully instantiated")
   }
 
@@ -1430,7 +1460,11 @@ class TestStateStoreProvider extends StateStoreProvider {
 
   override def close(): Unit = { }
 
-  override def getStore(version: Long): StateStore = null
+  override def getStore(
+    version: Long,
+    stateStoreCkptId: Option[String] = None,
+    forceSnapshotOnCommit: Boolean = false,
+    loadEmpty: Boolean = false): StateStore = null
 }
 
 /** A fake source that throws `ThrowingExceptionInCreateSource.exception` in `createSource` */
